﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Shared;
using System.Threading;

using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace ListedMotion
{
    public partial class ListedMotion : Form
    {
        private int AXIS = 0; // 본 예제에서 제어하는 대상 축
        private int LM_MAP_IDX = 0;
        private string g_sSeq = "";
        
        public ListedMotion()
        {
            InitializeComponent();
        }

        // common function
        void InitMotionDevices()
        {
            ////////////////////////////////////////////////////////////////////////////////////////
            // [CAUTION]: 아래와 다른 CME파일을 사용하는 경우에는 아래 코드를 수정하여 올바른 CME 파일의
            // 경로를 지정하십시오.
            const string CME_FILE_NAME = "Default.cme2";

            string szCmeFilePath, szSystemDir;

            szSystemDir = Environment.SystemDirectory; // 윈도우즈 시스템 디렉토리명 얻어오기 (일반적으로는 C:\Windows\System32)
            szCmeFilePath = szSystemDir + "\\" + CME_FILE_NAME;

            if (CMDLL.cmmGnInitFromFile(szCmeFilePath) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(IntPtr.Zero); // 에러 발생하였으면 에러 디스플레이.
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////////////
                // CME 파일에서 정의한 축수와 실제 장착되어 있는 모션 축수를 비교확인 하고자할 때는 
                // 아래와 같이 하면 됩니다.

                int nNumAxes_installed = 0;
                int nNumAxes_defined = 0;

                CMDLL.cmmAdvGetNumAvailAxes(ref nNumAxes_installed); // 현재 실제로 PC에 장착되어 있는 모션축 수 (cmmGnDeviceLoad() 함수에서 반환하는 축수와 동일)
                CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes_defined); // CME 파일에서 정의한 축 수(CME 파일이 지정되지 않는 경우에는 현재 장착된 축 수와 일치합니다.)

                if (nNumAxes_installed != nNumAxes_defined)
                {
                    MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }

            // SERVO-ON //
            int nNumAxes = 0;
            CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes);
            for (int i = 0; i < nNumAxes; i++)
            {
                CMDLL.cmmGnSetServoOn(i, (int)Defines._TCmBool.cmTRUE);
            }
        }

        private void ListedMotion_Load(object sender, EventArgs e)
        {
            // Load Motion & DIO devices //
            int nNumAxes=0;
            if (CMDLL.cmmGnDeviceLoad((int)Defines._TCmBool.cmTRUE, ref nNumAxes) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle); // 에러 발생하였으면 에러 디스플레이합니다.
                return;
            }
            else
            {
                InitMotionDevices();

                // Set timer to display current command position //
                this.timer1.Start();
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            // Get current command position of specified axis //
            string sPos;
            double fCurPos = 0;
            // Command position //
            CMDLL.cmmStGetPosition(AXIS, (int)Defines._TCmCntr.cmCNT_COMM, ref fCurPos);
            sPos = fCurPos.ToString();
            this.lblXPos.Text = sPos;
            this.lblSeqIdx.Text = g_sSeq;
        }

        /***********************************************************************
        * OnBtnStart() : "Start" Button's event handler
        * 여기서는 아래 그림과 같이 3-STEP 으로 가속했다가 3-STEP으로 감속하는 
        * 단축 이송 작업을 수행합니다. 각 스텝사이에 지연시간이 없어야지만 
        * 속도의 연속성을 확보할 수 있으므로 리스트 모션을 사용합니다.
        *              _____
        *         ____/  3  \____
        *    ____/ 2          4  \____
        *   / 1                    5  \
        ***********************************************************************/
        private void btnStart_Click(object sender, EventArgs e)
        {
            this.btnStart.Enabled = false;

            /////////////////////////////////////////////////////////////////////
            // List Motion 축 맵핑: 리스트 모션은 동일 장치내에 있는 축들로만
            // 구성되어져야 한다.
            if (CMDLL.cmmLmMapAxes(LM_MAP_IDX, 0x3, 0x0) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle);
                return;
            }

            double fVel = 10000, fVini = 0;
            CMDLL.cmmLmBeginList(LM_MAP_IDX); // List 에 등록 시작

            CMDLL.cmmSxOptSetIniSpeed(AXIS, 0.0); // 1st-step 초기 속도는 0으로 부터 시작하도록...
            CMDLL.cmmCfgSetSpeedPattern(AXIS, (int)Defines._TCmSpeedMode.cmSMODE_S, fVel, 50000, 50000);

            CMDLL.cmmSxSetSpeedRatio(AXIS, (int)Defines._TCmSpeedMode.cmSMODE_KEEP, 50, 100, 0); // 감속구간을 없애기 위해서 감속도를 0으로 설정
            CMDLL.cmmSxMove(AXIS, 10000, (int)Defines._TCmBool.cmTRUE); // 이때는 IsBlocking 파라미터가 TRUE이든 FALSE이든 영향이 없다.

            // 2nd-step은 1st-step의 작업속도에서부터 가속을 시작하도록 초기속도를 1st-step의 작업속도와 같은 값으로 설정한다.
            fVini = fVel * 0.5;
            CMDLL.cmmSxOptSetIniSpeed(AXIS, fVini);
            CMDLL.cmmSxSetSpeedRatio(AXIS, (int)Defines._TCmSpeedMode.cmSMODE_KEEP, 70, 100, 0); // 감속구간을 없애기 위해서 감속도를 0으로 설정
            CMDLL.cmmSxMove(AXIS, 10000, (int)Defines._TCmBool.cmTRUE);

            // 3rd-step은 4th-step의 작업속도에서 감속이 끝나도록 초기속도를 4th-step의 작업속도와 같은 값으로 설정한다.
            fVini = fVel * 0.7;
            CMDLL.cmmSxOptSetIniSpeed(AXIS, fVini);
            CMDLL.cmmSxSetSpeedRatio(AXIS, (int)Defines._TCmSpeedMode.cmSMODE_KEEP, 100, 100, 100);
            CMDLL.cmmSxMove(AXIS, 20000, (int)Defines._TCmBool.cmTRUE);

            // 4th-step은 5th-step의 작업속도에서 감속이 끝나도록 초기속도를 5th-step의 작업속도와 같은 값으로 설정한다.
            fVini = fVel * 0.5;
            CMDLL.cmmSxOptSetIniSpeed(AXIS, fVini);
            CMDLL.cmmSxSetSpeedRatio(AXIS, (int)Defines._TCmSpeedMode.cmSMODE_KEEP, 70, 0, 100); // 가속비를 0으로 하여 가속없이 작업속도에서 바로 출발하도록 한다.
            CMDLL.cmmSxMove(AXIS, -10000, (int)Defines._TCmBool.cmTRUE);

            // 5th-step은 초기속도를 0속도까지 감속하도록 0으로 설정한다. 
            fVini = 0;
            CMDLL.cmmSxOptSetIniSpeed(AXIS, fVini);
            CMDLL.cmmSxSetSpeedRatio(AXIS, (int)Defines._TCmSpeedMode.cmSMODE_KEEP, 50, 0, 100); // 가속비를 0으로 하여 가속없이 작업속도에서 바로 출발하도록 한다.
            CMDLL.cmmSxMove(AXIS, -10000, (int)Defines._TCmBool.cmTRUE);

            CMDLL.cmmLmEndList(LM_MAP_IDX); // 리스트 모션 등록을 마친다.

            CMDLL.cmmLmStartMotion(LM_MAP_IDX); // 등록된 리스트 모션을 실행한다(실제 이송은 여기에서부터 시작한다). 

            // Thread Homming Start
            new Thread(new ThreadStart(ThShowSeqWait)).Start();

            this.btnStart.Enabled = true;
        }

        void ThShowSeqWait()
        {
            // 리스트모션이 완료될 때까지 대기한다. //
            String sSeqTextNew, sSeqTextOld;
            int nIsDone = 0;
            int nSeqCount = 0;

            while (nIsDone == 0)
            {
                sSeqTextNew = "";
                sSeqTextOld = "";

                if (CMDLL.cmmLmIsDone(LM_MAP_IDX, ref nIsDone) == Defines.cmERR_NONE && nIsDone == 0)
                {
                    int nSeqIdx = 0;
                    CMDLL.cmmLmCurSequence(LM_MAP_IDX, ref nSeqIdx); // 현재 수행되고 있는 시퀀스 번호를 표시하기 위해서...

                    nSeqCount = nSeqIdx + 1;
                    sSeqTextNew = "SEQ #" + nSeqCount.ToString();
                }
                else
                {
                    sSeqTextNew = "Not active";
                }

                // 화면 깜밖거림을 방지하기 위해서 텍스트가 변경되었을때만 화면 업데이트 //
                sSeqTextOld = this.lblSeqIdx.Text;
                if (sSeqTextOld != sSeqTextNew)
                {
                    g_sSeq = sSeqTextNew;
                }
            }
        }

        /***********************************************************************
        * OnBtnStop() : "Stop" Button's event handler
        * : This button stops motion.
        ***********************************************************************/
        private void btnStop_Click(object sender, EventArgs e)
        {
            CMDLL.cmmLmAbortMotion(LM_MAP_IDX);
        }

        private void ListedMotion_FormClosing(object sender, FormClosingEventArgs e)
        {
            CMDLL.cmmLmAbortMotion(LM_MAP_IDX);
            CMDLL.cmmSxStop(AXIS, (int)Defines._TCmBool.cmFALSE, (int)Defines._TCmBool.cmFALSE);

            // 사용중인 타이머 종료.
            timer1.Stop();

            CMDLL.cmmGnDeviceUnload(); // Unload Device
        }
    }
}
