﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Shared;
using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace Homming
{
    public partial class HomeSetupForm : Form
    {
        private static int nAxis = 0;
        private static int nHomeMode = 0; // Home Mode
        private static int nHomeClearMode = 0; // Home Clear Mode
        private static int nOrgLogic = 0; // ORG Input Logic
        private static int nEzLogic = 0; // EZ Input Logic
        private static int nEzCount = 0; // EZ Count
        private static double fEscDist = 0; // Escape Distance
        private static double fOffDist = 0; // Offset Distance
        private static int nErrOut = 0; // Output ERC signal at completion
        private static int nSpeedMode = (int)Defines.SPDMODE.MODE_SCURVE; // Speed Mode
        private static double fWorkSpeed = 0; // Work Speed
        private static double fRvSpeed = 0; // Reverse Speed
        private static double fAcceleration = 0; // Acceleration
        private static double fDeceleration = 0; // Deceleration

        public HomeSetupForm(int p_nAxis)
        {
            nAxis = p_nAxis;
            
            InitializeComponent();
        }

        private void HomeSetupForm_Load(object sender, EventArgs e)
        {
            // 화면에 초기값을 설정하기 위해서 현재 설정되어 있는 속도 패턴을 읽어들인다.
            CMDLL.cmmHomeGetConfig(nAxis, ref nHomeMode, ref nEzCount, ref fEscDist, ref fOffDist);
            CMDLL.cmmHomeGetPosClrMode(nAxis, ref nHomeClearMode);
            CMDLL.cmmCfgGetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmORG_LOGIC, ref nOrgLogic);
            CMDLL.cmmCfgGetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmEZ_LOGIC, ref nEzLogic);
            CMDLL.cmmCfgGetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmERC_OUT, ref nErrOut);
            CMDLL.cmmHomeGetSpeedPattern(nAxis, ref nSpeedMode, ref fWorkSpeed, ref fAcceleration, ref fDeceleration, ref fRvSpeed);

            this.cmbHomeMode.SelectedIndex = nHomeMode;
            this.cmbHomeClrMode.SelectedIndex = nHomeClearMode;
            this.cmbOrgLogic.SelectedIndex = nOrgLogic;
            this.cmbEzLogic.SelectedIndex = nEzLogic;
            this.txtEzCount.Text = nEzCount.ToString();
            this.txtEscDist.Text = fEscDist.ToString();
            this.txtOffset.Text = fOffDist.ToString();

            if (nErrOut > 0)
            {
                this.chkErcOut.Checked = true;
            }
            else
            {
                this.chkErcOut.Checked = false;
            }

            this.cmbSpeedMode.SelectedIndex = nSpeedMode;
            this.txtWorkSpeed.Text = fWorkSpeed.ToString();
            this.txtRvsSpeed.Text = fRvSpeed.ToString();
            this.txtAcc.Text = fAcceleration.ToString();
            this.txtDec.Text = fDeceleration.ToString();
        }

        private void cmbHomeMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            nHomeMode = this.cmbHomeMode.SelectedIndex;
        }

        private void cmbHomeClrMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            nHomeClearMode = this.cmbHomeClrMode.SelectedIndex;
        }

        private void cmbOrgLogic_SelectedIndexChanged(object sender, EventArgs e)
        {
            nOrgLogic = this.cmbOrgLogic.SelectedIndex;
        }

        private void cmbEzLogic_SelectedIndexChanged(object sender, EventArgs e)
        {
            nEzLogic = this.cmbEzLogic.SelectedIndex;
        }

        private void cmbSpeedMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            nSpeedMode = this.cmbSpeedMode.SelectedIndex;
        }

        private void chkErcOut_Click(object sender, EventArgs e)
        {
            if(this.chkErcOut.Checked)
            {
                nErrOut = 1;
            }
        }

        private void btnApply_Click(object sender, EventArgs e)
        {
            nEzCount = int.Parse(this.txtEzCount.Text);
            fEscDist = Convert.ToDouble(this.txtEscDist.Text);
            fOffDist = Convert.ToDouble(this.txtOffset.Text);

            fWorkSpeed = Convert.ToDouble(this.txtWorkSpeed.Text);
            fRvSpeed = Convert.ToDouble(this.txtRvsSpeed.Text);
            fAcceleration = Convert.ToDouble(this.txtAcc.Text);
            fDeceleration = Convert.ToDouble(this.txtDec.Text);

            CMDLL.cmmHomeSetConfig(nAxis, nHomeMode, nEzCount, fEscDist, fOffDist);
            CMDLL.cmmHomeSetPosClrMode(nAxis, nHomeClearMode);
            CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmORG_LOGIC, nOrgLogic);
            CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmEZ_LOGIC, nEzLogic);
            CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmERC_OUT, nErrOut);
            CMDLL.cmmHomeSetSpeedPattern(nAxis, nSpeedMode, fWorkSpeed, fAcceleration, fDeceleration, fRvSpeed);
        }

        private void btnQuit_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}
