﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Shared;
using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace Homming
{
    public partial class Homming : Form
    {
        HomeSetupForm m_frmHomeSetup = null;
        
        public Homming()
        {
            InitializeComponent();
        }

        // common function
        void InitMotionDevices()
        {
            ////////////////////////////////////////////////////////////////////////////////////////
            // [CAUTION]: 아래와 다른 CME파일을 사용하는 경우에는 아래 코드를 수정하여 올바른 CME 파일의
            // 경로를 지정하십시오.
            const string CME_FILE_NAME = "Default.cme2";

            string szCmeFilePath, szSystemDir;

            szSystemDir = Environment.SystemDirectory; // 윈도우즈 시스템 디렉토리명 얻어오기 (일반적으로는 C:\Windows\System32)
            szCmeFilePath = "Default.cme2";//szSystemDir + "/" + CME_FILE_NAME;
            //int errorcode = 0;
            //errorcode = CMDLL.cmmGnInitFromFile(szCmeFilePath);
            if (CMDLL.cmmGnInitFromFile(szCmeFilePath) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(IntPtr.Zero); // 에러 발생하였으면 에러 디스플레이.
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////////////
                // CME 파일에서 정의한 축수와 실제 장착되어 있는 모션 축수를 비교확인 하고자할 때는 
                // 아래와 같이 하면 됩니다.

                int anNumAxes_installed = 0;
                int anNumAxes_defined = 0;

                CMDLL.cmmAdvGetNumAvailAxes(ref anNumAxes_installed); // 현재 실제로 PC에 장착되어 있는 모션축 수 (cmmGnDeviceLoad() 함수에서 반환하는 축수와 동일)
                CMDLL.cmmAdvGetNumDefinedAxes(ref anNumAxes_defined); // CME 파일에서 정의한 축 수(CME 파일이 지정되지 않는 경우에는 현재 장착된 축 수와 일치합니다.)

                if (anNumAxes_installed != anNumAxes_defined)
                {
                    MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }

            // SERVO-ON //
            int anNumAxes = 0;
            CMDLL.cmmAdvGetNumDefinedAxes(ref anNumAxes);
            for (int i = 0; i < anNumAxes; i++)
            {
                CMDLL.cmmGnSetServoOn(i, (int)Defines._TCmBool.cmTRUE);
            }
        }

        private void Homming_Load(object sender, EventArgs e)
        {
            // Load Motion & DIO devices //
            int nNumAxes = 0;
            if (CMDLL.cmmGnDeviceLoad((int)Defines._TCmBool.cmTRUE, ref nNumAxes) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle); // 에러 발생하였으면 에러 디스플레이합니다.
                return;
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////
                // 로드된 축수가 올바른지 체크: 본 예제에서는 원하는 축 수를 모르기 때문에 
                // 주석처리 해놓았습니다.

                //				const int NUM_AXES_TO_LOAD = 4; // 이 값을 시스템에서 필요한 축 수에 맞게 조정 
                //				if(anNumAxes[0] < NUM_AXES_TO_LOAD){
                //					MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                //				}

                // Motion 환경 초기화 //
                InitMotionDevices();

                // 축설정 콤보박스에 축리스트를 채워넣습니다. 
                byte[] aszAxisTitle = new byte[Defines.CMM_MAX_STR_LEN_AXIS_TITLE];  // byte => int 변경

                cmbAxisList.Items.Clear(); // Clear combobox
                string szTitle;

                for (int i = 0; i < nNumAxes; i++)
                {
                    CMDLL.cmmAdvGetString(i, (int)Defines._TCmStringID.cmSTR_AXIS_NAME, aszAxisTitle); // CME 빌더에서 정의한 각 축의 타이틀을 가져오는 함수.

                    szTitle = "";
                    for (int j = 0; j < aszAxisTitle.Length; j++)
                    {
                        szTitle += Convert.ToChar(aszAxisTitle[j]);
                    }

                    cmbAxisList.Items.Add(szTitle);
                }

                // 축설정 콤보박스 초기 선택값 지정
                if (cmbAxisList.Items.Count > 0)
                    cmbAxisList.SelectedIndex = 0;

                // Set timer to display current command position //
                this.timer1.Start();
            }
        }

        /***********************************************************************
		* TIMER event handler
		* : Read current command & feedback position and display it
		***********************************************************************/
        private void timer1_Tick(object sender, EventArgs e)
        {
            //------- 모션제어기 위치 모니터링 -------------------------------//
            int nSelAxis = this.cmbAxisList.SelectedIndex;
            string sPos;
            double fCurPos = 0;
            // Command position //
            CMDLL.cmmStGetPosition(nSelAxis, (int)Defines._TCmCntr.cmCNT_COMM, ref fCurPos);
            sPos = fCurPos.ToString();
            this.txtCmdPos.Text = sPos;

            // Feedback position //
            CMDLL.cmmStGetPosition(nSelAxis, (int)Defines._TCmCntr.cmCNT_FEED, ref fCurPos);
            sPos = fCurPos.ToString();
            this.txtFeedPos.Text = sPos;

            //원점복귀 이송 중 홈복귀 중인지 아닌지 확인 한다.
            int nIsBusy = 0;
            CMDLL.cmmHomeIsBusy(nSelAxis, ref nIsBusy);

            if(nIsBusy > 0)
            {
                this.txtHomeIsBusy.Text = "Homming is busy";
            }
            else
            {
                this.txtHomeIsBusy.Text = "Homming is not busy";
            }
        }

        /***********************************************************************
		* Position "Reset" 버튼의 콜백함수. Command & Feedback 
		* position을 0으로 리셋한다.
		***********************************************************************/
        private void btnResetPos_Click(object sender, EventArgs e)
        {
            int nSelAxis = cmbAxisList.SelectedIndex;
            CMDLL.cmmStSetPosition(nSelAxis, (int)Defines._TCmCntr.cmCNT_COMM, 0.0);
            CMDLL.cmmStSetPosition(nSelAxis, (int)Defines._TCmCntr.cmCNT_FEED, 0.0);
        }

        //******************************************************************************************
        // OnHomeConfig(): "Change Home Config" 버튼의 이벤트 핸들러. 이 것은 coding으로 원점복귀
        // 환경을 설정하는 방법을 예시하기 위한 것이다. 원점복귀 환경설정을 코딩으로 수행하는 
        // 방법은 CHomeSetup 클래스를 참조하면 된다. 일반적으로는 원점복귀의 환경설정은 CME 파일에서
        // 설정하므로 코딩으로 원점복귀 환경설정을 꼭 할 필요는 없다.
        //******************************************************************************************
        private void btnHomeConfig_Click(object sender, EventArgs e)
        {
            int nSelAxis = this.cmbAxisList.SelectedIndex;

            if (IsFormExist(typeof(HomeSetupForm)) == null)
            {
                m_frmHomeSetup = new HomeSetupForm(nSelAxis);
            }
            
            m_frmHomeSetup.Show();
            m_frmHomeSetup.Focus();
        }

        private void Homming_FormClosing(object sender, FormClosingEventArgs e)
        {
            // 사용중인 타이머 종료.
            timer1.Stop();

            if (m_frmHomeSetup != null)
            {
                m_frmHomeSetup.Dispose();
                m_frmHomeSetup = null;
            }

            int nSelAxis = this.cmbAxisList.SelectedIndex;
            CMDLL.cmmSxStop(nSelAxis, (int)Defines._TCmBool.cmFALSE, (int)Defines._TCmBool.cmFALSE);
            CMDLL.cmmGnDeviceUnload(); // Unload Device
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            int nSelAxis = this.cmbAxisList.SelectedIndex;

            CMDLL.cmmSxStop(nSelAxis, (int)Defines._TCmBool.cmFALSE, (int)Defines._TCmBool.cmFALSE);
        }

        // Form Check
        private Form IsFormExist(Type FormType)
        {
            foreach (Form OpenForm in Application.OpenForms)
            {
                if (OpenForm.GetType() == FormType)
                    return OpenForm;
            }
            return null;
        }

        private void btnHome_Click(object sender, EventArgs e)
        {
            int nSelAxis = this.cmbAxisList.SelectedIndex;

            //////////////////////////////////////////////////////////////////////////
            // 이전에 원점복귀를 성공적으로 수행했는지 체크하고자할 때에는 cmmHomeGetSuccess()
            // 함수를 사용한다. cmmHomeGetSuccess()에서 반환하는 플래그는 프로그램이 
            // 종료되었다가 다시 실행되어도 계속 유지된다(단, PC가 재부팅되면 리셋된다)
            int dwIsSuccess = 0;
            CMDLL.cmmHomeGetSuccess(nSelAxis, ref dwIsSuccess); // 원점복귀가 성공되었는지 체크 

            if (dwIsSuccess > 0) // 이전에 원점복귀가 이미 수행되었으면...(이것은 프로그램이 종료되었가 다시 실행되었을 때도 유효하다)
            {
                DialogResult result = MessageBox.Show("이전에 원점복귀가 이미 수행되었습니다. 그래도 원점복귀를 다시 수행하겠습니까?", "Confirm", MessageBoxButtons.YesNo);

                if (result == DialogResult.No)
                {
                    return;
                }
            }

            // 원점복귀 수행 //
            int nDirection = 0;
            int nIsBlocking = 0;

            if (this.rdoPlusDir.Checked)
            {
                nDirection = 1;
            }

            if (CMDLL.cmmHomeMove(nSelAxis, nDirection, nIsBlocking) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle);
            }

            //////////////////////////////////////////////////////////////////////////
            // [CAUTION] 위에서 cmmHomeMove() 함수 대신에 cmmHomeMoveStart() 함수를 
            // 사용할 수도 있다. 그런데 cmmHomeMoveStart() 함수를 사용한 경우에는 
            // 아래와 같이 cmmHomeWaitDone() 함수를 이용하여 원점복귀 완료여부를 
            // 체크해주어야 한다.
            // cmmHomeMoveStart(nAxis, m_nDirection);
            // cmmHomeWaitDone(nAxis, FALSE/*Thread에서는 TRUE로 사용할 것*/);

            // 원점복귀가 정상적으로 수행되었는지를 체크하는 것이 바람직하다 //
            CMDLL.cmmHomeGetSuccess(nSelAxis, ref dwIsSuccess); // 원점복귀가 성공되었는지 체크
            if (dwIsSuccess > 0)
            {
                this.txtHomeSuccess.Text = "원점복귀 완료";
            }
            else
            {
                this.txtHomeSuccess.Text = "원점복귀 실패";
            }
        }
    }
}
