﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Shared;

using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace IxLinear
{
    public partial class IxLinear : Form
    {
        private int m_nNumWorkCh = 2;
        private int m_nOpMode = (int)Defines.OPMODE.MODE_VEL;

        private bool m_bIsIxMapValid = false;
        private const int IX_MAP_IDX = 0;  // Interpolation Map Index

        double[] m_afCmdPos;
        double[] m_afFeedPos;

        // properties
        public int SELOPMODE
        {
            get { return m_nOpMode; }
            set { m_nOpMode = value; }
        }
        public int NUMWORKCH
        {
            get { return m_nNumWorkCh; }
            set { m_nNumWorkCh = value; }
        }

        // common function
        void InitMotionDevices()
        {
            ////////////////////////////////////////////////////////////////////////////////////////
            // [CAUTION]: 아래와 다른 CME파일을 사용하는 경우에는 아래 코드를 수정하여 올바른 CME 파일의
            // 경로를 지정하십시오.
            const string CME_FILE_NAME = "Default.cme2";

            string szCmeFilePath, szSystemDir;

            szSystemDir = Environment.SystemDirectory; // 윈도우즈 시스템 디렉토리명 얻어오기 (일반적으로는 C:\Windows\System32)
            szCmeFilePath = szSystemDir + "\\" + CME_FILE_NAME;

            if (CMDLL.cmmGnInitFromFile(szCmeFilePath) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(IntPtr.Zero); // 에러 발생하였으면 에러 디스플레이.
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////////////
                // CME 파일에서 정의한 축수와 실제 장착되어 있는 모션 축수를 비교확인 하고자할 때는 
                // 아래와 같이 하면 됩니다.

                int anNumAxes_installed = 0;
                int anNumAxes_defined =  0;

                CMDLL.cmmAdvGetNumAvailAxes(ref anNumAxes_installed); // 현재 실제로 PC에 장착되어 있는 모션축 수 (cmmGnDeviceLoad() 함수에서 반환하는 축수와 동일)
                CMDLL.cmmAdvGetNumDefinedAxes(ref anNumAxes_defined); // CME 파일에서 정의한 축 수(CME 파일이 지정되지 않는 경우에는 현재 장착된 축 수와 일치합니다.)

                if (anNumAxes_installed != anNumAxes_defined)
                {
                    MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }

            // SERVO-ON //
            int anNumAxes = 0;
            CMDLL.cmmAdvGetNumDefinedAxes(ref anNumAxes);
            for (int i = 0; i < anNumAxes; i++)
            {
                CMDLL.cmmGnSetServoOn(i, (int)Defines._TCmBool.cmTRUE);
            }
        }
        
        public IxLinear()
        {
            InitializeComponent();
        }

        private void IxLinear_Load(object sender, EventArgs e)
        {
            // Load Motion & DIO devices //
            int anNumAxes = 0;
            if (CMDLL.cmmGnDeviceLoad((int)Defines._TCmBool.cmTRUE, ref anNumAxes) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle); // 에러 발생하였으면 에러 디스플레이합니다.
                return;
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////
                // 로드된 축수가 올바른지 체크: 본 예제에서는 원하는 축 수를 모르기 때문에 
                // 주석처리 해놓았습니다.

                //				const int NUM_AXES_TO_LOAD = 4; // 이 값을 시스템에서 필요한 축 수에 맞게 조정 
                //				if(anNumAxes[0] < NUM_AXES_TO_LOAD){
                //					MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                //				}

                // Motion 환경 초기화 //
                InitMotionDevices();

                ////////////////////////////////////////////////////////////////////////////////////
                int i;

                // command 와 feedback postion list 를 저장할 배열 정의.
                m_afCmdPos = new double[NUMWORKCH];
                m_afFeedPos = new double[NUMWORKCH];

                // 사용 가능한 축의 갯수 얻어오기
                int anNumAxesToDisp = 0;
                CMDLL.cmmAdvGetNumAvailAxes(ref anNumAxesToDisp);
                int nNumAxes = anNumAxesToDisp;

                for (i = 0; i < nNumAxes; i++)
                {
                    this.cmbXAxis.Items.Insert(i, "Axis" + i.ToString());
                    this.cmbYAxis.Items.Insert(i, "Axis" + i.ToString());
                }

                // 축선택의 기본값으로는 0번과 1번 두 축이 선택된 것으로 한다.
                this.cmbXAxis.SelectedIndex = 0;
                this.cmbYAxis.SelectedIndex = 1;

                // 초기 속도 모드
                this.cmbMstSpdMode.SelectedIndex = (int)Defines.SPDMODE.MODE_SCURVE;
                this.cmbVecSpdMode.SelectedIndex = (int)Defines.SPDMODE.MODE_SCURVE;

                // 초기 거리값 설정
                this.txtXAxisPos.Text = "10000";
                this.txtYAxisPos.Text = "10000";

                this.txtMstVelSpd.Text = "100";
                this.txtMstAccSpd.Text = "100";
                this.txtMstDecSpd.Text = "100";
                this.txtVecVelSpd.Text = "10000";
                this.txtVecAccSpd.Text = "100000";
                this.txtVecDecSpd.Text = "100000";

                // 초기 속도 패턴 설정
                for (i = 0; i < nNumAxes; i++)
                {
                    CMDLL.cmmCfgSetSpeedPattern(i, (int)Defines.SPDMODE.MODE_SCURVE, 10000, 100000, 100000);
                }

                OnSpeedModeChanged(this.rdoMstSpdMode, null);

                // Set timer to read and display current command position //
                this.timer1.Start();
            }
        }

        private void IxLinear_FormClosing(object sender, FormClosingEventArgs e)
        {
            // 사용중인 타이머 종료.
            timer1.Stop();

            CMDLL.cmmGnDeviceUnload(); // Unload Device
        }

        /***********************************************************************
		* TIMER event handler
		* : display command and feedback position list
		***********************************************************************/
        private void timer1_Tick(object sender, EventArgs e)
        {
            //------- 모션제어기 위치 모니터링 -------------------------------//
            int nAxisX = this.cmbXAxis.SelectedIndex;
            int nAxisY = this.cmbYAxis.SelectedIndex;
            string sPos;
            double afCurPos = 0;

            // Command position (X) //
            CMDLL.cmmStGetPosition(nAxisX, (int)Defines._TCmCntr.cmCNT_COMM, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.txtCmdXAxis.Text = sPos;
            // Feedback position (X) //
            CMDLL.cmmStGetPosition(nAxisX, (int)Defines._TCmCntr.cmCNT_FEED, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.txtFeedXAxis.Text = sPos;
            // Command position (Y) //
            CMDLL.cmmStGetPosition(nAxisY, (int)Defines._TCmCntr.cmCNT_COMM, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.txtCmdYAxis.Text = sPos;
            // Feedback position (Y) //
            CMDLL.cmmStGetPosition(nAxisY, (int)Defines._TCmCntr.cmCNT_FEED, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.txtFeedYAxis.Text = sPos;

            if (this.rdoVecSpdMode.Checked)
            {
                this.txtMstAxisCompare.Text = "Master 축 확인";
            }
            else
            {
                if (Convert.ToDouble(this.txtXAxisPos.Text) >= Convert.ToDouble(this.txtYAxisPos.Text))
                {
                    this.txtMstAxisCompare.Text = "Master 축은 X 축입니다.";
                }
                else
                {
                    this.txtMstAxisCompare.Text = "Master 축은 Y 축입니다.";
                }
            }
        }

        private void OnSpeedModeChanged(object sender, EventArgs e)
        {
            if (sender == rdoMstSpdMode)
            {
                this.cmbMstSpdMode.Enabled = true;
                this.txtMstVelSpd.Enabled = true;
                this.txtMstAccSpd.Enabled = true;
                this.txtMstDecSpd.Enabled = true;

                this.cmbVecSpdMode.Enabled = false;
                this.txtVecVelSpd.Enabled = false;
                this.txtVecAccSpd.Enabled = false;
                this.txtVecDecSpd.Enabled = false;
            }
            else if (sender == rdoVecSpdMode)
            {
                this.cmbMstSpdMode.Enabled = false;
                this.txtMstVelSpd.Enabled = false;
                this.txtMstAccSpd.Enabled = false;
                this.txtMstDecSpd.Enabled = false;

                this.cmbVecSpdMode.Enabled = true;
                this.txtVecVelSpd.Enabled = true;
                this.txtVecAccSpd.Enabled = true;
                this.txtVecDecSpd.Enabled = true;
            }
        }

        /***********************************************************************
		* Position "Reset" 버튼의 콜백함수. Command & Feedback 
		* position을 0으로 리셋한다.
		***********************************************************************/
        private void btnResetPos_Click(object sender, EventArgs e)
        {
            int nAxisX = this.cmbXAxis.SelectedIndex;
            int nAxisY = this.cmbYAxis.SelectedIndex;

            CMDLL.cmmStSetPosition(nAxisX, (int)Defines._TCmCntr.cmCNT_COMM, 0.0); // Reset command position of X-axis
            CMDLL.cmmStSetPosition(nAxisX, (int)Defines._TCmCntr.cmCNT_FEED, 0.0); // Reset feedback position of X-axis
            CMDLL.cmmStSetPosition(nAxisY, (int)Defines._TCmCntr.cmCNT_COMM, 0.0); // Reset command position of X-axis
            CMDLL.cmmStSetPosition(nAxisY, (int)Defines._TCmCntr.cmCNT_FEED, 0.0); // Reset feedback position of X-axis
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            // 보간맵의 축설정이 아직 이루어지지 않은 경우에는 에러를 유발하므로 바로 리턴한다. //
            if (!m_bIsIxMapValid)
                return;

            if (CMDLL.cmmIxStop(IX_MAP_IDX, (int)Defines._TCmBool.cmTRUE, (int)Defines._TCmBool.cmFALSE) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle);
            }

            btnGo.Enabled = true;
        }

        private void btnGo_Click(object sender, EventArgs e)
        {
            btnGo.Enabled = false;

            int nAxisX = this.cmbXAxis.SelectedIndex;
            int nAxisY = this.cmbYAxis.SelectedIndex;

            // X축과 Y축이 같은 축번호이면 에러 치리
            if (nAxisX == nAxisY)
            {
                MessageBox.Show("Error: X and Y axes must be different from each other!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                this.btnGo.Enabled = true;
                return;
            }

            int dwAxesMask1 = 0x0, dwAxesMask2 = 0x0;
            if (nAxisX < 32)
                dwAxesMask1 |= (1 << nAxisX); // X축 번호가 0~31인 경우에는 Mask1의 비트를 셋팅하고...
            else
                dwAxesMask2 |= (1 << (nAxisX - 32)); // X축 번호가 32~63인 경우에는 Mask2의 비트를 셋팅한다...

            if (nAxisY < 32)
                dwAxesMask1 |= (1 << nAxisY); // Y축 번호가 0~31인 경우에는 Mask1의 비트를 셋팅하고...
            else
                dwAxesMask2 |= (1 << (nAxisY - 32)); // Y축 번호가 32~63인 경우에는 Mask2의 비트를 셋팅한다...

            if (CMDLL.cmmIxMapAxes(IX_MAP_IDX, dwAxesMask1, dwAxesMask2) == Defines.cmERR_NONE)
                m_bIsIxMapValid = true;

            // Speed Setting //
            if (this.rdoVecSpdMode.Checked)
            {	// Vector Mode 이면..
                int nVectSpeedMode = this.cmbVecSpdMode.SelectedIndex;
                double fVectVel = Convert.ToDouble(this.txtVecVelSpd.Text);
                double fVectAcc = Convert.ToDouble(this.txtVecAccSpd.Text);
                double fVectDec = Convert.ToDouble(this.txtVecDecSpd.Text);

                CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmTRUE,
                    nVectSpeedMode, fVectVel, fVectAcc, fVectDec);
            }
            else
            {
                int nMastSpeedMode = this.cmbMstSpdMode.SelectedIndex;

                double fMastVel = Convert.ToDouble(this.txtMstDecSpd.Text);
                double fMastAcc = Convert.ToDouble(this.txtMstAccSpd.Text);
                double fMastDec = Convert.ToDouble(this.txtMstDecSpd.Text);

                CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE,
                    nMastSpeedMode, fMastVel, fMastAcc, fMastDec);
            }

            double fPosDataX = Convert.ToDouble(this.txtXAxisPos.Text);
            double fPosDataY = Convert.ToDouble(this.txtYAxisPos.Text);

            double[] afPosList = new double[2];
            afPosList[0] = fPosDataX;
            afPosList[1] = fPosDataY;

            if (this.rdoAbsPosMode.Checked)
            {
                if (CMDLL.cmmIxLineTo(IX_MAP_IDX, afPosList, (int)Defines._TCmBool.cmFALSE/* This must be TRUE in Thread */) != Defines.cmERR_NONE)
                {
                    CMDLL.cmmErrShowLast(this.Handle);
                }
            }
            else
            {
                if (CMDLL.cmmIxLine(IX_MAP_IDX, afPosList, (int)Defines._TCmBool.cmFALSE/* This must be TRUE in Thread */) != Defines.cmERR_NONE)
                {
                    CMDLL.cmmErrShowLast(this.Handle);
                }
            }

            btnGo.Enabled = true;
        }

        private void cmbXAxis_SelectedIndexChanged(object sender, EventArgs e)
        {
            int nAxisX = this.cmbXAxis.SelectedIndex;

            // 사용 가능한 축의 갯수 얻어오기
            int anNumAxesToDisp = 0;
            CMDLL.cmmAdvGetNumAvailAxes(ref anNumAxesToDisp);

            int nAxisXCount = anNumAxesToDisp - 1;

            if (nAxisX == nAxisXCount)
            {
                this.cmbYAxis.SelectedIndex = 0;
            }
            else
            {
                this.cmbYAxis.SelectedIndex = nAxisX + 1;
            }
        }
    }
}
