﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Shared;
using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace MasterSlave
{
    public partial class MasterSlave : Form
    {
        public MasterSlave()
        {
            InitializeComponent();
        }

        // common function
        void InitMotionDevices()
        {
            ////////////////////////////////////////////////////////////////////////////////////////
            // [CAUTION]: 아래와 다른 CME파일을 사용하는 경우에는 아래 코드를 수정하여 올바른 CME 파일의
            // 경로를 지정하십시오.
            const string CME_FILE_NAME = "Default.cme2";

            string szCmeFilePath, szSystemDir;

            szSystemDir = Environment.SystemDirectory; // 윈도우즈 시스템 디렉토리명 얻어오기 (일반적으로는 C:\Windows\System32)
            szCmeFilePath = szSystemDir + "\\" + CME_FILE_NAME;

            if (CMDLL.cmmGnInitFromFile(szCmeFilePath) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(IntPtr.Zero); // 에러 발생하였으면 에러 디스플레이.
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////////////
                // CME 파일에서 정의한 축수와 실제 장착되어 있는 모션 축수를 비교확인 하고자할 때는 
                // 아래와 같이 하면 됩니다.

                int nNumAxes_installed = 0;
                int nNumAxes_defined = 0;

                CMDLL.cmmAdvGetNumAvailAxes(ref nNumAxes_installed); // 현재 실제로 PC에 장착되어 있는 모션축 수 (cmmGnDeviceLoad() 함수에서 반환하는 축수와 동일)
                CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes_defined); // CME 파일에서 정의한 축 수(CME 파일이 지정되지 않는 경우에는 현재 장착된 축 수와 일치합니다.)

                if (nNumAxes_installed != nNumAxes_defined)
                {
                    MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }

            // SERVO-ON //
            int nNumAxes = 0;
            CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes);
            for (int i = 0; i < nNumAxes; i++)
            {
                CMDLL.cmmGnSetServoOn(i, (int)Defines._TCmBool.cmTRUE);
            }
        }

        private void MasterSlave_Load(object sender, EventArgs e)
        {
            // Load Motion & DIO devices //
            int nNumAxes=0;
            if (CMDLL.cmmGnDeviceLoad((int)Defines._TCmBool.cmTRUE, ref nNumAxes) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle); // 에러 발생하였으면 에러 디스플레이합니다.
                return;
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////
                // 로드된 축수가 올바른지 체크: 본 예제에서는 원하는 축 수를 모르기 때문에 
                // 주석처리 해놓았습니다.

                //				const int NUM_AXES_TO_LOAD = 4; // 이 값을 시스템에서 필요한 축 수에 맞게 조정 
                //				if(anNumAxes[0] < NUM_AXES_TO_LOAD){
                //					MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                //				}

                // Motion 환경 초기화 //
                InitMotionDevices();

                ////////////////////////////////////////////////////////////////////////////////////
                // 축설정 콤보박스에 축리스트를 채워넣습니다. 
                byte[] aszAxisTitle = new byte[Defines.CMM_MAX_STR_LEN_AXIS_TITLE];

                this.cmbSlvAxis.Items.Clear(); // Clear combobox
                string szTitle;

                for (int i = 0; i < nNumAxes; i++)
                {
                    CMDLL.cmmAdvGetString(i, (int)Defines._TCmStringID.cmSTR_AXIS_NAME, aszAxisTitle); // CME 빌더에서 정의한 각 축의 타이틀을 가져오는 함수.

                    szTitle = "";
                    for (int j = 0; j < aszAxisTitle.Length; j++)
                    {
                        szTitle += Convert.ToChar(aszAxisTitle[j]);
                    }

                    this.cmbSlvAxis.Items.Add(szTitle);
                }

                if (this.cmbSlvAxis.Items.Count > 0)
                    this.cmbSlvAxis.SelectedIndex = 1;

                // Speed ratio 슬라이더의 값 범위 설정 //
                this.trbSpdRatio.SetRange(1, 100);
                this.trbSpdRatio.Value = 50;
                
                // Set timer to display current command position //
                this.timer1.Start();
            }		
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            int nMasterAxis = 0;
            int nSlaveAxis = this.cmbSlvAxis.SelectedIndex;

            // 본 예제는 슬레이브축을 변경할 수 있도록 해놓았으므로 마스터축도 가변적이다.
            // 따라서 마스터축의 축번호를 cmmMsGetMasterAxis() 함수를 통하여 얻는다.
            // 하자만 실제 사용자가 프로그램할 때는 대부분 마스터축이 몇번축인지를 알고
            // 있으므로 cmmMsGetMasterAxis() 함수를 쓸 필요는 없을 것이다.
            CMDLL.cmmMsGetMasterAxis(nSlaveAxis, ref nMasterAxis);

            // Slave state display //
            int nSlaveState = 0;
            string sTempStr = "";

            CMDLL.cmmMsCheckSlaveState(nSlaveAxis, ref nSlaveState);
            switch (nSlaveState)
            {
                case 0:
                    sTempStr = "Unregistered";
                    this.btnRegSlave.Enabled = true;
                    this.btnUnregSlave.Enabled = false;
                    break;

                case -1: // Error occured
                    sTempStr = "Error occured";
                    this.btnRegSlave.Enabled = true;
                    this.btnUnregSlave.Enabled = true;
                    break;

                case 1: // Registered
                    sTempStr = "Registered";
                    this.btnRegSlave.Enabled = false;
                    this.btnUnregSlave.Enabled = true;
                    break;

                default:
                    sTempStr = "Unknown state";
                    break;
            }
            this.lblSlvState.Text = sTempStr;

            // Master axis name display //
            byte[] szMasterAxisName = new byte[Defines.CMM_MAX_STR_LEN_AXIS_TITLE];
            CMDLL.cmmAdvGetString(nMasterAxis, (int)Defines._TCmStringID.cmSTR_AXIS_NAME, szMasterAxisName);
            string szTitle = "";
            for (int j = 0; j < szMasterAxisName.Length; j++)
            {
                szTitle += Convert.ToChar(szMasterAxisName[j]);
            }
            this.lblMstAxis.Text = szTitle;

            // Position display //
            string sPos;
            double fCurPos = 0;

            CMDLL.cmmStGetPosition(nSlaveAxis, (int)Defines._TCmCntr.cmCNT_COMM, ref fCurPos);
            sPos = fCurPos.ToString();
            this.lblSlvPosC.Text = sPos;

            CMDLL.cmmStGetPosition(nSlaveAxis, (int)Defines._TCmCntr.cmCNT_FEED, ref fCurPos);
            sPos = fCurPos.ToString();
            this.lblSlvPosF.Text = sPos;

            CMDLL.cmmStGetPosition(nMasterAxis, (int)Defines._TCmCntr.cmCNT_COMM, ref fCurPos);
            sPos = fCurPos.ToString();
            this.lblMstPosC.Text = sPos;

            CMDLL.cmmStGetPosition(nMasterAxis, (int)Defines._TCmCntr.cmCNT_FEED, ref fCurPos);
            sPos = fCurPos.ToString();
            this.lblMstPosF.Text = sPos;
        }

        private void trbSpdRatio_Scroll(object sender, EventArgs e)
        {
            this.lblSpeedRatio.Text = trbSpdRatio.Value.ToString() + " (%)";
        }

        private void btnResetPos_Click(object sender, EventArgs e)
        {
            // 본 예제는 슬레이브축을 변경할 수 있도록 해놓았으므로 마스터축도 가변적이다.
            // 따라서 마스터축의 축번호를 cmmMsGetMasterAxis() 함수를 통하여 얻는다.
            // 하자만 실제 사용자가 프로그램할 때는 대부분 마스터축이 몇번축인지를 알고
            // 있으므로 cmmMsGetMasterAxis() 함수를 쓸 필요는 없을 것이다.
            int nMasterAxis = 0;
            int nSlaveAxis = this.cmbSlvAxis.SelectedIndex;
            CMDLL.cmmMsGetMasterAxis(nSlaveAxis, ref nMasterAxis);

            CMDLL.cmmStSetPosition(nSlaveAxis, (int)Defines._TCmCntr.cmCNT_COMM, 0.0);
            CMDLL.cmmStSetPosition(nSlaveAxis, (int)Defines._TCmCntr.cmCNT_FEED, 0.0);
            CMDLL.cmmStSetPosition(nMasterAxis, (int)Defines._TCmCntr.cmCNT_COMM, 0.0);
            CMDLL.cmmStSetPosition(nMasterAxis, (int)Defines._TCmCntr.cmCNT_FEED, 0.0);
        }

        private void btnRegSlave_Click(object sender, EventArgs e)
        {
            int nMasterAxis = 0;
            int nSlaveAxis = this.cmbSlvAxis.SelectedIndex;
            double fSlvMaxSpeed = Convert.ToDouble(this.txtSlvMaxSpeed.Text);
            int nInverseDir = 0;
            
            CMDLL.cmmMsGetMasterAxis(nSlaveAxis, ref nMasterAxis);
            CMDLL.cmmCfgSetOutMode(nMasterAxis, (int)Defines._TCmOutMode.cmOMODE_CWCCW0); // cmOMODE_CWCCW0 또는 cmOMODE_CWCCW1 을 사용하세요.

            if (this.chkSlvInverse.Checked)
            {
                nInverseDir = 1;
            }

            if (CMDLL.cmmMsRegisterSlave(nSlaveAxis, fSlvMaxSpeed, nInverseDir) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle);
            }
        }

        private void btnUnregSlave_Click(object sender, EventArgs e)
        {
            int nSlaveAxis = this.cmbSlvAxis.SelectedIndex;

            if (CMDLL.cmmMsUnregisterSlave(nSlaveAxis) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle);
            }
        }

        private void btnJogN_MouseDown(object sender, MouseEventArgs e)
        {
            // 본 예제는 슬레이브축을 변경할 수 있도록 해놓았으므로 마스터축도 가변적이다.
            // 따라서 마스터축의 축번호를 cmmMsGetMasterAxis() 함수를 통하여 얻는다.
            // 하자만 실제 사용자가 프로그램할 때는 대부분 마스터축이 몇번축인지를 알고
            // 있으므로 cmmMsGetMasterAxis() 함수를 쓸 필요는 없을 것이다.
            int nMasterAxis = 0;
            int nSlaveAxis = this.cmbSlvAxis.SelectedIndex;

            CMDLL.cmmMsGetMasterAxis(nSlaveAxis, ref nMasterAxis);

            // "Velocity move" start //
            double fVwork = Convert.ToDouble(this.trbSpdRatio.Value);
            CMDLL.cmmSxSetSpeedRatio(nMasterAxis, (int)Defines._TCmSpeedMode.cmSMODE_KEEP, fVwork, 100.0, 100.0);
            if (CMDLL.cmmSxVMoveStart(nMasterAxis, (int)Defines._TCmDir.cmDIR_N) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle);
            }
        }

        private void btnJogN_MouseUp(object sender, MouseEventArgs e)
        {
            // 본 예제는 슬레이브축을 변경할 수 있도록 해놓았으므로 마스터축도 가변적이다.
            // 따라서 마스터축의 축번호를 cmmMsGetMasterAxis() 함수를 통하여 얻는다.
            // 하자만 실제 사용자가 프로그램할 때는 대부분 마스터축이 몇번축인지를 알고
            // 있으므로 cmmMsGetMasterAxis() 함수를 쓸 필요는 없을 것이다.
            int nMasterAxis = 0;
            int nSlaveAxis = this.cmbSlvAxis.SelectedIndex;

            CMDLL.cmmMsGetMasterAxis(nSlaveAxis, ref nMasterAxis);

            // Stop Motion //
            CMDLL.cmmSxStop(nMasterAxis, (int)Defines._TCmBool.cmFALSE, (int)Defines._TCmBool.cmFALSE);
        }

        private void btnJogP_MouseDown(object sender, MouseEventArgs e)
        {
            // 본 예제는 슬레이브축을 변경할 수 있도록 해놓았으므로 마스터축도 가변적이다.
            // 따라서 마스터축의 축번호를 cmmMsGetMasterAxis() 함수를 통하여 얻는다.
            // 하자만 실제 사용자가 프로그램할 때는 대부분 마스터축이 몇번축인지를 알고
            // 있으므로 cmmMsGetMasterAxis() 함수를 쓸 필요는 없을 것이다.
            int nMasterAxis = 0;
            int nSlaveAxis = this.cmbSlvAxis.SelectedIndex;

            CMDLL.cmmMsGetMasterAxis(nSlaveAxis, ref nMasterAxis);

            // "Velocity move" start //
            double fVwork = Convert.ToDouble(this.trbSpdRatio.Value);
            CMDLL.cmmSxSetSpeedRatio(nMasterAxis, (int)Defines._TCmSpeedMode.cmSMODE_KEEP, fVwork, 100.0, 100.0);
            if (CMDLL.cmmSxVMoveStart(nMasterAxis, (int)Defines._TCmDir.cmDIR_P) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle);
            }
        }

        private void btnJogP_MouseUp(object sender, MouseEventArgs e)
        {
            // 본 예제는 슬레이브축을 변경할 수 있도록 해놓았으므로 마스터축도 가변적이다.
            // 따라서 마스터축의 축번호를 cmmMsGetMasterAxis() 함수를 통하여 얻는다.
            // 하자만 실제 사용자가 프로그램할 때는 대부분 마스터축이 몇번축인지를 알고
            // 있으므로 cmmMsGetMasterAxis() 함수를 쓸 필요는 없을 것이다.
            int nMasterAxis = 0;
            int nSlaveAxis = this.cmbSlvAxis.SelectedIndex;

            CMDLL.cmmMsGetMasterAxis(nSlaveAxis, ref nMasterAxis);

            // Stop Motion //
            CMDLL.cmmSxStop(nMasterAxis, (int)Defines._TCmBool.cmFALSE, (int)Defines._TCmBool.cmFALSE);
        }

        private void MasterSlave_FormClosing(object sender, FormClosingEventArgs e)
        {
            // 사용중인 타이머 종료.
            timer1.Stop();

            CMDLL.cmmGnDeviceUnload(); // Unload Device
        }
    }
}
