﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Shared;

using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace IxArc
{
    public partial class IxArcForm : Form
    {
        public enum ARC_MODE { ARC_ANGLE, ARC_POINT, ARC_3POINT };
        
        //const int IX_MAP_IDX = 0;
        private const int IX_MAP_IDX = 0;  // Interpolation Map Index
        int m_nArcMotionMode = (int)ARC_MODE.ARC_ANGLE;

        public IxArcForm()
        {
            InitializeComponent();
        }

        // common function
        void InitMotionDevices()
        {
            ////////////////////////////////////////////////////////////////////////////////////////
            // [CAUTION]: 아래와 다른 CME파일을 사용하는 경우에는 아래 코드를 수정하여 올바른 CME 파일의
            // 경로를 지정하십시오.
            const string CME_FILE_NAME = "Default.cme2";

            string szCmeFilePath, szSystemDir;

            szSystemDir = Environment.SystemDirectory; // 윈도우즈 시스템 디렉토리명 얻어오기 (일반적으로는 C:\Windows\System32)
            szCmeFilePath = szSystemDir + "\\" + CME_FILE_NAME;

            if (CMDLL.cmmGnInitFromFile(szCmeFilePath) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(IntPtr.Zero); // 에러 발생하였으면 에러 디스플레이.
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////////////
                // CME 파일에서 정의한 축수와 실제 장착되어 있는 모션 축수를 비교확인 하고자할 때는 
                // 아래와 같이 하면 됩니다.

                int anNumAxes_installed = 0;
                int anNumAxes_defined = 0;

                CMDLL.cmmAdvGetNumAvailAxes(ref anNumAxes_installed); // 현재 실제로 PC에 장착되어 있는 모션축 수 (cmmGnDeviceLoad() 함수에서 반환하는 축수와 동일)
                CMDLL.cmmAdvGetNumDefinedAxes(ref anNumAxes_defined); // CME 파일에서 정의한 축 수(CME 파일이 지정되지 않는 경우에는 현재 장착된 축 수와 일치합니다.)

                if (anNumAxes_installed != anNumAxes_defined)
                {
                    MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }

            // SERVO-ON //
            int anNumAxes = 0;
            CMDLL.cmmAdvGetNumDefinedAxes(ref anNumAxes);
            for (int i = 0; i < anNumAxes; i++)
            {
                CMDLL.cmmGnSetServoOn(i, (int)Defines._TCmBool.cmTRUE);
            }
        }

        private void cmbSpeedMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cmbSpeedMode.SelectedIndex == 0) // Master Speed Mode
            {
                grpSpeedConfigX.Enabled = true;
                grpSpeedConfigY.Enabled = true;
                grpSpeedConfigVector.Enabled = false;
            }
            else if (cmbSpeedMode.SelectedIndex == 1) // Vector Speed Mode
            {
                grpSpeedConfigX.Enabled = false;
                grpSpeedConfigY.Enabled = false;
                grpSpeedConfigVector.Enabled = true;
            }
        }

        private void IxArcForm_Load(object sender, EventArgs e)
        {
            // Load Motion & DIO devices //
            int anNumAxes = 0;
            if (CMDLL.cmmGnDeviceLoad((int)Defines._TCmBool.cmTRUE, ref anNumAxes) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle); // 에러 발생하였으면 에러 디스플레이합니다.
                return;
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////
                // 로드된 축수가 올바른지 체크: 본 예제에서는 원하는 축 수를 모르기 때문에 
                // 주석처리 해놓았습니다.

                //				const int NUM_AXES_TO_LOAD = 4; // 이 값을 시스템에서 필요한 축 수에 맞게 조정 
                //				if(anNumAxes[0] < NUM_AXES_TO_LOAD){
                //					MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                //				}

                // Motion 환경 초기화 //
                InitMotionDevices();

                ////////////////////////////////////////////////////////////////////////////////////
                int i;

                // 사용 가능한 축의 갯수 얻어오기
                int anNumAxesToDisp = 0;
                CMDLL.cmmAdvGetNumAvailAxes(ref anNumAxesToDisp);
                int nNumAxes = anNumAxesToDisp;

                for (i = 0; i < nNumAxes; i++)
                {
                    this.cmbAxisX.Items.Insert(i, "Axis" + i.ToString());
                    this.cmbAxisY.Items.Insert(i, "Axis" + i.ToString());
                }

                // 축선택의 기본값으로는 0번과 1번 두 축이 선택된 것으로 한다.
                this.cmbAxisX.SelectedIndex = 0;
                this.cmbAxisY.SelectedIndex = 1;

                this.cmbMovMode.SelectedIndex = 0;
                this.cmbSpeedPattern.SelectedIndex = (int)Defines.SPDMODE.MODE_SCURVE;
                this.cmbSpeedMode.SelectedIndex = 0;

                // 초기 속도 패턴 설정
                for (i = 0; i < nNumAxes; i++)
                {
                    CMDLL.cmmCfgSetSpeedPattern(i, (int)Defines.SPDMODE.MODE_SCURVE, 10000, 100000, 100000);
                }

                // Set timer to read and display current command position //
                this.timer1.Start();
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            int nAxisX = this.cmbAxisX.SelectedIndex;
            int nAxisY = this.cmbAxisY.SelectedIndex;
            int nCount = 0;

            CMDLL.cmmStGetCount(nAxisX, (int)Defines._TCmCntr.cmCNT_COMM, ref nCount);
            this.txt3PntP1AxisX.Text = nCount.ToString();
            CMDLL.cmmStGetCount(nAxisY, (int)Defines._TCmCntr.cmCNT_COMM, ref nCount);
            this.txt3PntP1AxisY.Text = nCount.ToString();
        }

        private void cmbMovMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            int nMovMode = this.cmbMovMode.SelectedIndex;

            if(nMovMode == 0)
            {
                //Arc Method 창의 라벨 단위 이름 변경 - Offset
                //Arc-Angle의 라벨 단위 이름 변경
                this.lblAngCenAxisX.Text = "Axis-X Center Offset (Pulse)";
                this.lblAngCenAxisY.Text = "Axis-Y Center Offset (Pulse)";

                //Arc-Point의 라벨 단위 이름 변경
                this.lblPntCenAxisX.Text = "Axis-X Center Offset (Pulse)";
                this.lblPntCenAxisY.Text = "Axis-X Center Offset (Pulse)";

                //3-Point의 라벨 단위 이름 변경
                this.lbl3PntP2AxisX.Text = "Axis-X Offset (Pulse)";
                this.lbl3PntP2AxisY.Text = "Axis-Y Offset (Pulse)";
                this.lbl3PntP3AxisX.Text = "Axis-X Offset (Pulse)";
                this.lbl3PntP3AxisY.Text = "Axis-Y Offset (Pulse)";
            }
            else
            {
                //Arc Method 창의 라벨 단위 이름 변경 - Position
                //Arc-Angle의 라벨 단위 이름 변경
                this.lblAngCenAxisX.Text = "Axis-X Center Position (Pulse)";
                this.lblAngCenAxisY.Text = "Axis-Y Center Position (Pulse)";

                //Arc-Point의 라벨 단위 이름 변경
                this.lblPntCenAxisX.Text = "Axis-X Center Position (Pulse)";
                this.lblPntCenAxisY.Text = "Axis-X Center Position (Pulse)";

                //3-Point의 라벨 단위 이름 변경
                this.lbl3PntP2AxisX.Text = "Axis-X Position (Pulse)";
                this.lbl3PntP2AxisY.Text = "Axis-Y Position (Pulse)";
                this.lbl3PntP3AxisX.Text = "Axis-X Position (Pulse)";
                this.lbl3PntP3AxisY.Text = "Axis-Y Position (Pulse)";
            }
        }

        private void cmbAxisX_SelectedIndexChanged(object sender, EventArgs e)
        {
            int nAxisX = this.cmbAxisX.SelectedIndex;

            // 사용 가능한 축의 갯수 얻어오기
            int anNumAxesToDisp = 0;
            CMDLL.cmmAdvGetNumAvailAxes(ref anNumAxesToDisp);

            int nAxisXCount = anNumAxesToDisp - 1;

            if (nAxisX == nAxisXCount)
            {
                this.cmbAxisX.SelectedIndex = 0;
            }
            else
            {
                this.cmbAxisY.SelectedIndex = nAxisX + 1;
            }
        }

        public void CommSetSpeedPattern()
        {
            int nAxisX = this.cmbAxisX.SelectedIndex;
            int nAxisY = this.cmbAxisY.SelectedIndex;
            int nSpeedMode = this.cmbSpeedMode.SelectedIndex;

            double fMaxX = Convert.ToDouble(this.txtMaxX.Text);
            double fInitX = Convert.ToDouble(this.txtInitX.Text);
            double fMaxY = Convert.ToDouble(this.txtMaxY.Text);
            double fInitY = Convert.ToDouble(this.txtInitY.Text);
            double fMaxVec = Convert.ToDouble(this.txtMaxVec.Text);
            
            if (nSpeedMode == 0)
            {
                //Axis X Speed Config
                CMDLL.cmmCfgSetSpeedRange(nAxisX, fMaxX);
                CMDLL.cmmSxOptSetIniSpeed(nAxisX, fInitX);

                //Axis Y Speed Config
                CMDLL.cmmCfgSetSpeedRange(nAxisY, fMaxY);
                CMDLL.cmmSxOptSetIniSpeed(nAxisY, fInitY);
            }
            else
            {
                //Vector Speed Config
                CMDLL.cmmCfgSetSpeedRange(nAxisX, fMaxVec);
                CMDLL.cmmCfgSetSpeedRange(nAxisY, fMaxVec);
            }
        }

        // CCW(-) 방향으로 구동
        private void btnMoveN_Click(object sender, EventArgs e)
        {
            btnMoveN.Enabled = false;
            btnMoveP.Enabled = false;
            
            int nAxisX = this.cmbAxisX.SelectedIndex;
            int nAxisY = this.cmbAxisY.SelectedIndex;

            int nAxesMask1 = 0x0;
            int nAxesMask2 = 0x0;

            if (nAxisX < 32)
            {
                nAxesMask1 |= (1 << nAxisX);
            }
            else
            {
                nAxesMask2 |= ((1 << nAxisX) - 32);
            }
            if (nAxisY < 32)
            {
                nAxesMask1 |= (1 << nAxisY);
            }
            else
            {
                nAxesMask2 |= ((1 << nAxisY) - 32);
            }

            CMDLL.cmmIxMapAxes(IX_MAP_IDX, nAxesMask1, nAxesMask2);

            CommSetSpeedPattern();

            int nMovMode = this.cmbMovMode.SelectedIndex;
            int nSpeedPattern = this.cmbSpeedPattern.SelectedIndex;
            int nSpeedMode = this.cmbSpeedMode.SelectedIndex;

            // Axis-X Speed Config
            double fWorkX = Convert.ToDouble(this.txtWorkX.Text);
            double fAccX = Convert.ToDouble(this.txtAccX.Text);
            double fDecX = Convert.ToDouble(this.txtDecX.Text);

            // Axis-Y Speed Config
            double fWorkY = Convert.ToDouble(this.txtWorkY.Text);
            double fAccY = Convert.ToDouble(this.txtAccY.Text);
            double fDecY = Convert.ToDouble(this.txtDecY.Text);

            // Vector Speed Config
            double fWorkVec = Convert.ToDouble(this.txtWorkVec.Text);
            double fAccVec = Convert.ToDouble(this.txtAccVec.Text);
            double fDecVec = Convert.ToDouble(this.txtDecVec.Text);

            // Arc Method - Arc Angle
            double fAngCenAxisX = Convert.ToDouble(this.txtAngCenAxisX.Text);
            double fAngCenAxisY = Convert.ToDouble(this.txtAngCenAxisY.Text);
            double fAngEndDeg = Convert.ToDouble(this.txtAngEndDeg.Text);

            // Arc Method - Arc Point
            double fPntCenAxisX = Convert.ToDouble(this.txtPntCenAxisX.Text);
            double fPntCenAxisY = Convert.ToDouble(this.txtPntCenAxisY.Text);
            double fPntEndAxisX = Convert.ToDouble(this.txtPntEndAxisX.Text);
            double fPntEndAxisY = Convert.ToDouble(this.txtPntEndAxisY.Text);

            // Arc Method - Arc 3Point
            double f3PntP2AxisX = Convert.ToDouble(this.txt3PntP2AxisX.Text);
            double f3PntP2AxisY = Convert.ToDouble(this.txt3PntP2AxisY.Text);
            double f3PntP3AxisX = Convert.ToDouble(this.txt3PntP3AxisX.Text);
            double f3PntP3AxisY = Convert.ToDouble(this.txt3PntP3AxisY.Text);

            if (nMovMode == 0) // Moving Mode 를 확인
            {
                if(nSpeedMode == 0) // Speed Mode 를 확인
                {
                    switch(m_nArcMotionMode)
                    {
                        case (int)ARC_MODE.ARC_ANGLE :

                            //Relative - Master - Arc_Angle
                            //cmmIxSetSpeedPattern - Master
                            //cmmIxArcAStart
                            //cmmIxWateDone
                            CMDLL.cmmCfgSetSpeedPattern(nAxisX, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmCfgSetSpeedPattern(nAxisY, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE, nSpeedPattern, 100, 100, 100);
                            CMDLL.cmmIxArcAStart(IX_MAP_IDX, -fAngCenAxisX, -fAngCenAxisY, fAngEndDeg);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                        case (int)ARC_MODE.ARC_POINT:

                            //Relative - Master - Arc_Point
                            //cmmIxSetSpeedPattern - Master
                            //cmmIxArcPStart
                            //cmmIxWateDone
                            CMDLL.cmmCfgSetSpeedPattern(nAxisX, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmCfgSetSpeedPattern(nAxisY, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE, nSpeedPattern, 100, 100, 100);
                            CMDLL.cmmIxArcPStart(IX_MAP_IDX, fPntCenAxisX, fPntCenAxisY, fPntEndAxisX, fPntEndAxisY, (int)Defines._TCmArcDir.cmARC_CCW);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                        case (int)ARC_MODE.ARC_3POINT:

                            //Relative - Master - 3_Point
                            //cmmIxSetSpeedPattern - Master
                            //cmmIxArc3PStart
                            //cmmIxWateDone
                            CMDLL.cmmCfgSetSpeedPattern(nAxisX, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmCfgSetSpeedPattern(nAxisY, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE, nSpeedPattern, 100, 100, 100);
                            CMDLL.cmmIxArc3PStart(IX_MAP_IDX, -f3PntP2AxisX, -f3PntP2AxisY, -f3PntP3AxisX, -f3PntP3AxisX, 0);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                    }
                }
                else
                {
                    switch (m_nArcMotionMode)
                    {
                        case (int)ARC_MODE.ARC_ANGLE:

                            //Relative - Master - Arc_Angle
                            //cmmIxSetSpeedPattern - Vector
                            //cmmIxArcAStart
                            //cmmIxWateDone
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmTRUE, nSpeedPattern, fWorkVec, fAccVec, fDecVec);
                            CMDLL.cmmIxArcAStart(IX_MAP_IDX, -fAngCenAxisX, -fAngCenAxisY, fAngEndDeg);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                        case (int)ARC_MODE.ARC_POINT:

                            //Relative - Master - Arc_Point
                            //cmmIxSetSpeedPattern - Vector
                            //cmmIxArcPStart
                            //cmmIxWateDone
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmTRUE, nSpeedPattern, fWorkVec, fAccVec, fDecVec);
                            CMDLL.cmmIxArcPStart(IX_MAP_IDX, fPntCenAxisX, fPntCenAxisY, fPntEndAxisX, fPntEndAxisY, (int)Defines._TCmArcDir.cmARC_CCW);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                        case (int)ARC_MODE.ARC_3POINT:

                            //Relative - Master - 3_Point
                            //cmmIxSetSpeedPattern - Vector
                            //cmmIxArc3PStart
                            //cmmIxWateDone
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmTRUE, nSpeedPattern, fWorkVec, fAccVec, fDecVec);
                            CMDLL.cmmIxArc3PStart(IX_MAP_IDX, -f3PntP2AxisX, -f3PntP2AxisY, -f3PntP3AxisX, -f3PntP3AxisX, 0);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                    }
                }
            }
            else
            {
                if (nSpeedMode == 0) // Speed Mode 를 확인
                {
                    switch (m_nArcMotionMode)
                    {
                        case (int)ARC_MODE.ARC_ANGLE:

                            //Relative - Master - Arc_Angle
                            //cmmIxSetSpeedPattern - Master
                            //cmmIxArcAStart
                            //cmmIxWateDone
                            CMDLL.cmmCfgSetSpeedPattern(nAxisX, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmCfgSetSpeedPattern(nAxisY, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE, nSpeedPattern, 100, 100, 100);
                            CMDLL.cmmIxArcAToStart(IX_MAP_IDX, -fAngCenAxisX, -fAngCenAxisY, fAngEndDeg);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                        case (int)ARC_MODE.ARC_POINT:

                            //Relative - Master - Arc_Point
                            //cmmIxSetSpeedPattern - Master
                            //cmmIxArcPStart
                            //cmmIxWateDone
                            CMDLL.cmmCfgSetSpeedPattern(nAxisX, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmCfgSetSpeedPattern(nAxisY, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE, nSpeedPattern, 100, 100, 100);
                            CMDLL.cmmIxArcPToStart(IX_MAP_IDX, fPntCenAxisX, fPntCenAxisY, fPntEndAxisX, fPntEndAxisY, (int)Defines._TCmArcDir.cmARC_CCW);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                        case (int)ARC_MODE.ARC_3POINT:

                            //Relative - Master - 3_Point
                            //cmmIxSetSpeedPattern - Master
                            //cmmIxArc3PStart
                            //cmmIxWateDone
                            CMDLL.cmmCfgSetSpeedPattern(nAxisX, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmCfgSetSpeedPattern(nAxisY, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE, nSpeedPattern, 100, 100, 100);
                            CMDLL.cmmIxArc3PToStart(IX_MAP_IDX, -f3PntP2AxisX, -f3PntP2AxisY, -f3PntP3AxisX, -f3PntP3AxisX, 0);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                    }
                }
                else
                {
                    switch (m_nArcMotionMode)
                    {
                        case (int)ARC_MODE.ARC_ANGLE:

                            //Relative - Master - Arc_Angle
                            //cmmIxSetSpeedPattern - Vector
                            //cmmIxArcAStart
                            //cmmIxWateDone
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmTRUE, nSpeedPattern, fWorkVec, fAccVec, fDecVec);
                            CMDLL.cmmIxArcAToStart(IX_MAP_IDX, -fAngCenAxisX, -fAngCenAxisY, fAngEndDeg);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                        case (int)ARC_MODE.ARC_POINT:

                            //Relative - Master - Arc_Point
                            //cmmIxSetSpeedPattern - Vector
                            //cmmIxArcPStart
                            //cmmIxWateDone
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmTRUE, nSpeedPattern, fWorkVec, fAccVec, fDecVec);
                            CMDLL.cmmIxArcPToStart(IX_MAP_IDX, fPntCenAxisX, fPntCenAxisY, fPntEndAxisX, fPntEndAxisY, (int)Defines._TCmArcDir.cmARC_CCW);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                        case (int)ARC_MODE.ARC_3POINT:

                            //Relative - Master - 3_Point
                            //cmmIxSetSpeedPattern - Vector
                            //cmmIxArc3PStart
                            //cmmIxWateDone
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmTRUE, nSpeedPattern, fWorkVec, fAccVec, fDecVec);
                            CMDLL.cmmIxArc3PToStart(IX_MAP_IDX, -f3PntP2AxisX, -f3PntP2AxisY, -f3PntP3AxisX, -f3PntP3AxisX, 0);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                    }
                }
            }

            btnMoveN.Enabled = true;
            btnMoveP.Enabled = true;
        }

        private void tabArcMethod_SelectedIndexChanged(object sender, EventArgs e)
        {
            int nTabArcMethodSelect = this.tabArcMethod.SelectedIndex;

            if (nTabArcMethodSelect == 0)
            {
                m_nArcMotionMode = (int)ARC_MODE.ARC_ANGLE;
            }
            else if (nTabArcMethodSelect == 1)
            {
                m_nArcMotionMode = (int)ARC_MODE.ARC_POINT;
            }
            else if (nTabArcMethodSelect == 2)
            {
                m_nArcMotionMode = (int)ARC_MODE.ARC_3POINT;
            }
        }

        // CW(+) 방향으로 구동
        private void btnMoveP_Click(object sender, EventArgs e)
        {
            btnMoveN.Enabled = false;
            btnMoveP.Enabled = false;
            
            int nAxisX = this.cmbAxisX.SelectedIndex;
            int nAxisY = this.cmbAxisY.SelectedIndex;

            int nAxesMask1 = 0x0;
            int nAxesMask2 = 0x0;

            if (nAxisX < 32)
            {
                nAxesMask1 |= (1 << nAxisX);
            }
            else
            {
                nAxesMask2 |= ((1 << nAxisX) - 32);
            }
            if (nAxisY < 32)
            {
                nAxesMask1 |= (1 << nAxisY);
            }
            else
            {
                nAxesMask2 |= ((1 << nAxisY) - 32);
            }

            CMDLL.cmmIxMapAxes(IX_MAP_IDX, nAxesMask1, nAxesMask2);

            CommSetSpeedPattern();

            int nMovMode = this.cmbMovMode.SelectedIndex;
            int nSpeedPattern = this.cmbSpeedPattern.SelectedIndex;
            int nSpeedMode = this.cmbSpeedMode.SelectedIndex;

            // Axis-X Speed Config
            double fWorkX = Convert.ToDouble(this.txtWorkX.Text);
            double fAccX = Convert.ToDouble(this.txtAccX.Text);
            double fDecX = Convert.ToDouble(this.txtDecX.Text);

            // Axis-Y Speed Config
            double fWorkY = Convert.ToDouble(this.txtWorkY.Text);
            double fAccY = Convert.ToDouble(this.txtAccY.Text);
            double fDecY = Convert.ToDouble(this.txtDecY.Text);

            // Vector Speed Config
            double fWorkVec = Convert.ToDouble(this.txtWorkVec.Text);
            double fAccVec = Convert.ToDouble(this.txtAccVec.Text);
            double fDecVec = Convert.ToDouble(this.txtDecVec.Text);

            // Arc Method - Arc Angle
            double fAngCenAxisX = Convert.ToDouble(this.txtAngCenAxisX.Text);
            double fAngCenAxisY = Convert.ToDouble(this.txtAngCenAxisY.Text);

            // Arc Method - Arc Point
            double fPntCenAxisX = Convert.ToDouble(this.txtPntCenAxisX.Text);
            double fPntCenAxisY = Convert.ToDouble(this.txtPntCenAxisY.Text);
            double fPntEndAxisX = Convert.ToDouble(this.txtPntEndAxisX.Text);
            double fPntEndAxisY = Convert.ToDouble(this.txtPntEndAxisY.Text);

            // Arc Method - Arc 3Point
            double f3PntP2AxisX = Convert.ToDouble(this.txt3PntP2AxisX.Text);
            double f3PntP2AxisY = Convert.ToDouble(this.txt3PntP2AxisY.Text);
            double f3PntP3AxisX = Convert.ToDouble(this.txt3PntP3AxisX.Text);
            double f3PntP3AxisY = Convert.ToDouble(this.txt3PntP3AxisY.Text);

            double fAngEndDeg=0;

            if (nMovMode == 0) // Moving Mode 를 확인
            {
                if (nSpeedMode == 0) // Speed Mode 를 확인
                {
                    switch (m_nArcMotionMode)
                    {
                        case (int)ARC_MODE.ARC_ANGLE:

                            //Relative - Master - Arc_Angle
                            //cmmIxSetSpeedPattern - Master
                            //cmmIxArcAStart
                            //cmmIxWateDone
                            CMDLL.cmmCfgSetSpeedPattern(nAxisX, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmCfgSetSpeedPattern(nAxisY, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE, nSpeedPattern, 100, 100, 100);
                            fAngEndDeg = Convert.ToDouble(this.txtAngEndDeg.Text);
                            CMDLL.cmmIxArcAStart(IX_MAP_IDX, fAngCenAxisX, fAngCenAxisY, fAngEndDeg);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                        case (int)ARC_MODE.ARC_POINT:

                            //Relative - Master - Arc_Point
                            //cmmIxSetSpeedPattern - Master
                            //cmmIxArcPStart
                            //cmmIxWateDone
                            CMDLL.cmmCfgSetSpeedPattern(nAxisX, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmCfgSetSpeedPattern(nAxisY, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE, nSpeedPattern, 100, 100, 100);
                            CMDLL.cmmIxArcPStart(IX_MAP_IDX, fPntCenAxisX, fPntCenAxisY, fPntEndAxisX, fPntEndAxisY, (int)Defines._TCmArcDir.cmARC_CW);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                        case (int)ARC_MODE.ARC_3POINT:

                            //Relative - Master - 3_Point
                            //cmmIxSetSpeedPattern - Master
                            //cmmIxArc3PStart
                            //cmmIxWateDone
                            CMDLL.cmmCfgSetSpeedPattern(nAxisX, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmCfgSetSpeedPattern(nAxisY, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE, nSpeedPattern, 100, 100, 100);

                            fAngEndDeg = Convert.ToDouble(this.txtAngleFor3P.Text);
                            CMDLL.cmmIxArc3PStart(IX_MAP_IDX, f3PntP2AxisX, f3PntP2AxisY, f3PntP3AxisX, f3PntP3AxisX, fAngEndDeg);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                    }
                }
                else
                {
                    switch (m_nArcMotionMode)
                    {
                        case (int)ARC_MODE.ARC_ANGLE:

                            //Relative - Master - Arc_Angle
                            //cmmIxSetSpeedPattern - Vector
                            //cmmIxArcAStart
                            //cmmIxWateDone
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmTRUE, nSpeedPattern, fWorkVec, fAccVec, fDecVec);
                            fAngEndDeg = Convert.ToDouble(this.txtAngEndDeg.Text);
                            CMDLL.cmmIxArcAStart(IX_MAP_IDX, fAngCenAxisX, fAngCenAxisY, fAngEndDeg);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                        case (int)ARC_MODE.ARC_POINT:

                            //Relative - Master - Arc_Point
                            //cmmIxSetSpeedPattern - Vector
                            //cmmIxArcPStart
                            //cmmIxWateDone
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmTRUE, nSpeedPattern, fWorkVec, fAccVec, fDecVec);
                            CMDLL.cmmIxArcPStart(IX_MAP_IDX, fPntCenAxisX, fPntCenAxisY, fPntEndAxisX, fPntEndAxisY, (int)Defines._TCmArcDir.cmARC_CW);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                        case (int)ARC_MODE.ARC_3POINT:

                            //Relative - Master - 3_Point
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmTRUE, nSpeedPattern, fWorkVec, fAccVec, fDecVec);
                            fAngEndDeg = Convert.ToDouble(this.txtAngleFor3P.Text);
                            CMDLL.cmmIxArc3PStart(IX_MAP_IDX, f3PntP2AxisX, f3PntP2AxisY, f3PntP3AxisX, f3PntP3AxisX, fAngEndDeg);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                    }
                }
            }
            else
            {
                if (nSpeedMode == 0) // Speed Mode 를 확인
                {
                    switch (m_nArcMotionMode)
                    {
                        case (int)ARC_MODE.ARC_ANGLE:

                            //Relative - Master - Arc_Angle
                            //cmmIxSetSpeedPattern - Master
                            //cmmIxArcAStart
                            //cmmIxWateDone
                            CMDLL.cmmCfgSetSpeedPattern(nAxisX, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmCfgSetSpeedPattern(nAxisY, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE, nSpeedPattern, 100, 100, 100);
                            fAngEndDeg = Convert.ToDouble(this.txtAngEndDeg.Text);
                            CMDLL.cmmIxArcAToStart(IX_MAP_IDX, fAngCenAxisX, fAngCenAxisY, fAngEndDeg);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                        case (int)ARC_MODE.ARC_POINT:

                            //Relative - Master - Arc_Point
                            //cmmIxSetSpeedPattern - Master
                            //cmmIxArcPStart
                            //cmmIxWateDone
                            CMDLL.cmmCfgSetSpeedPattern(nAxisX, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmCfgSetSpeedPattern(nAxisY, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE, nSpeedPattern, 100, 100, 100);
                            CMDLL.cmmIxArcPToStart(IX_MAP_IDX, fPntCenAxisX, fPntCenAxisY, fPntEndAxisX, fPntEndAxisY, (int)Defines._TCmArcDir.cmARC_CW);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                        case (int)ARC_MODE.ARC_3POINT:

                            //Relative - Master - 3_Point
                            //cmmIxSetSpeedPattern - Master
                            //cmmIxArc3PStart
                            //cmmIxWateDone
                            CMDLL.cmmCfgSetSpeedPattern(nAxisX, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmCfgSetSpeedPattern(nAxisY, nSpeedPattern, fWorkX, fAccX, fDecX);
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE, nSpeedPattern, 100, 100, 100);
                            fAngEndDeg = Convert.ToDouble(this.txtAngleFor3P.Text);
                            CMDLL.cmmIxArc3PToStart(IX_MAP_IDX, f3PntP2AxisX, f3PntP2AxisY, f3PntP3AxisX, f3PntP3AxisX, fAngEndDeg);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                    }
                }
                else
                {
                    switch (m_nArcMotionMode)
                    {
                        case (int)ARC_MODE.ARC_ANGLE:

                            //Relative - Master - Arc_Angle
                            //cmmIxSetSpeedPattern - Vector
                            //cmmIxArcAStart
                            //cmmIxWateDone
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmTRUE, nSpeedPattern, fWorkVec, fAccVec, fDecVec);
                            CMDLL.cmmIxArcAToStart(IX_MAP_IDX, fAngCenAxisX, fAngCenAxisY, fAngEndDeg);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                        case (int)ARC_MODE.ARC_POINT:

                            //Relative - Master - Arc_Point
                            //cmmIxSetSpeedPattern - Vector
                            //cmmIxArcPStart
                            //cmmIxWateDone
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmTRUE, nSpeedPattern, fWorkVec, fAccVec, fDecVec);
                            CMDLL.cmmIxArcPToStart(IX_MAP_IDX, fPntCenAxisX, fPntCenAxisY, fPntEndAxisX, fPntEndAxisY, (int)Defines._TCmArcDir.cmARC_CW);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                        case (int)ARC_MODE.ARC_3POINT:

                            //Relative - Master - 3_Point
                            //cmmIxSetSpeedPattern - Vector
                            //cmmIxArc3PStart
                            //cmmIxWateDone
                            CMDLL.cmmIxSetSpeedPattern(IX_MAP_IDX, (int)Defines._TCmBool.cmTRUE, nSpeedPattern, fWorkVec, fAccVec, fDecVec);
                            fAngEndDeg = Convert.ToDouble(this.txtAngleFor3P.Text);
                            CMDLL.cmmIxArc3PToStart(IX_MAP_IDX, f3PntP2AxisX, f3PntP2AxisY, f3PntP3AxisX, f3PntP3AxisX, fAngEndDeg);
                            CMDLL.cmmIxWaitDone(IX_MAP_IDX, (int)Defines._TCmBool.cmFALSE);
                            break;
                    }
                }
            }

            btnMoveN.Enabled = true;
            btnMoveP.Enabled = true;
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            if (CMDLL.cmmIxStop(IX_MAP_IDX, (int)Defines._TCmBool.cmTRUE, (int)Defines._TCmBool.cmFALSE) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle);
            }
        }

        private void btnStopEmg_Click(object sender, EventArgs e)
        {
            if (CMDLL.cmmIxStopEmg(IX_MAP_IDX) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle);
            }
        }
    }
}
