﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Shared;
using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace OverridePosition
{
    public partial class OverridePosition : Form
    {
        SpeedSettingForm m_frmSpeedSetup = null;
        
        public OverridePosition()
        {
            InitializeComponent();
        }

        // common function
        void InitMotionDevices()
        {
            ////////////////////////////////////////////////////////////////////////////////////////
            // [CAUTION]: 아래와 다른 CME파일을 사용하는 경우에는 아래 코드를 수정하여 올바른 CME 파일의
            // 경로를 지정하십시오.
            const string CME_FILE_NAME = "Default.cme2";

            string szCmeFilePath, szSystemDir;

            szSystemDir = Environment.SystemDirectory; // 윈도우즈 시스템 디렉토리명 얻어오기 (일반적으로는 C:\Windows\System32)
            szCmeFilePath = szSystemDir + "\\" + CME_FILE_NAME;

            if (CMDLL.cmmGnInitFromFile(szCmeFilePath) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(IntPtr.Zero); // 에러 발생하였으면 에러 디스플레이.
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////////////
                // CME 파일에서 정의한 축수와 실제 장착되어 있는 모션 축수를 비교확인 하고자할 때는 
                // 아래와 같이 하면 됩니다.

                int anNumAxes_installed = 0;
                int anNumAxes_defined = 0;

                CMDLL.cmmAdvGetNumAvailAxes(ref anNumAxes_installed); // 현재 실제로 PC에 장착되어 있는 모션축 수 (cmmGnDeviceLoad() 함수에서 반환하는 축수와 동일)
                CMDLL.cmmAdvGetNumDefinedAxes(ref anNumAxes_defined); // CME 파일에서 정의한 축 수(CME 파일이 지정되지 않는 경우에는 현재 장착된 축 수와 일치합니다.)

                if (anNumAxes_installed != anNumAxes_defined)
                {
                    MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }

            // SERVO-ON //
            int anNumAxes = 0;
            CMDLL.cmmAdvGetNumDefinedAxes(ref anNumAxes);
            for (int i = 0; i < anNumAxes; i++)
            {
                CMDLL.cmmGnSetServoOn(i, (int)Defines._TCmBool.cmTRUE);
            }
        }

        private void OverridePosition_Load(object sender, EventArgs e)
        {
            // Load Motion & DIO devices //
            int anNumAxes = 0;
            if (CMDLL.cmmGnDeviceLoad((int)Defines._TCmBool.cmTRUE, ref anNumAxes) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle); // 에러 발생하였으면 에러 디스플레이합니다.
                return;
            }
            else
            {
                // Motion 환경 초기화 //
                InitMotionDevices();

                ////////////////////////////////////////////////////////////////////////////////////
                int i;

                // 사용 가능한 축의 갯수 얻어오기
                int anNumAxesToDisp = 0;
                CMDLL.cmmAdvGetNumAvailAxes(ref anNumAxesToDisp);
                int nNumAxes = anNumAxesToDisp;

                for (i = 0; i < nNumAxes; i++)
                {
                    this.cmbAxis.Items.Insert(i, "Axis" + i.ToString());
                }

                // 축선택의 기본값으로는 0번과 1번 두 축이 선택된 것으로 한다.
                this.cmbAxis.SelectedIndex = 0;

                // 초기 속도 패턴 설정
                for (i = 0; i < nNumAxes; i++)
                {
                    CMDLL.cmmCfgSetSpeedPattern(i, (int)Defines.SPDMODE.MODE_SCURVE, 10000, 100000, 100000);
                }

                // Speed ratio 슬라이더의 값 범위 설정 //
                trbSpdRatio.SetRange(1, 100);
                trbSpdRatio.Value = (trbSpdRatio.Maximum < 100) ? trbSpdRatio.Maximum : 100;

                // Set timer to read and display current command position //
                this.timer1.Start();
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            //------- 모션제어기 위치 모니터링 -------------------------------//
            int nAxis = this.cmbAxis.SelectedIndex;
            string sPos;
            double afCurPos = 0;

            // Command position (X) //
            CMDLL.cmmStGetPosition(nAxis, (int)Defines._TCmCntr.cmCNT_COMM, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.lblPosC.Text = sPos;
            // Feedback position (X) //
            CMDLL.cmmStGetPosition(nAxis, (int)Defines._TCmCntr.cmCNT_FEED, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.lblPosF.Text = sPos;

            int nIsStopped = 0;
            CMDLL.cmmSxIsDone(nAxis, ref nIsStopped);
            if (nIsStopped == 0)
            {
                this.btnMove.Text = "Override";
            }
            else
            {
                this.btnMove.Text = "Move";
                this.lblStatus.Text = "정지상태";
            }
        }

        private void trbSpdRatio_Scroll(object sender, EventArgs e)
        {
            this.lblSpeedRatio.Text = trbSpdRatio.Value.ToString() + " (%)";
        }

        /***********************************************************************
        * OnResetPos() : Position "Reset" 버튼의 콜백함수. Command & Feedback 
        * position을 0으로 리셋한다.
        ***********************************************************************/
        private void btnResetPos_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            CMDLL.cmmStSetPosition(nAxis, (int)Defines._TCmCntr.cmCNT_COMM, 0.0); // Reset command position of X-axis
            CMDLL.cmmStSetPosition(nAxis, (int)Defines._TCmCntr.cmCNT_FEED, 0.0); // Reset feedback position of X-axis
        }

        private void btnSpeedSetup_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;

            if (IsFormExist(typeof(SpeedSettingForm)) == null)
            {
                m_frmSpeedSetup = new SpeedSettingForm(nAxis);
            }

            m_frmSpeedSetup.ShowDialog();
            m_frmSpeedSetup.Focus();
        }

        private void btnMove_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nIsStopped = 0;
            int nResult = 0;

            // 속도비 설정 //
            double fVelR = Convert.ToDouble(this.trbSpdRatio.Value);
            CMDLL.cmmSxSetSpeedRatio(nAxis, (int)Defines._TCmSpeedMode.cmSMODE_KEEP, fVelR, 100, 100); // Speed ratio 설정: 가감속은 편의상 100%로 고정함

            int nPosVal = int.Parse(this.txtPosVal.Text);
            bool boolIsAbsMode = this.rdoAbsPos.Checked;
            CMDLL.cmmSxIsDone(nAxis, ref nIsStopped);
            if (nIsStopped == 1)
            {
                if(boolIsAbsMode)
                {
                    CMDLL.cmmSxMoveToStart(nAxis, nPosVal);
                }
                else
                {
                    CMDLL.cmmSxMoveStart(nAxis, nPosVal);
                }
                this.lblStatus.Text = "이송 시작됨, 이제 이송 거리를 재설정 하시고 Override 버튼을 클릭하세요";
            }
            else
            {
                if (boolIsAbsMode)
                {
                    int nIsIgnord = 0;
                    nResult = CMDLL.cmmOverrideMoveTo(nAxis, nPosVal, ref nIsIgnord);
                    if(nResult != Defines.cmERR_NONE)
                    {
                        CMDLL.cmmErrShowLast(this.Handle);
                    }
                    // nIsIgnord==1 이면 이송이 이미 완료되었거나 감속이 거의 끝난 상태로 오버라이드가 무시된 경우이다. 이때는 절대치 목표좌표 까지 단축 이송으로 보정 이송해줄 필요가 있다.//
                    if (nIsIgnord == 1) // 이송이 완료되었어도 반드시 해당위치로 가야한다면 아래와 같이 MoveTo 함수를 수행해준다.
                    {
                        DialogResult result = MessageBox.Show("오버라이드가 무시됨", "Confirm", MessageBoxButtons.YesNo);

                        if (result == DialogResult.Yes)
                        {
                            CMDLL.cmmSxMoveToStart(nAxis, nPosVal);
                            CMDLL.cmmSxWaitDone(nAxis, (int)Defines._TCmBool.cmFALSE);
                        }
                    }
                    else
                    {
                        this.lblStatus.Text = "위치 오버라이드 적용 됨";
                    }
                }
                else
                {
                    int nIsIgnord = 0;
                    nResult = CMDLL.cmmOverrideMove(nAxis, nPosVal, ref nIsIgnord);
                    if (nResult != Defines.cmERR_NONE)
                    {
                        CMDLL.cmmErrShowLast(this.Handle);
                    }

                    if (nIsIgnord == 1) // 이송이 완료되었어도 반드시 해당위치로 가야한다면 아래와 같이 MoveTo 함수를 수행해준다.
                    {
                        MessageBox.Show("오버라이드가 무시됨", "Confirm", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    }
                    else
                    {
                        this.lblStatus.Text = "위치 오버라이드 적용 됨";
                    }
                }
            }
        }

        /***********************************************************************
        * OnbtnStop() : Position "Reset" 버튼의 콜백함수. Command & Feedback 
        * position을 0으로 리셋한다.
        ***********************************************************************/
        private void btnStop_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            CMDLL.cmmSxStop(nAxis, (int)Defines._TCmBool.cmTRUE, (int)Defines._TCmBool.cmFALSE);
            this.lblStatus.Text = "정지됨";
        }

        // Form Check
        private Form IsFormExist(Type FormType)
        {
            foreach (Form OpenForm in Application.OpenForms)
            {
                if (OpenForm.GetType() == FormType)
                    return OpenForm;
            }
            return null;
        }

        private void OverridePosition_FormClosing(object sender, FormClosingEventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            CMDLL.cmmSxStop(nAxis, (int)Defines._TCmBool.cmFALSE, (int)Defines._TCmBool.cmFALSE);

            // 사용중인 타이머 종료.
            timer1.Stop();

            CMDLL.cmmGnDeviceUnload(); // Unload Device
        }
    }
}
