﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Shared;

using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace IxxHelical
{
    public partial class IxxHelical : Form
    {
        private int[] nAxes = new int[4];
        private int HELICAL_ID = 0; // ID는 0 ~ 15 사이의 값을 취할 수 있습니다.

        // common function
        void InitMotionDevices()
        {
            ////////////////////////////////////////////////////////////////////////////////////////
            // [CAUTION]: 아래와 다른 CME파일을 사용하는 경우에는 아래 코드를 수정하여 올바른 CME 파일의
            // 경로를 지정하십시오.
            const string CME_FILE_NAME = "Default.cme2";

            string szCmeFilePath, szSystemDir;

            szSystemDir = Environment.SystemDirectory; // 윈도우즈 시스템 디렉토리명 얻어오기 (일반적으로는 C:\Windows\System32)
            szCmeFilePath = szSystemDir + "\\" + CME_FILE_NAME;

            if (CMDLL.cmmGnInitFromFile(szCmeFilePath) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(IntPtr.Zero); // 에러 발생하였으면 에러 디스플레이.
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////////////
                // CME 파일에서 정의한 축수와 실제 장착되어 있는 모션 축수를 비교확인 하고자할 때는 
                // 아래와 같이 하면 됩니다.

                int anNumAxes_installed = 0;
                int anNumAxes_defined = 0;

                CMDLL.cmmAdvGetNumAvailAxes(ref anNumAxes_installed); // 현재 실제로 PC에 장착되어 있는 모션축 수 (cmmGnDeviceLoad() 함수에서 반환하는 축수와 동일)
                CMDLL.cmmAdvGetNumDefinedAxes(ref anNumAxes_defined); // CME 파일에서 정의한 축 수(CME 파일이 지정되지 않는 경우에는 현재 장착된 축 수와 일치합니다.)

                if (anNumAxes_installed != anNumAxes_defined)
                {
                    MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }

            // SERVO-ON //
            int anNumAxes = 0;
            CMDLL.cmmAdvGetNumDefinedAxes(ref anNumAxes);
            for (int i = 0; i < anNumAxes; i++)
            {
                CMDLL.cmmGnSetServoOn(i, (int)Defines._TCmBool.cmTRUE);
            }
        }
        
        public IxxHelical()
        {
            InitializeComponent();
        }

        private void IxxHelical_Load(object sender, EventArgs e)
        {
            // Load Motion & DIO devices //
            int anNumAxes = 0;
            if (CMDLL.cmmGnDeviceLoad((int)Defines._TCmBool.cmTRUE, ref anNumAxes) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle); // 에러 발생하였으면 에러 디스플레이합니다.
                return;
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////
                // 로드된 축수가 올바른지 체크: 본 예제에서는 원하는 축 수를 모르기 때문에 
                // 주석처리 해놓았습니다.

                //				const int NUM_AXES_TO_LOAD = 4; // 이 값을 시스템에서 필요한 축 수에 맞게 조정 
                //				if(anNumAxes[0] < NUM_AXES_TO_LOAD){
                //					MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                //				}

                ////////////////////////////////////////////////////////////////////////////////////

                if (anNumAxes < 4)
                {
                    MessageBox.Show("이 예제는 4축 이상의 컨트롤러에서만 유효한 예제입니다! \n프로그램을 종료합니다.");
                    return;
                }

                // Motion 환경 초기화 //
                InitMotionDevices();

                nAxes[0] = 0;
                nAxes[1] = 1;
                nAxes[2] = 2;
                nAxes[3] = 3;

                this.cmbSpeedMaster.SelectedIndex = 2;
                this.cmbSpeedMode.SelectedIndex = (int)Defines.SPDMODE.MODE_CONST;
                this.cmbDirU.SelectedIndex = 1;
                
                // Set timer to read and display current command position //
                this.timer1.Start();
            }
        }

        private void IxxHelical_FormClosing(object sender, FormClosingEventArgs e)
        {
            // 사용중인 타이머 종료.
            timer1.Stop();

            CMDLL.cmmGnDeviceUnload(); // Unload Device
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            string sPos;
            double afCurPos = 0;

            //--------- Position display -----------------------//

            // Command position (X) //
            CMDLL.cmmStGetPosition(0, (int)Defines._TCmCntr.cmCNT_COMM, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.lblPosC0.Text = sPos;
            // Feedback position (X) //
            CMDLL.cmmStGetPosition(0, (int)Defines._TCmCntr.cmCNT_FEED, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.lblPosF0.Text = sPos;

            // Command position (Y) //
            CMDLL.cmmStGetPosition(1, (int)Defines._TCmCntr.cmCNT_COMM, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.lblPosC1.Text = sPos;
            // Feedback position (Y) //
            CMDLL.cmmStGetPosition(1, (int)Defines._TCmCntr.cmCNT_FEED, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.lblPosF1.Text = sPos;

            // Command position (Z) //
            CMDLL.cmmStGetPosition(2, (int)Defines._TCmCntr.cmCNT_COMM, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.lblPosC2.Text = sPos;
            // Feedback position (Z) //
            CMDLL.cmmStGetPosition(2, (int)Defines._TCmCntr.cmCNT_FEED, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.lblPosF2.Text = sPos;

            // Command position (U) //
            CMDLL.cmmStGetPosition(3, (int)Defines._TCmCntr.cmCNT_COMM, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.lblPosC3.Text = sPos;
            // Feedback position (U) //
            CMDLL.cmmStGetPosition(3, (int)Defines._TCmCntr.cmCNT_FEED, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.lblPosF3.Text = sPos;
        }

        private void btnResetPos_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < 4; i++ )
            {
                CMDLL.cmmStSetPosition(i, (int)Defines._TCmCntr.cmCNT_COMM, 0.0); // Reset command position
                CMDLL.cmmStSetPosition(i, (int)Defines._TCmCntr.cmCNT_FEED, 0.0); // Reset command position
            }
        }

        private void btnGo_Click(object sender, EventArgs e)
        {
            this.btnGo.Enabled = false;
            
            double[] fCoorList = new double[4];
            double fActualDistU = 0;

            double fArcCentX = Convert.ToDouble(this.txtArcCentX.Text);
            double fArcCentY = Convert.ToDouble(this.txtArcCentY.Text);
            double fDistZ = Convert.ToDouble(this.txtDistZ.Text);
            double fDirU = Convert.ToDouble(this.cmbDirU.SelectedIndex);
            int nNumberRepeat = int.Parse(this.txtNumRepeat.Text);

            int nSpeedMaster = this.cmbSpeedMaster.SelectedIndex;
            int nSpeedMode = this.cmbSpeedMode.SelectedIndex;
            double fVel = Convert.ToDouble(this.txtVel.Text);
            double fAcc = Convert.ToDouble(this.txtAcc.Text);
            double fDec = Convert.ToDouble(this.txtDec.Text);

            // 참고: 코드의 간결성을 아래에서는 에러 처리를 생략합니다. //
            int LM_MAP = 0;
            fCoorList[0] = fArcCentX; // X(Axis0) 축의 원호 중심 거리
            fCoorList[1] = fArcCentY; // Y(Axis1) 축의 원호 중심 거리
            fCoorList[2] = fDistZ; // Z(Axis2) 축의 원호 중심 거리 
            fCoorList[3] = fDirU; // U(Axis3) 축의 방향 (U축의 거리는 원주 길이와 같도록 자동으로 설정되며 사용자는 그 방향만 설정할 수 있습니다)

            if(nNumberRepeat > 1) // 반복횟수(나사산이 2개 이상)가 2 이상인 경우에는 모션의 연속성을 확보하기 위해서 리스트 모션을 사용합니다.
            {
                CMDLL.cmmLmMapAxes(LM_MAP, 0xf, 0x0); // Set List-motion Map (Axis0 ~ Axis3)
                CMDLL.cmmLmBeginList(LM_MAP); // Start to list-up motion commands

                for (int i = 0; i < nNumberRepeat; i++ )
                {
                    if (i == 0) // 첫번째는 가속은 적용하되 다음 모션과의 속도 연속성을 위해서 감속구간을 없앱니다(0으로 합니다)
                    {
                        CMDLL.cmmIxxHelOnceSetSpeed(HELICAL_ID, nSpeedMaster, nSpeedMode, fVel, fAcc, 0);
                    }
                    else if (i == (nNumberRepeat - 1)) // 첫번째는 감속은 적용하되 이전 모션과의 속도 연속성을 위해서 가속구간을 없앱니다(0으로 합니다)
                    {
                        CMDLL.cmmIxxHelOnceSetSpeed(HELICAL_ID, nSpeedMaster, nSpeedMode, fVel, 0, fDec);
                    }
                    else // 나머지 구간에서는 가감속 구간을 모두 없앱니다.
                    {
                        CMDLL.cmmIxxHelOnceSetSpeed(HELICAL_ID, nSpeedMaster, nSpeedMode, fVel, 0, 0);
                    }
                    CMDLL.cmmIxxHelOnce(HELICAL_ID, 4, nAxes, fCoorList, 360, ref fActualDistU, (int)Defines._TCmBool.cmTRUE);
                }
                CMDLL.cmmLmEndList(LM_MAP); // End to list-up motion commands
                CMDLL.cmmLmStartMotion(LM_MAP); // real start of listed motion commands
                CMDLL.cmmLmWaitDone(LM_MAP, (int)Defines._TCmBool.cmFALSE);
                LM_MAP = 0;
            }
            else // 1회의 헬리컬보간은 리스트모션을 사용하지 않아도 된다.
            {
                CMDLL.cmmIxxHelOnceSetSpeed(HELICAL_ID, nSpeedMaster, nSpeedMode, fVel, fAcc, fDec);
                CMDLL.cmmIxxHelOnce(HELICAL_ID, 4, nAxes, fCoorList, 360, ref fActualDistU, (int)Defines._TCmBool.cmTRUE);
            }

            this.btnGo.Enabled = true;
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            this.btnGo.Enabled = true;
            this.btnStop.Enabled = false;

            CMDLL.cmmMxStop(4, nAxes, (int)Defines._TCmBool.cmTRUE, (int)Defines._TCmBool.cmFALSE);

            this.btnStop.Enabled = true;
        }
    }
}
