﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Shared;
using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace OverrideSpeed
{
    public partial class OverrideSpeed : Form
    {
        private static double fWorkSpeed = 0;
        private static double fAcceleration = 0;
        private static double fDeceleration = 0;
        
        public OverrideSpeed()
        {
            InitializeComponent();
        }

        // common function
        void InitMotionDevices()
        {
            ////////////////////////////////////////////////////////////////////////////////////////
            // [CAUTION]: 아래와 다른 CME파일을 사용하는 경우에는 아래 코드를 수정하여 올바른 CME 파일의
            // 경로를 지정하십시오.
            const string CME_FILE_NAME = "Default.cme2";

            string szCmeFilePath, szSystemDir;

            szSystemDir = Environment.SystemDirectory; // 윈도우즈 시스템 디렉토리명 얻어오기 (일반적으로는 C:\Windows\System32)
            szCmeFilePath = szSystemDir + "\\" + CME_FILE_NAME;

            if (CMDLL.cmmGnInitFromFile(szCmeFilePath) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(IntPtr.Zero); // 에러 발생하였으면 에러 디스플레이.
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////////////
                // CME 파일에서 정의한 축수와 실제 장착되어 있는 모션 축수를 비교확인 하고자할 때는 
                // 아래와 같이 하면 됩니다.

                int anNumAxes_installed = 0;
                int anNumAxes_defined = 0;

                CMDLL.cmmAdvGetNumAvailAxes(ref anNumAxes_installed); // 현재 실제로 PC에 장착되어 있는 모션축 수 (cmmGnDeviceLoad() 함수에서 반환하는 축수와 동일)
                CMDLL.cmmAdvGetNumDefinedAxes(ref anNumAxes_defined); // CME 파일에서 정의한 축 수(CME 파일이 지정되지 않는 경우에는 현재 장착된 축 수와 일치합니다.)

                if (anNumAxes_installed != anNumAxes_defined)
                {
                    MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }

            // SERVO-ON //
            int anNumAxes = 0;
            CMDLL.cmmAdvGetNumDefinedAxes(ref anNumAxes);
            for (int i = 0; i < anNumAxes; i++)
            {
                CMDLL.cmmGnSetServoOn(i, (int)Defines._TCmBool.cmTRUE);
            }
        }

        private void OverrideSpeed_Load(object sender, EventArgs e)
        {
            // Load Motion & DIO devices //
            int anNumAxes = 0;
            if (CMDLL.cmmGnDeviceLoad((int)Defines._TCmBool.cmTRUE, ref anNumAxes) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle); // 에러 발생하였으면 에러 디스플레이합니다.
                return;
            }
            else
            {
                // Motion 환경 초기화 //
                InitMotionDevices();

                ////////////////////////////////////////////////////////////////////////////////////
                int i;

                // 사용 가능한 축의 갯수 얻어오기
                int anNumAxesToDisp = 0;
                CMDLL.cmmAdvGetNumAvailAxes(ref anNumAxesToDisp);
                int nNumAxes = anNumAxesToDisp;

                for (i = 0; i < nNumAxes; i++)
                {
                    this.cmbAxis.Items.Insert(i, "Axis" + i.ToString());
                }

                // 축선택의 기본값으로는 0번과 1번 두 축이 선택된 것으로 한다.
                this.cmbAxis.SelectedIndex = 0;

                cmbAxis_SelectedIndexChanged(null, null);

                // ratio 슬라이더의 값 범위 설정 //
                this.tbrVelRatio.SetRange(1, 100);
                this.tbrVelRatio.Value = (this.tbrVelRatio.Maximum < 100) ? this.tbrVelRatio.Maximum : 100;
                this.tbrAccRatio.SetRange(1, 100);
                this.tbrAccRatio.Value = (this.tbrAccRatio.Maximum < 100) ? this.tbrAccRatio.Maximum : 100;
                this.tbrDecRatio.SetRange(1, 100);
                this.tbrDecRatio.Value = (this.tbrDecRatio.Maximum < 100) ? this.tbrDecRatio.Maximum : 100;

                // Set timer to read and display current command position //
                this.timer1.Start();
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            string sPos;
            double afCurPos = 0;

            // Command position display //
            CMDLL.cmmStGetPosition(nAxis, (int)Defines._TCmCntr.cmCNT_COMM, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.lblPosC.Text = sPos;

            // Feedback position display //
            CMDLL.cmmStGetPosition(nAxis, (int)Defines._TCmCntr.cmCNT_FEED, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.lblPosF.Text = sPos;

            // Command velocity display //
            CMDLL.cmmStGetSpeed(nAxis, (int)Defines._TCmCntr.cmCNT_COMM, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.lblVelC.Text = sPos;

            // Feedback velocity display //
            CMDLL.cmmStGetSpeed(nAxis, (int)Defines._TCmCntr.cmCNT_FEED, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.lblVelF.Text = sPos;

            //-------- 각 버튼의 Dimmed 처리를 상황에 맞추어서 수행 ---------//
            int nIsStopped = 0;
            CMDLL.cmmSxIsDone(nAxis, ref nIsStopped);
            if(nIsStopped == 1)
            {
                this.btnStart.Enabled = true;
                this.btnStop.Enabled = false;
                this.btnOverPattern.Enabled = false;
                this.btnOverRatio.Enabled = false;
            }
            else
            {
                this.btnStart.Enabled = false;
                this.btnStop.Enabled = true;
                this.btnOverPattern.Enabled = true;
                this.btnOverRatio.Enabled = true;
            }
        }

        //**********************************************************************************
        // OnSelchangeAxisSel(): "Axis" Combobox의 선택이 변경되었을 때 호출되는 이벤트 핸들러
        // 축 선택이 바뀌면 해당 축의 현재 속도 설정을 읽어서 화면을 업데이트해준다.
        //**********************************************************************************
        private void cmbAxis_SelectedIndexChanged(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nMode = 0;
            CMDLL.cmmCfgGetSpeedPattern(nAxis, ref nMode, ref fWorkSpeed, ref fAcceleration, ref fDeceleration);
            this.cmbSpeedMode.SelectedIndex = nMode;

            double fVelR = 0, fAccR = 0, fDecR = 0;
            int nNullMode = 0;
            CMDLL.cmmSxGetSpeedRatio(nAxis, ref nNullMode, ref fVelR, ref fAccR, ref fDecR);
            this.tbrVelRatio.Text = fVelR.ToString();
            this.tbrAccRatio.Text = fAccR.ToString();
            this.tbrDecRatio.Text = fDecR.ToString();
        }

        private void tbrVelRatio_Scroll(object sender, EventArgs e)
        {
            this.lblSpeedRatio.Text = tbrVelRatio.Value.ToString() + " (%)";
        }

        private void tbrAccRatio_Scroll(object sender, EventArgs e)
        {
            this.lblAccRatio.Text = tbrAccRatio.Value.ToString() + " (%)";
        }

        private void tbrDecRatio_Scroll(object sender, EventArgs e)
        {
            this.lblDecRatio.Text = tbrDecRatio.Value.ToString() + " (%)";
        }

        //**************************************************************************************
        // OnOverrideSpeedPattern(): "Speed Pattern" 그룹의 "Override" 버튼의 이벤트 핸들러.
        // 정격속도 설정을 오버라이딩한다. 
        //**************************************************************************************
        private void btnOverPattern_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nSpeedMode = this.cmbSpeedMode.SelectedIndex;
            CMDLL.cmmCfgSetSpeedPattern(nAxis, nSpeedMode, fWorkSpeed, fAcceleration, fDeceleration); // 오버라이딩할 속도를 설정한다.
            CMDLL.cmmOverrideSpeedSet(nAxis); // 여기에서 실제 속도 오버라이딩이 수행된다. 

            this.lblStatus.Text = "속도 오버라이드 적용 됨";
        }

        //**************************************************************************************
        // OnOverrideSpeedRatio(): "Speed Ratio" 그룹의 "Override" 버튼의 이벤트 핸들러.
        // 속도비 설정을 오버라이딩한다. 
        //**************************************************************************************
        private void btnOverRatio_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nSpeedMode = this.cmbSpeedMode.SelectedIndex;
            double fVelR = Convert.ToDouble(this.tbrVelRatio.Value);
            double fAccR = Convert.ToDouble(this.tbrAccRatio.Value);
            double fDecR = Convert.ToDouble(this.tbrDecRatio.Value);
            CMDLL.cmmSxSetSpeedRatio(nAxis, nSpeedMode, fVelR, fAccR, fDecR); // 오버라이딩할 속도를 설정한다.
            CMDLL.cmmOverrideSpeedSet(nAxis); // 여기에서 실제 속도 오버라이딩이 수행된다. 

            this.lblStatus.Text = "속도 오버라이드 적용 됨";
        }

        //**************************************************************************************
        // OnStart(): "Start" 버튼 이벤트 핸들러. (+) 방향으로 속도모드 조그 이송을 시작한다.
        // 일반적으로 속도 설정을 할때 환경설정에서 정격속도를 설정해놓고 프로그램에서는 
        // 속도비를 조절하여 사용하는 것이 일반적이다. 따라서 일반적으로는 cmmSxSetSpeedRatio()만
        // 을 사용하면 된다. 본 예제에서는 cmmCfgSetSpeedPattern()에 의해서 결정되는 정격속도도
        // Overriding 될 수 있음을 예시하기 위해서 cmmCfgSetSpeedPattern() 함수도 함께 사용하였다.
        //**************************************************************************************
        private void btnStart_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nSpeedMode = this.cmbSpeedMode.SelectedIndex;
            // 정격속도 설정 //
            CMDLL.cmmCfgSetSpeedPattern(nAxis, nSpeedMode, fWorkSpeed, fAcceleration, fDeceleration);
            // 속도비 설정 //
            double fVelR = Convert.ToDouble(this.tbrVelRatio.Value);
            double fAccR = Convert.ToDouble(this.tbrAccRatio.Value);
            double fDecR = Convert.ToDouble(this.tbrDecRatio.Value);
            CMDLL.cmmSxSetSpeedRatio(nAxis, nSpeedMode, fVelR, fAccR, fDecR);

            if (CMDLL.cmmSxVMoveStart(nAxis, (int)Defines._TCmDir.cmDIR_P) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle);
            }

            this.lblStatus.Text = "이송 시작됨, 이제 이송 속도 및 속도비율을 재설정 하시고 Override 버튼을 클릭하세요";
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            CMDLL.cmmSxStop(nAxis, (int)Defines._TCmBool.cmTRUE, (int)Defines._TCmBool.cmFALSE);

            this.lblStatus.Text = "정지됨";
        }

        private void OverrideSpeed_FormClosing(object sender, FormClosingEventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            CMDLL.cmmSxStop(nAxis, (int)Defines._TCmBool.cmFALSE, (int)Defines._TCmBool.cmFALSE);

            // 사용중인 타이머 종료.
            timer1.Stop();

            CMDLL.cmmGnDeviceUnload(); // Unload Device
        }

        private void txtVwork_TextChanged(object sender, EventArgs e)
        {
            fWorkSpeed = Convert.ToDouble(this.txtVwork.Text);
        }

        private void txtAcc_TextChanged(object sender, EventArgs e)
        {
            fAcceleration = Convert.ToDouble(this.txtAcc.Text);
        }

        private void txtDec_TextChanged(object sender, EventArgs e)
        {
            fDeceleration = Convert.ToDouble(this.txtDec.Text);
        }
    }
}
