﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Shared;
using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace MxMotion
{
    public partial class SpeedSettingForm : Form
    {
        private static int nSetCh = 0;
        private static int nSpeedMode = (int)Defines.SPDMODE.MODE_SCURVE;
        private static double fMaxSpeed = 0;
        private static double fInitSpeed = 0;
        private static double fWorkSpeed = 0;
        private static double fAcceleration = 0;
        private static double fDeceleration = 0;

        int[] m_nAxes;
        int   m_nAxesCount = 0;
        
        public static int SETCH
        {
            get { return nSetCh; }
            set { nSetCh = value; }
        }

        public static int SPEEDMODE
        {
            get { return nSpeedMode; }
            set { nSpeedMode = value; }
        }

        public static double MAXSPEED
        {
            get { return fMaxSpeed; }
            set { fMaxSpeed = value; }
        }

        public static double INITSPEED
        {
            get { return fInitSpeed; }
            set { fInitSpeed = value; }
        }

        public static double WORKSPEED
        {
            get { return fWorkSpeed; }
            set { fWorkSpeed = value; }
        }

        public static double ACCELERATION
        {
            get { return fAcceleration; }
            set { fAcceleration = value; }
        }

        public static double DECELERATION
        {
            get { return fDeceleration; }
            set { fDeceleration = value; }
        }

        public SpeedSettingForm(int[] p_nAxes, int p_nAxesCount)
        {
            m_nAxes = p_nAxes; // MxMotion Form 에서 설정된 Checkbox 에 Check 된 Axis 축 List
            m_nAxesCount = p_nAxesCount; // MxMotion Form 에서 설정된 Checkbox Axis 축 갯수
            
            InitializeComponent();
        }

        private void SpeedSettingForm_Load(object sender, EventArgs e)
        {
            // 축설정 콤보박스에 축리스트를 채워넣습니다. 
            byte[] aszAxisTitle = new byte[Defines.CMM_MAX_STR_LEN_AXIS_TITLE];

            cmbAxisList.Items.Clear(); // Clear combobox
            string szTitle;

            for (int i = 0; i < m_nAxesCount; i++)
            {
                CMDLL.cmmAdvGetString(i, (int)Defines._TCmStringID.cmSTR_AXIS_NAME, aszAxisTitle); // CME 빌더에서 정의한 각 축의 타이틀을 가져오는 함수.

                szTitle = "";
                for (int j = 0; j < aszAxisTitle.Length; j++)
                {
                    szTitle += Convert.ToChar(aszAxisTitle[j]);
                }

                if (m_nAxes[i] > -1)
                {
                    cmbAxisList.Items.Add(szTitle);
                }
            }

            // 축설정 콤보박스 초기 선택값 지정
            if (cmbAxisList.Items.Count > 0)
                cmbAxisList.SelectedIndex = 0;
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            CMDLL.cmmCfgSetSpeedRange(SETCH, MAXSPEED); // 최대속도(MAXSPEED) 설정
            CMDLL.cmmSxOptSetIniSpeed(SETCH, INITSPEED); // 초기속도(INITSPEED) 설정
            CMDLL.cmmCfgSetSpeedPattern(SETCH, SPEEDMODE, WORKSPEED, ACCELERATION, DECELERATION); // 속도패턴(SPEEDMODE : CONSTANT, TRAPEZOIDAL, S-CURVE), 작업속도(WORKSPEED), 가속도(ACCELERATION), 감속도(DECELERATION) 설정
        }

        private void cmbSpeedMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            SPEEDMODE = cmbSpeedMode.SelectedIndex;
        }

        private void txtMaxSpeed_TextChanged(object sender, EventArgs e)
        {
            MAXSPEED = Convert.ToDouble(txtMaxSpeed.Text);
        }

        private void txtInitSpeed_TextChanged(object sender, EventArgs e)
        {
            INITSPEED = Convert.ToDouble(txtInitSpeed.Text);
        }

        private void txtWorkSpeed_TextChanged(object sender, EventArgs e)
        {
            WORKSPEED = Convert.ToDouble(txtWorkSpeed.Text);
        }

        private void txtAcc_TextChanged(object sender, EventArgs e)
        {
            ACCELERATION = Convert.ToDouble(txtAcc.Text);
        }

        private void txtDec_TextChanged(object sender, EventArgs e)
        {
            DECELERATION = Convert.ToDouble(txtDec.Text);
        }

        private void cmbAxisList_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Combobox 에 선택된 Axis(축) 을 기준으로 설정
            nSetCh = int.Parse(cmbAxisList.SelectedItem.ToString().Substring(4, 6));

            // 화면에 초기값을 설정하기 위해서 현재 설정되어 있는 속도 패턴을 읽어들인다.
            double fInitSpeed = 0;
            int nSpeedMode = 0;
            double fWorkSpeed = 0;
            double fAcc = 0;
            double fDec = 0;
            double fMinSpeed = 0, fMaxSpeed = 0;

            CMDLL.cmmCfgGetSpeedRange(SETCH, ref fMinSpeed, ref fMaxSpeed);
            MAXSPEED = fMaxSpeed;

            CMDLL.cmmSxOptGetIniSpeed(SETCH, ref fInitSpeed);
            INITSPEED = fInitSpeed;

            CMDLL.cmmCfgGetSpeedPattern(SETCH, ref nSpeedMode, ref fWorkSpeed, ref fAcc, ref fDec);
            SPEEDMODE = nSpeedMode;
            WORKSPEED = fWorkSpeed;
            ACCELERATION = fAcc;
            DECELERATION = fDec;

            this.cmbSpeedMode.SelectedIndex = SPEEDMODE;
            this.txtMaxSpeed.Text = MAXSPEED.ToString();
            this.txtInitSpeed.Text = INITSPEED.ToString();
            this.txtWorkSpeed.Text = WORKSPEED.ToString();
            this.txtAcc.Text = ACCELERATION.ToString();
            this.txtDec.Text = DECELERATION.ToString();

            // 속도 단위 스트링 표시 //
            byte[] aszSpeedUnit = new byte[Defines.CMM_MAX_STR_LEN_VEL_UNIT];

            CMDLL.cmmAdvGetString(SETCH, (int)Defines._TCmStringID.cmSTR_SPEED_UNIT, aszSpeedUnit);

            string sAccUnit = "";
            for (int j = 0; j < aszSpeedUnit.Length; j++)
            {
                sAccUnit += Convert.ToChar(aszSpeedUnit[j]);
            }

            this.lblUnit1.Text = sAccUnit;
            this.lblUnit2.Text = sAccUnit;
            this.lblUnit3.Text = sAccUnit;
            this.lblUnit4.Text = sAccUnit;
        }
    }
}
