﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Shared;
using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace PositionCompare
{
    public partial class PositionCompare : Form
    {
        public enum _TCmpSource { ONE_POINT_TRIGGER = 0, REGULAR_INTERVAL_POINT = 1, VOLUNTARY_INTERVAL_POINT = 2 };

        int m_nCmpSource = (int)_TCmpSource.ONE_POINT_TRIGGER;
        
        public PositionCompare()
        {
            InitializeComponent();
        }

        // common function
        void InitMotionDevices()
        {
            ////////////////////////////////////////////////////////////////////////////////////////
            // [CAUTION]: 아래와 다른 CME파일을 사용하는 경우에는 아래 코드를 수정하여 올바른 CME 파일의
            // 경로를 지정하십시오.
            const string CME_FILE_NAME = "Default.cme2";

            string szCmeFilePath, szSystemDir;

            szSystemDir = Environment.SystemDirectory; // 윈도우즈 시스템 디렉토리명 얻어오기 (일반적으로는 C:\Windows\System32)
            szCmeFilePath = szSystemDir + "\\" + CME_FILE_NAME;

            if (CMDLL.cmmGnInitFromFile(szCmeFilePath) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(IntPtr.Zero); // 에러 발생하였으면 에러 디스플레이.
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////////////
                // CME 파일에서 정의한 축수와 실제 장착되어 있는 모션 축수를 비교확인 하고자할 때는 
                // 아래와 같이 하면 됩니다.

                int nNumAxes_installed = 0;
                int nNumAxes_defined = 0;

                CMDLL.cmmAdvGetNumAvailAxes(ref nNumAxes_installed); // 현재 실제로 PC에 장착되어 있는 모션축 수 (cmmGnDeviceLoad() 함수에서 반환하는 축수와 동일)
                CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes_defined); // CME 파일에서 정의한 축 수(CME 파일이 지정되지 않는 경우에는 현재 장착된 축 수와 일치합니다.)

                if (nNumAxes_installed != nNumAxes_defined)
                {
                    MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }

            // SERVO-ON //
            int nNumAxes = 0;
            CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes);
            for (int i = 0; i < nNumAxes; i++)
            {
                CMDLL.cmmGnSetServoOn(i, (int)Defines._TCmBool.cmTRUE);
            }
        }

        private void PositionCompare_Load(object sender, EventArgs e)
        {
            // Load Motion & DIO devices //
            int nNumAxes=0;
            if (CMDLL.cmmGnDeviceLoad((int)Defines._TCmBool.cmTRUE, ref nNumAxes) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle); // 에러 발생하였으면 에러 디스플레이합니다.
                return;
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////
                // 로드된 축수가 올바른지 체크: 본 예제에서는 원하는 축 수를 모르기 때문에 
                // 주석처리 해놓았습니다.

                //				const int NUM_AXES_TO_LOAD = 4; // 이 값을 시스템에서 필요한 축 수에 맞게 조정 
                //				if(anNumAxes[0] < NUM_AXES_TO_LOAD){
                //					MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                //				}

                // Motion 환경 초기화 //
                InitMotionDevices();

                ////////////////////////////////////////////////////////////////////////////////////
                // 축설정 콤보박스에 축리스트를 채워넣습니다. 
                byte[] aszAxisTitle = new byte[Defines.CMM_MAX_STR_LEN_AXIS_TITLE];

                cmbAxis.Items.Clear(); // Clear combobox
                string szTitle;

                for (int i = 0; i < nNumAxes; i++)
                {
                    CMDLL.cmmAdvGetString(i, (int)Defines._TCmStringID.cmSTR_AXIS_NAME, aszAxisTitle); // CME 빌더에서 정의한 각 축의 타이틀을 가져오는 함수.

                    szTitle = "";
                    for (int j = 0; j < aszAxisTitle.Length; j++)
                    {
                        szTitle += Convert.ToChar(aszAxisTitle[j]);
                    }

                    cmbAxis.Items.Add(szTitle);
                }

                if (cmbAxis.Items.Count > 0)
                    cmbAxis.SelectedIndex = 0;

                this.cmbCmpSource.SelectedIndex = 0;
                this.cmbMethod1.SelectedIndex = 0;
                this.cmbMethod2.SelectedIndex = 0;
                this.cmbMethod3.SelectedIndex = 0;

                OnChangePositionCompareConf(this.rdoOnePoint, null);
            }		
        }

        private void OnChangePositionCompareConf(object sender, EventArgs e)
        {
            if (sender == rdoOnePoint)
            {
                // One Point Trigger
                this.lblMethod1.Enabled = true;
                this.cmbMethod1.Enabled = true;
                this.lblCmpPoint.Enabled = true;
                this.txtCmpPoint.Enabled = true;
                // Regular Interval Point
                this.lblMethod2.Enabled = false;
                this.cmbMethod2.Enabled = false;
                this.lblStartPoint.Enabled = false;
                this.txtStartPoint.Enabled = false;
                this.lblPointInt.Enabled = false;
                this.txtPointInt.Enabled = false;
                this.lblNumPoint.Enabled = false;
                this.txtNumPoint.Enabled = false;
                // Voluntary Interval Point
                this.lblMethod3.Enabled = false;
                this.cmbMethod3.Enabled = false;
                this.lblPosition.Enabled = false;
                this.txtPosition.Enabled = false;
                this.btnAddItem.Enabled = false;
                this.lstVoluntary.Enabled = false;
                this.lblVoluntary.Enabled = false;

                m_nCmpSource = (int)_TCmpSource.ONE_POINT_TRIGGER;
            }
            else if (sender == rdoRegularPoint)
            {
                // One Point Trigger
                this.lblMethod1.Enabled = false;
                this.cmbMethod1.Enabled = false;
                this.lblCmpPoint.Enabled = false;
                this.txtCmpPoint.Enabled = false;
                // Regular Interval Point
                this.lblMethod2.Enabled = true;
                this.cmbMethod2.Enabled = true;
                this.lblStartPoint.Enabled = true;
                this.txtStartPoint.Enabled = true;
                this.lblPointInt.Enabled = true;
                this.txtPointInt.Enabled = true;
                this.lblNumPoint.Enabled = true;
                this.txtNumPoint.Enabled = true;
                // Voluntary Interval Point
                this.lblMethod3.Enabled = false;
                this.cmbMethod3.Enabled = false;
                this.lblPosition.Enabled = false;
                this.txtPosition.Enabled = false;
                this.btnAddItem.Enabled = false;
                this.lstVoluntary.Enabled = false;
                this.lblVoluntary.Enabled = false;

                m_nCmpSource = (int)_TCmpSource.REGULAR_INTERVAL_POINT;
            }
            else if (sender == rdoVoluntaryPoint)
            {
                // One Point Trigger
                this.lblMethod1.Enabled = false;
                this.cmbMethod1.Enabled = false;
                this.lblCmpPoint.Enabled = false;
                this.txtCmpPoint.Enabled = false;
                // Regular Interval Point
                this.lblMethod2.Enabled = false;
                this.cmbMethod2.Enabled = false;
                this.lblStartPoint.Enabled = false;
                this.txtStartPoint.Enabled = false;
                this.lblPointInt.Enabled = false;
                this.txtPointInt.Enabled = false;
                this.lblNumPoint.Enabled = false;
                this.txtNumPoint.Enabled = false;
                // Voluntary Interval Point
                this.lblMethod3.Enabled = true;
                this.cmbMethod3.Enabled = true;
                this.lblPosition.Enabled = true;
                this.txtPosition.Enabled = true;
                this.btnAddItem.Enabled = true;
                this.lstVoluntary.Enabled = true;
                this.lblVoluntary.Enabled = true;

                m_nCmpSource = (int)_TCmpSource.VOLUNTARY_INTERVAL_POINT;
            }
        }

        private void btnAddItem_Click(object sender, EventArgs e)
        {
            if (this.txtPosition.Text == null)
            {
                MessageBox.Show("Position 을 입력하세요.");
                return;
            }

            this.lstVoluntary.Items.Add(this.txtPosition.Text);
        }

        private void lstVoluntary_DoubleClick(object sender, EventArgs e)
        {
            if (this.lstVoluntary.Items.Count > 0)
            {
                int nRow = this.lstVoluntary.SelectedIndex;
                this.lstVoluntary.Items.RemoveAt(nRow);
            }
        }

        private void btnSetCmp_Click(object sender, EventArgs e)
        {
            this.btnSetCmp.Enabled = false;

            int nAxis = this.cmbAxis.SelectedIndex;
            int nCmpSource = this.cmbCmpSource.SelectedIndex;
            int nCmpMethod = 0;
            switch (m_nCmpSource)
            {
                case (int)_TCmpSource.ONE_POINT_TRIGGER :
                    nCmpMethod = this.cmbMethod1.SelectedIndex;
                    double fCmpPoint = Convert.ToDouble(this.txtCmpPoint.Text);
                    CMDLL.cmmCmpTrgSetConfig(nAxis, nCmpSource, nCmpMethod);
                    CMDLL.cmmCmpTrgSetOneData(nAxis, fCmpPoint);
                    break;

                case (int)_TCmpSource.REGULAR_INTERVAL_POINT:
                    nCmpMethod = this.cmbMethod2.SelectedIndex;
                    double fStartPoint = Convert.ToDouble(this.txtStartPoint.Text);
                    double fPointInt = Convert.ToDouble(this.txtPointInt.Text);
                    int nNumPoint = int.Parse(this.txtNumPoint.Text);
                    CMDLL.cmmCmpTrgSetConfig(nAxis, nCmpSource, nCmpMethod);
                    CMDLL.cmmCmpTrgContBuildTable(nAxis, fStartPoint, fPointInt, nNumPoint);
                    CMDLL.cmmCmpTrgContStart(nAxis);
                    break;

                case (int)_TCmpSource.VOLUNTARY_INTERVAL_POINT:
                    nCmpMethod = this.cmbMethod3.SelectedIndex;
                    int nListCount = this.lstVoluntary.Items.Count;
                    double[] fListTable = new double[nListCount];
                    if (nListCount <= 0)
                    {
                        MessageBox.Show("Item list is now empty");
                        this.btnSetCmp.Enabled = true;
                        return;
                    }
                    for (int i = 0; i < nListCount; i++ )
                    {
                        fListTable[i] = Convert.ToDouble(this.lstVoluntary.Items[i].ToString());
                    }
                    CMDLL.cmmCmpTrgSetConfig(nAxis, nCmpSource, nCmpMethod);
                    CMDLL.cmmCmpTrgContRegTable(nAxis, fListTable, nListCount);
                    CMDLL.cmmCmpTrgContStart(nAxis);
                    break;
            }

            MessageBox.Show("Now you can run Motion Builder V3 and move the CMP-SET Axis, Using Interrupt Event function you can check if CMP has triggered or not");
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;

            CMDLL.cmmCmpTrgContStop(nAxis);

            this.btnSetCmp.Enabled = true;
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nStatus = 0;

            if (CMDLL.cmmStReadMotionState(nAxis, ref nStatus) != (int)Defines._TCmMotionState.cmMST_STOP)
            {
                CMDLL.cmmSxStop(nAxis, (int)Defines._TCmBool.cmTRUE, (int)Defines._TCmBool.cmFALSE);
            }

            this.Close();
        }

        private void PositionCompare_FormClosing(object sender, FormClosingEventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nStatus = 0;

            if (CMDLL.cmmStReadMotionState(nAxis, ref nStatus) != (int)Defines._TCmMotionState.cmMST_STOP)
            {
                CMDLL.cmmSxStop(nAxis, (int)Defines._TCmBool.cmTRUE, (int)Defines._TCmBool.cmFALSE);
            }

            CMDLL.cmmGnDeviceUnload(); // Unload Device
        }
    }
}
