﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using Shared;

using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace Logging
{
    public partial class Logging : Form
    {
        bool g_bThreadStop = true;

        int g_nDir = (int)Defines._TCmDir.cmDIR_N;
        int g_nAxis = 0;
        int g_nSpeedMode = 0;
        int g_uIntv = 0;
        string g_sLogMsg1 = "";
        string g_sLogMsg2 = "";
        int g_nRunCount = 0;
        bool g_bNewLog = false;
        
        double g_fWork = 0, g_fAcc = 0, g_fDec = 0, g_fDist = 0;

        private int ERR_BUF_SIZE = 100;
        private int MAX_HISTORY_NUM = 256;
        
        public Logging()
        {
            InitializeComponent();
        }

        // common function
        void InitMotionDevices()
        {
            ////////////////////////////////////////////////////////////////////////////////////////
            // [CAUTION]: 아래와 다른 CME파일을 사용하는 경우에는 아래 코드를 수정하여 올바른 CME 파일의
            // 경로를 지정하십시오.
            const string CME_FILE_NAME = "Default.cme2";

            string szCmeFilePath, szSystemDir;

            szSystemDir = Environment.SystemDirectory; // 윈도우즈 시스템 디렉토리명 얻어오기 (일반적으로는 C:\Windows\System32)
            szCmeFilePath = szSystemDir + "\\" + CME_FILE_NAME;

            if (CMDLL.cmmGnInitFromFile(szCmeFilePath) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(IntPtr.Zero); // 에러 발생하였으면 에러 디스플레이.
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////////////
                // CME 파일에서 정의한 축수와 실제 장착되어 있는 모션 축수를 비교확인 하고자할 때는 
                // 아래와 같이 하면 됩니다.

                int nNumAxes_installed = 0;
                int nNumAxes_defined = 0;

                CMDLL.cmmAdvGetNumAvailAxes(ref nNumAxes_installed); // 현재 실제로 PC에 장착되어 있는 모션축 수 (cmmGnDeviceLoad() 함수에서 반환하는 축수와 동일)
                CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes_defined); // CME 파일에서 정의한 축 수(CME 파일이 지정되지 않는 경우에는 현재 장착된 축 수와 일치합니다.)

                if (nNumAxes_installed != nNumAxes_defined)
                {
                    MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }

            // SERVO-ON //
            int nNumAxes = 0;
            CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes);
            for (int i = 0; i < nNumAxes; i++)
            {
                CMDLL.cmmGnSetServoOn(i, (int)Defines._TCmBool.cmTRUE);
            }
        }

        private void Logging_Load(object sender, EventArgs e)
        {
            // Load Motion & DIO devices //
            int nNumAxes = 0;
            if (CMDLL.cmmGnDeviceLoad((int)Defines._TCmBool.cmTRUE, ref nNumAxes) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle); // 에러 발생하였으면 에러 디스플레이합니다.
                return;
            }
            else
            {
                // Motion 환경 초기화 //
                InitMotionDevices();

                //////////////////////////////////////////////////////////////////////////
                // 사용자 인터페이스를 초기화 합니다.
                //////////////////////////////////////////////////////////////////////////
                InitializeUI();
                //////////////////////////////////////////////////////////////////////////

                // Set timer to read and display current command position //
                this.timer1.Start();
            }
        }

        void InitializeUI()
        {
            // 사용 가능한 축의 갯수 얻어오기
            int anNumAxesToDisp = 0;
            CMDLL.cmmAdvGetNumAvailAxes(ref anNumAxesToDisp);
            int nNumAxes = anNumAxesToDisp;

            for (int i = 0; i < nNumAxes; i++)
            {
                this.cmbAxis.Items.Insert(i, "Axis" + i.ToString());
            }
            this.cmbAxis.SelectedIndex = 0;

            this.cmbSpeedMode.SelectedIndex = (int)Defines._TCmSpeedMode.cmSMODE_S;

            this.chkRepeat.Checked = true;

            this.txtWork.Text = "100000";
            this.txtAcc.Text = "1000000";
            this.txtDec.Text = "1000000";
            this.txtDist.Text = "10000";
            this.txtIntv.Text = "500";

            this.chkErrLogging.Checked = true;
            this.chkPosLogging.Checked = true;

            this.txtLogPath.Text = "C:\\COMILOG.txt";

            this.chkFileLogging.Checked = true;
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            //------- 모션제어기 위치 모니터링 -------------------------------//
            int nAxis = this.cmbAxis.SelectedIndex;
            string sPos;
            double afCurPos = 0;

            // Command position display //
            CMDLL.cmmStGetPosition(nAxis, (int)Defines._TCmCntr.cmCNT_COMM, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.txtCurPos.Text = sPos;

            if (g_bNewLog)
            {
                g_bNewLog = false;

                this.lstScreenLog.Items.Add(g_sLogMsg1);
                this.lstScreenLog.Items.Add(g_sLogMsg2);
                this.lstScreenLog.SelectedIndex = this.lstScreenLog.Items.Count - 1; // 항상 최종 추가된 아이템을 가리키도록 합니다.

                if (g_nRunCount > MAX_HISTORY_NUM) // 스크린 로그용 리스트 박스의 히스토리가 일정량 쌓이면 주기적으로 삭제시켜줍니다.
                {
                    this.lstScreenLog.Items.Clear();
                }
            }
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            this.lstScreenLog.Items.Clear();
        }

        private void btnResetPos_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;

            CMDLL.cmmStSetPosition(nAxis, (int)Defines._TCmCntr.cmCNT_COMM, 0.0); // Reset COMMAND Position
        }

        private void btnMoveN_Click(object sender, EventArgs e)
        {
            this.btnMoveN.Enabled = false;
            this.btnMoveP.Enabled = false;
            
            g_nDir = (int)Defines._TCmDir.cmDIR_N;
            g_bThreadStop = false;
            // 쓰레드 동작을 시작한다.
            SetMotionValues();
            g_nRunCount = 0;
            new Thread(new ThreadStart(ThSxMove)).Start();
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            this.btnMoveN.Enabled = true;
            this.btnMoveP.Enabled = true;
            
            g_bThreadStop = true;
        }

        private void btnMoveP_Click(object sender, EventArgs e)
        {
            this.btnMoveN.Enabled = false;
            this.btnMoveP.Enabled = false;
            
            g_nDir = (int)Defines._TCmDir.cmDIR_P;
            g_bThreadStop = false;
            // 쓰레드 동작을 시작한다.
            SetMotionValues();
            g_nRunCount = 0;
            new Thread(new ThreadStart(ThSxMove)).Start();
        }

        void ThSxMove()
        {
            double fCurDist = 0, fPos = 0;
            string sLogBuf = "";
            //Buffer[] sBuf = new Buffer[ERR_BUF_SIZE];
            
            while(!g_bThreadStop)
            {
                // "Move (-)"버튼이면 Distance에 음수를 취하고, "Move (+)" 버튼이면 양수를 취한다 //
                fCurDist = g_fDist * ((g_nDir == (int)Defines._TCmDir.cmDIR_P) ? 1 : -1);
                if ((g_nRunCount%2) == 1)
                {
                    fCurDist *= -1;
                }
                CMDLL.cmmSxMove(g_nAxis, fCurDist, (int)Defines._TCmBool.cmTRUE);
                Thread.Sleep(g_uIntv);

                g_nRunCount++;
                if (this.chkFileLogging.Checked)
                {
                    DateTime currTime = DateTime.Now;
                    
                    // File Stream
                    string sFile = "";
                    FileStream fs = null;
                    sFile = "C:\\COMILOG_" + currTime.ToString("MM") + currTime.ToString("dd") + ".txt";
                    if(!File.Exists(sFile))
                    {
                        fs = new FileStream(sFile, FileMode.Create); // 새파일 생성
                    }
                    else
                    {
                        fs = new FileStream(sFile, FileMode.Open); // 파일 열기
                    }
                    StreamReader reader = new StreamReader(fs);
                    StreamWriter writer = new StreamWriter(fs, System.Text.Encoding.Unicode);
                    string strRead;
                    
                    if (this.chkErrLogging.Checked)
                    {
                        CMDLL.cmmErrGetString(g_nAxis, ref sLogBuf, ERR_BUF_SIZE);
                        g_sLogMsg1 = "Axis" + g_nAxis + " 에러스트링 : " + sLogBuf;
                        strRead = reader.ReadToEnd(); // 항상 마지막 line 에 위치하도록 한다
                        writer.WriteLine(g_sLogMsg1);
                        g_bNewLog = true;
                    }

                    if (this.chkPosLogging.Checked)
                    {
                        CMDLL.cmmStGetPosition(g_nAxis, (int)Defines._TCmCntr.cmCNT_COMM, ref fPos);
                        g_sLogMsg2 = currTime.ToString("MM") + "월"
                            + currTime.ToString("dd") + "일 "
                            + currTime.ToString("HH:mm:ss")
                            + " => Axis" + g_nAxis
                            + " 지령펄스 위치 : " + fPos.ToString();
                        strRead = reader.ReadToEnd(); // 항상 마지막 line 에 위치하도록 한다
                        writer.WriteLine(g_sLogMsg2);
                        g_bNewLog = true;
                    }

                    writer.Close();
                }

                if (!this.chkRepeat.Checked)
                {
                    g_bThreadStop = true;
                    break;
                }

                Thread.Sleep(g_uIntv);
            }
        }

        void SetMotionValues()
        {
            g_nAxis = this.cmbAxis.SelectedIndex;

            g_fWork = Convert.ToDouble(this.txtWork.Text);
            g_fAcc = Convert.ToDouble(this.txtAcc.Text);
            g_fDec = Convert.ToDouble(this.txtDec.Text);
            g_fDist = Convert.ToDouble(this.txtDist.Text);
            g_uIntv = int.Parse(this.txtIntv.Text);

            g_nSpeedMode = this.cmbSpeedMode.SelectedIndex;

            CMDLL.cmmCfgSetSpeedRange(g_nAxis, 6553500);
            CMDLL.cmmCfgSetSpeedPattern(g_nAxis, g_nSpeedMode, g_fWork, g_fAcc, g_fDec);
        }

        private void Logging_FormClosing(object sender, FormClosingEventArgs e)
        {
            // 사용중인 타이머 종료.
            timer1.Stop();

            CMDLL.cmmGnDeviceUnload(); // Unload Device
        }
    }
}
