﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Shared;
using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace PositionLatch
{
    public partial class PositionLatchForm : Form
    {
        // delegate
        delegate int delGetAxis();
        delegate int delLatchSource();
        delegate void delChkIsIntLatched(bool bVal);
        delegate void delLtcValue(string sVal);

        public PositionLatchForm()
        {
            InitializeComponent();
        }

        // common function
        void InitMotionDevices()
        {
            ////////////////////////////////////////////////////////////////////////////////////////
            // [CAUTION]: 아래와 다른 CME파일을 사용하는 경우에는 아래 코드를 수정하여 올바른 CME 파일의
            // 경로를 지정하십시오.
            const string CME_FILE_NAME = "Default.cme2";

            string szCmeFilePath, szSystemDir;

            szSystemDir = Environment.SystemDirectory; // 윈도우즈 시스템 디렉토리명 얻어오기 (일반적으로는 C:\Windows\System32)
            szCmeFilePath = szSystemDir + "\\" + CME_FILE_NAME;

            if (CMDLL.cmmGnInitFromFile(szCmeFilePath) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(IntPtr.Zero); // 에러 발생하였으면 에러 디스플레이.
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////////////
                // CME 파일에서 정의한 축수와 실제 장착되어 있는 모션 축수를 비교확인 하고자할 때는 
                // 아래와 같이 하면 됩니다.

                int nNumAxes_installed = 0;
                int nNumAxes_defined = 0;

                CMDLL.cmmAdvGetNumAvailAxes(ref nNumAxes_installed); // 현재 실제로 PC에 장착되어 있는 모션축 수 (cmmGnDeviceLoad() 함수에서 반환하는 축수와 동일)
                CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes_defined); // CME 파일에서 정의한 축 수(CME 파일이 지정되지 않는 경우에는 현재 장착된 축 수와 일치합니다.)

                if (nNumAxes_installed != nNumAxes_defined)
                {
                    MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }

            // SERVO-ON //
            int nNumAxes = 0;
            CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes);
            for (int i = 0; i < nNumAxes; i++)
            {
                CMDLL.cmmGnSetServoOn(i, (int)Defines._TCmBool.cmTRUE);
            }
        }

        private void PositionLatch_Load(object sender, EventArgs e)
        {
            // Load Motion & DIO devices //
            int nNumAxes = 0;
            if (CMDLL.cmmGnDeviceLoad((int)Defines._TCmBool.cmTRUE, ref nNumAxes) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle); // 에러 발생하였으면 에러 디스플레이합니다.
                return;
            }
            else
            {
                // Motion 환경 초기화 //
                InitMotionDevices();

                // 사용 가능한 축의 갯수 얻어오기
                int anNumAxesToDisp = 0;
                CMDLL.cmmAdvGetNumAvailAxes(ref anNumAxesToDisp);
                nNumAxes = anNumAxesToDisp;

                for (int i = 0; i < nNumAxes; i++)
                {
                    this.cmbAxis.Items.Insert(i, "Axis" + i.ToString());
                }
                this.cmbAxis.SelectedIndex = 0;

                cmbAxis_SelectedIndexChanged(null, null);

                // Set timer to read and display current command position //
                this.timer1.Start();
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nLatchSource = this.cmbLtcSource.SelectedIndex;
            int nIsLatched = 0;
            double fLatchCount = 0;

            CMDLL.cmmLtcIsLatched(nAxis, ref nIsLatched);

            if (nIsLatched == 1)
            {
                CMDLL.cmmLtcReadLatch(nAxis, nLatchSource, ref fLatchCount);
                this.txtLtcVal.Text = fLatchCount.ToString();

                this.chkIsReadLatched.Checked = true;
            }
            else
            {
                this.chkIsReadLatched.Checked = false;
            }
        }

        private void cmbLtcLogic_SelectedIndexChanged(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nLatchLogic = this.cmbLtcLogic.SelectedIndex;

            CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmLTC_LOGIC, nLatchLogic);
        }

        private void chkLtcMask_CheckedChanged(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nIntMask = 0x0;
            if(this.chkLtcMask.Checked)
            {
                nIntMask |= (1 << 14); // BIT14 (zero-based) 값이 Latch 에 대한 Mask 값이다.
            }
            else
            {
                nIntMask &= ~(1 << 14); // BIT14 (zero-based) 값이 Latch 에 대한 Mask 값이다.
            }

            CMDLL.cmmIntSetMask(nAxis, nIntMask);
            CMDLL.cmmIntHandlerSetup_CLB((int)Defines._TCmIntHandlerType.cmIHT_CALLBACK, InterruptCallback, 0, this.Handle);
            CMDLL.cmmIntHandlerEnable((int)Defines._TCmBool.cmTRUE);
        }

        /***********************************************************************************************
        * CALLBACK 방식 처리 
        ************************************************************************************************/
        void InterruptCallback(IntPtr lParam)
        {
            //////////////////////////////////////////////////////////////////////////////////////////
            // 주의: 쓰레드내에서 GUI 관련작업을 수행하면 에러를 발생시키는 것과 마찬가지로 CALLBACK
            // 함수내에서 GUI 관련작업을 수행하면 예기치않은 에러를 발생시킬 수 있으므로, 가급적 
            // GUI 관련작업은 피하는 것이 좋다. 단, GUI의 속성을 참조하는 것은 상관없다.
            InterruptHandlerCommon(lParam);
        }

        /***********************************************************************************************
        * InterrHandlerCommon(): 각 Event Handler에서 공통으로 호출하는 함수. 
        ************************************************************************************************/
        void InterruptHandlerCommon(IntPtr lParam)
        {
            int nEvtState = 0;
            double fLatchCount = 0;
            int nIsIntLatched = 0; // 인터럽트 이벤트로 래치 발생 여부

            int nAxis = GetAxis();
            int nLatchSource = GetLatchSource();

            CMDLL.cmmIntReadEventStatus(nAxis, ref nEvtState);
            nIsIntLatched = ((nEvtState >> 14) & 0x1); // BIT14 (zero-based) 값이 Latch 에 대한 Mask 값이다.
            if (nIsIntLatched == 1)
            {
                SetCheckIsIntLatched(true);

                CMDLL.cmmLtcReadLatch(nAxis, nLatchSource, ref fLatchCount);
                SetLtcValue(fLatchCount.ToString());
            }
            else
            {
                SetCheckIsIntLatched(false);
            }
        }

        int GetAxis()
        {
            if (this.cmbAxis.InvokeRequired)
            {
                delGetAxis d = new delGetAxis(GetAxis);
                return (int)this.Invoke(d, new object[] {  });
            }
            else
            {
                return this.cmbAxis.SelectedIndex;
            }
        }

        int GetLatchSource()
        {
            if (this.cmbLtcSource.InvokeRequired)
            {
                delLatchSource d = new delLatchSource(GetLatchSource);
                return (int)this.Invoke(d, new object[] { });
            }
            else
            {
                return this.cmbLtcSource.SelectedIndex;
            }
        }

        void SetCheckIsIntLatched(bool bVal)
        {
            if (this.chkIsIntLatched.InvokeRequired)
            {
                delChkIsIntLatched d = new delChkIsIntLatched(SetCheckIsIntLatched);
                this.Invoke(d, new object[] { bVal });
            }
            else
            {
                this.chkIsIntLatched.Checked = bVal;
            }
        }

        void SetLtcValue(string sVal)
        {
            if (this.txtLtcVal.InvokeRequired)
            {
                delLtcValue d = new delLtcValue(SetLtcValue);
                this.Invoke(d, new object[] { sVal });
            }
            else
            {
                this.txtLtcVal.Text = sVal;
            }
        }

        private void cmbAxis_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.cmbLtcSource.SelectedIndex = 0;
            this.cmbLtcLogic.SelectedIndex = 0;

            this.txtLtcVal.Text = "(래치안됨)";

            this.chkLtcMask.Checked = false;
            this.chkIsIntLatched.Checked = false;
            this.chkIsReadLatched.Checked = false;
        }
    }
}
