﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Shared;
using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace StateMonitor
{
    public partial class StateMonitor : Form
    {
        public StateMonitor()
        {
            InitializeComponent();
        }

        // common function
        void InitMotionDevices()
        {
            ////////////////////////////////////////////////////////////////////////////////////////
            // [CAUTION]: 아래와 다른 CME파일을 사용하는 경우에는 아래 코드를 수정하여 올바른 CME 파일의
            // 경로를 지정하십시오.
            const string CME_FILE_NAME = "Default.cme2";

            string szCmeFilePath, szSystemDir;

            szSystemDir = Environment.SystemDirectory; // 윈도우즈 시스템 디렉토리명 얻어오기 (일반적으로는 C:\Windows\System32)
            szCmeFilePath = szSystemDir + "\\" + CME_FILE_NAME;

            if (CMDLL.cmmGnInitFromFile(szCmeFilePath) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(IntPtr.Zero); // 에러 발생하였으면 에러 디스플레이.
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////////////
                // CME 파일에서 정의한 축수와 실제 장착되어 있는 모션 축수를 비교확인 하고자할 때는 
                // 아래와 같이 하면 됩니다.

                int anNumAxes_installed = 0;
                int anNumAxes_defined = 0;

                CMDLL.cmmAdvGetNumAvailAxes(ref anNumAxes_installed); // 현재 실제로 PC에 장착되어 있는 모션축 수 (cmmGnDeviceLoad() 함수에서 반환하는 축수와 동일)
                CMDLL.cmmAdvGetNumDefinedAxes(ref anNumAxes_defined); // CME 파일에서 정의한 축 수(CME 파일이 지정되지 않는 경우에는 현재 장착된 축 수와 일치합니다.)

                if (anNumAxes_installed != anNumAxes_defined)
                {
                    MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }

        }

        private void StateMonitor_Load(object sender, EventArgs e)
        {
            // Load Motion & DIO devices //
            int anNumAxes = 0;
            if (CMDLL.cmmGnDeviceLoad((int)Defines._TCmBool.cmTRUE, ref anNumAxes) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle); // 에러 발생하였으면 에러 디스플레이합니다.
                return;
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////
                // 로드된 축수가 올바른지 체크: 본 예제에서는 원하는 축 수를 모르기 때문에 
                // 주석처리 해놓았습니다.

                //				const int NUM_AXES_TO_LOAD = 4; // 이 값을 시스템에서 필요한 축 수에 맞게 조정 
                //				if(anNumAxes[0] < NUM_AXES_TO_LOAD){
                //					MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                //				}

                // Motion 환경 초기화 //
                InitMotionDevices();

                ////////////////////////////////////////////////////////////////////////////////////
                int i;

                // 사용 가능한 축의 갯수 얻어오기
                int anNumAxesToDisp = 0;
                CMDLL.cmmAdvGetNumAvailAxes(ref anNumAxesToDisp);
                int nNumAxes = anNumAxesToDisp;

                for (i = 0; i < nNumAxes; i++)
                {
                    this.cmbAxis.Items.Insert(i, "Axis" + i.ToString());
                }

                // 축선택의 기본값으로는 0번과 1번 두 축이 선택된 것으로 한다.
                this.cmbAxis.SelectedIndex = 0;

                GetLogicState();
                GetModeState();

                // Set timer to read and display current command position //
                this.timer1.Start();
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int dwTemp = 0;
            string sPos;
            double afCurPos = 0;

            // Servo ON/OFF 상태 디스플레이 //
            CMDLL.cmmGnGetServoOn(nAxis, ref dwTemp);
            if (dwTemp == (int)Defines._TCmBool.cmFALSE)
            {
                this.chkServoOn.Checked = false;
            }
            else
            {
                this.chkServoOn.Checked = true;
            }

            // Motion State Display //
            byte[] szMotState = new byte[(int)Defines.CMM_MAX_STR_LEN_ERR];
            string szTitle;
            CMDLL.cmmStReadMotionState(nAxis, ref dwTemp);
            CMDLL.cmmStGetMstString(dwTemp, szMotState, (int)Defines.CMM_MAX_STR_LEN_ERR);

            szTitle = "";
            for (int j = 0; j < szMotState.Length; j++)
            {
                szTitle += Convert.ToChar(szMotState[j]);
            }

            this.txtMotionState.Text = szTitle;

            // Command position display //
            CMDLL.cmmStGetPosition(nAxis, (int)Defines._TCmCntr.cmCNT_COMM, ref afCurPos);
            sPos = afCurPos.ToString();
            this.txtCmdPos.Text = sPos;

            // Feedback position display //
            CMDLL.cmmStGetPosition(nAxis, (int)Defines._TCmCntr.cmCNT_FEED, ref afCurPos);
            sPos = afCurPos.ToString();
            this.txtCmdFeed.Text = sPos;

            // Command velocity display //
            CMDLL.cmmStGetSpeed(nAxis, (int)Defines._TCmCntr.cmCNT_COMM, ref afCurPos);
            sPos = afCurPos.ToString();
            this.txtCmdSpd.Text = sPos;

            // Feedback velocity display //
            CMDLL.cmmStGetSpeed(nAxis, (int)Defines._TCmCntr.cmCNT_FEED, ref afCurPos);
            sPos = afCurPos.ToString();
            this.txtSpdFeed.Text = sPos;

            // MIO Display //
            int nMioState = 0;
            int nMioBitResult = 0;
            int nMioShiftResult = 0;
            
            CMDLL.cmmStReadMioStatuses(nAxis, ref nMioState); // Read all MIO states

            nMioShiftResult = (nMioState >> (int)Defines._TCmMioState.cmIOST_DIR); // DIR
            nMioBitResult = nMioShiftResult & 1;
            if (nMioBitResult == 1)
            {
                this.pbxDIR.BackColor = Color.Lime;
            }
            else
            {
                this.pbxDIR.BackColor = Color.Silver;
            }

            nMioShiftResult = (nMioState >> (int)Defines._TCmMioState.cmIOST_ALM); // ALM
            nMioBitResult = nMioShiftResult & 1;
            if (nMioBitResult == 1)
            {
                this.pbxALM.BackColor = Color.Red;
            }
            else
            {
                this.pbxALM.BackColor = Color.Silver;
            }

            nMioShiftResult = (nMioState >> (int)Defines._TCmMioState.cmIOST_EZ); // EZ
            nMioBitResult = nMioShiftResult & 1;
            if (nMioBitResult == 1)
            {
                this.pbxEZ.BackColor = Color.Lime;
            }
            else
            {
                this.pbxEZ.BackColor = Color.Silver;
            }

            nMioShiftResult = (nMioState >> (int)Defines._TCmMioState.cmIOST_INP); // INP
            nMioBitResult = nMioShiftResult & 1;
            if (nMioBitResult == 1)
            {
                this.pbxINP.BackColor = Color.Lime;
            }
            else
            {
                this.pbxINP.BackColor = Color.Silver;
            }

            nMioShiftResult = (nMioState >> (int)Defines._TCmMioState.cmIOST_ELN); // -EL
            nMioBitResult = nMioShiftResult & 1;
            if (nMioBitResult == 1)
            {
                this.pbxELN.BackColor = Color.Red;
            }
            else
            {
                this.pbxELN.BackColor = Color.Silver;
            }

            nMioShiftResult = (nMioState >> (int)Defines._TCmMioState.cmIOST_ELP); // +EL
            nMioBitResult = nMioShiftResult & 1;
            if (nMioBitResult == 1)
            {
                this.pbxELP.BackColor = Color.Red;
            }
            else
            {
                this.pbxELP.BackColor = Color.Silver;
            }

            nMioShiftResult = (nMioState >> (int)Defines._TCmMioState.cmIOST_RDY); // RDY
            nMioBitResult = nMioShiftResult & 1;
            if (nMioBitResult == 1)
            {
                this.pbxRDY.BackColor = Color.Lime;
            }
            else
            {
                this.pbxRDY.BackColor = Color.Silver;
            }

            nMioShiftResult = (nMioState >> (int)Defines._TCmMioState.cmIOST_ORG); // ORG
            nMioBitResult = nMioShiftResult & 1;
            if (nMioBitResult == 1)
            {
                this.pbxORG.BackColor = Color.Lime;
            }
            else
            {
                this.pbxORG.BackColor = Color.Silver;
            }

            nMioShiftResult = (nMioState >> (int)Defines._TCmMioState.cmIOST_SD); // SD
            nMioBitResult = nMioShiftResult & 1;
            if (nMioBitResult == 1)
            {
                this.pbxSD.BackColor = Color.Lime;
            }
            else
            {
                this.pbxSD.BackColor = Color.Silver;
            }

            nMioShiftResult = (nMioState >> (int)Defines._TCmMioState.cmIOST_LTC); // LTC
            nMioBitResult = nMioShiftResult & 1;
            if (nMioBitResult == 1)
            {
                this.pbxLTC.BackColor = Color.Lime;
            }
            else
            {
                this.pbxLTC.BackColor = Color.Silver;
            }

            nMioShiftResult = (nMioState >> (int)Defines._TCmMioState.cmIOST_DRN); // -DR
            nMioBitResult = nMioShiftResult & 1;
            if (nMioBitResult == 1)
            {
                this.pbxDRN.BackColor = Color.Lime;
            }
            else
            {
                this.pbxDRN.BackColor = Color.Silver;
            }

            nMioShiftResult = (nMioState >> (int)Defines._TCmMioState.cmIOST_DRP); // +DR
            nMioBitResult = nMioShiftResult & 1;
            if (nMioBitResult == 1)
            {
                this.pbxDRP.BackColor = Color.Lime;
            }
            else
            {
                this.pbxDRP.BackColor = Color.Silver;
            }

            nMioShiftResult = (nMioState >> (int)Defines._TCmMioState.cmIOST_STA); // STA
            nMioBitResult = nMioShiftResult & 1;
            if (nMioBitResult == 1)
            {
                this.pbxSTA.BackColor = Color.Lime;
            }
            else
            {
                this.pbxSTA.BackColor = Color.Silver;
            }

            nMioShiftResult = (nMioState >> (int)Defines._TCmMioState.cmIOST_STP); // STP
            nMioBitResult = nMioShiftResult & 1;
            if (nMioBitResult == 1)
            {
                this.pbxSTP.BackColor = Color.Lime;
            }
            else
            {
                this.pbxSTP.BackColor = Color.Silver;
            }

            nMioShiftResult = (nMioState >> (int)Defines._TCmMioState.cmIOST_ERC); // ERC
            nMioBitResult = nMioShiftResult & 1;
            if (nMioBitResult == 1)
            {
                this.pbxERC.BackColor = Color.Lime;
            }
            else
            {
                this.pbxERC.BackColor = Color.Silver;
            }

            nMioShiftResult = (nMioState >> (int)Defines._TCmMioState.cmIOST_CLR); // CLR
            nMioBitResult = nMioShiftResult & 1;
            if (nMioBitResult == 1)
            {
                this.pbxCLR.BackColor = Color.Lime;
            }
            else
            {
                this.pbxCLR.BackColor = Color.Silver;
            }

            nMioShiftResult = (nMioState >> (int)Defines._TCmMioState.cmIOST_PCS); // PCS
            nMioBitResult = nMioShiftResult & 1;
            if (nMioBitResult == 1)
            {
                this.pbxPCS.BackColor = Color.Lime;
            }
            else
            {
                this.pbxPCS.BackColor = Color.Silver;
            }
        }

        // Logic의 상태를 받아온다.
        public void GetLogicState()
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nProVal = 0;

            CMDLL.cmmCfgGetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmALM_LOGIC, ref nProVal); //ALM Logic
            this.cmbLogicALM.SelectedIndex = nProVal;

            CMDLL.cmmCfgGetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmEL_LOGIC, ref nProVal); //EL Logic
            this.cmbLogicEL.SelectedIndex = nProVal;

            CMDLL.cmmCfgGetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmERC_LOGIC, ref nProVal); //ERC Logic
            this.cmbLogicERC.SelectedIndex = nProVal;

            CMDLL.cmmCfgGetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmINP_LOGIC, ref nProVal); //INP Logic
            this.cmbLogicINP.SelectedIndex = nProVal;

            CMDLL.cmmCfgGetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmLTC_LOGIC, ref nProVal); //LTC Logic
            this.cmbLogicLTC.SelectedIndex = nProVal;

            CMDLL.cmmCfgGetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmORG_LOGIC, ref nProVal); //ORG Logic
            this.cmbLogicORG.SelectedIndex = nProVal;

            CMDLL.cmmCfgGetMioProperty(nAxis, (int)Defines._TCmMioPropIdEx.cmMPID_SVON_LOGIC, ref nProVal); //Servo On Logic
            this.cmbLogicSVON.SelectedIndex = nProVal;
        }

        // Mode의 상태를 받아온다.
        public void GetModeState()
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nProVal = 0;

            CMDLL.cmmCfgGetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmALM_MODE, ref nProVal); //ALM Mode
            this.cmbModeALM.SelectedIndex = nProVal;

            CMDLL.cmmCfgGetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmEL_MODE, ref nProVal); //EL Mode
            this.cmbModeEL.SelectedIndex = nProVal;

            CMDLL.cmmCfgGetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmERC_OUT, ref nProVal); //ERC Mode
            this.cmbModeERC.SelectedIndex = nProVal;

            CMDLL.cmmCfgGetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmINP_EN, ref nProVal); //INP Mode
            this.cmbModeINP.SelectedIndex = nProVal;

            CMDLL.cmmCfgGetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmSD_MODE, ref nProVal); //SD Mode
            this.cmbModeSD.SelectedIndex = nProVal;

            CMDLL.cmmCfgGetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmSTA_MODE, ref nProVal); //STA Mode
            this.cmbModeSTA.SelectedIndex = nProVal;

            CMDLL.cmmCfgGetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmSTP_MODE, ref nProVal); //STP Mode
            this.cmbModeSTP.SelectedIndex = nProVal;
        }

        /***********************************************************************
        * OnResetPos() : Position "Reset" 버튼의 콜백함수. Command & Feedback 
        * position을 0으로 리셋한다.
        ***********************************************************************/
        private void btnResetPos_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;

            CMDLL.cmmStSetPosition(nAxis, (int)Defines._TCmCntr.cmCNT_COMM, 0.0); // Reset command position of X-axis
            CMDLL.cmmStSetPosition(nAxis, (int)Defines._TCmCntr.cmCNT_FEED, 0.0); // Reset feedback position of X-axis
        }

        // Logic 의 상태를 변경한다
        private void SetLogicState(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nProVal = 0;

            if (sender == cmbLogicALM)
            {
                nProVal = this.cmbLogicALM.SelectedIndex;
                CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmALM_LOGIC, nProVal); //ALM Logic
            }
            else if (sender == cmbLogicEL)
            {
                nProVal = this.cmbLogicEL.SelectedIndex;
                CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmEL_LOGIC, nProVal); //EL Logic
            }
            else if (sender == cmbLogicERC)
            {
                nProVal = this.cmbLogicERC.SelectedIndex;
                CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmERC_LOGIC, nProVal); //ERC Logic
            }
            else if (sender == cmbLogicINP)
            {
                nProVal = this.cmbLogicINP.SelectedIndex;
                CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmINP_LOGIC, nProVal); //INP Logic
            }
            else if (sender == cmbLogicLTC)
            {
                nProVal = this.cmbLogicLTC.SelectedIndex;
                CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmLTC_LOGIC, nProVal); //LTC Logic
            }
            else if (sender == cmbLogicORG)
            {
                nProVal = this.cmbLogicORG.SelectedIndex;
                CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmORG_LOGIC, nProVal); //ORG Logic
            }
            else if (sender == cmbLogicSVON)
            {
                nProVal = this.cmbLogicSVON.SelectedIndex;
                CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropIdEx.cmMPID_SVON_LOGIC, nProVal); //Servo On Logic
            }
        }

        // Mode 의 상태를 변경한다
        private void SetModeState(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nProVal = 0;

            if (sender == cmbModeALM)
            {
                nProVal = this.cmbModeALM.SelectedIndex;
                CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmALM_MODE, nProVal); //ALM Mode
            }
            else if (sender == cmbModeEL)
            {
                nProVal = this.cmbModeEL.SelectedIndex;
                CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmEL_MODE, nProVal); //EL Mode
            }
            else if (sender == cmbModeERC)
            {
                nProVal = this.cmbModeERC.SelectedIndex;
                CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmERC_OUT, nProVal); //ERC Mode
            }
            else if (sender == cmbModeINP)
            {
                nProVal = this.cmbModeINP.SelectedIndex;
                CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmINP_EN, nProVal); //INP Mode
            }
            else if (sender == cmbModeSD)
            {
                nProVal = this.cmbModeSD.SelectedIndex;
                CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmSD_MODE, nProVal); //SD Mode
            }
            else if (sender == cmbModeSTA)
            {
                nProVal = this.cmbModeSTA.SelectedIndex;
                CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmSTA_MODE, nProVal); //STA Mode
            }
            else if (sender == cmbModeSTP)
            {
                nProVal = this.cmbModeSTP.SelectedIndex;
                CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmSTP_MODE, nProVal); //STP Mode
            }
        }

        // 축(Axis)을 선택할 때마다 각 축에 대한 Logic 과 Mode 를 불러온다.
        private void cmbAxis_SelectedIndexChanged(object sender, EventArgs e)
        {
            GetLogicState();
            GetModeState();
        }

        private void chkServoOn_CheckedChanged(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nChkBox = 0;
            
            if(this.chkServoOn.Checked)
            {
                nChkBox = 1;
            }

            CMDLL.cmmGnSetServoOn(nAxis, nChkBox);
        }

        private void btnERCOut_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;

            CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmERC_ONTIME, 6);
            CMDLL.cmmAdvErcOut(nAxis, (int)Defines._TCmBool.cmFALSE);
        }

        private void btnARMReset_MouseDown(object sender, MouseEventArgs e)
        {
            this.pbxALMR.BackColor = Color.Blue;
        }

        private void btnARMReset_MouseUp(object sender, MouseEventArgs e)
        {
            this.pbxALMR.BackColor = Color.Silver;
            
            int nAxis = this.cmbAxis.SelectedIndex;

            CMDLL.cmmGnSetAlarmRes(nAxis, (int)Defines._TCmBool.cmFALSE);
        }

        private void StateMonitor_FormClosing(object sender, FormClosingEventArgs e)
        {
            // 사용중인 타이머 종료.
            timer1.Stop();

            CMDLL.cmmGnDeviceUnload(); // Unload Device
        }
    }
}
