﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Shared;
using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace PositionLatchQueue
{
    public partial class PositionLatchQueue : Form
    {
        private const int QUE_SIZE = 100; // 래치큐 크기

        // delegate
        delegate void delChkIsIntLatched(bool bVal); // 래치큐 기능 활성화 CHECK
        delegate void delSetLtcQueueValue(string sVal); // 래치큐 VALUE
        delegate void delSetLtcPeekAtValue(string sVal); // 래치큐 PEEK AT
        delegate void delSetLtcPeekIndex(int nVal); // 선택 PEEK AT SelectedIndex 변경

        IAsyncResult g_invoked01 = null;
        IAsyncResult g_invoked02 = null;
        IAsyncResult g_invoked03 = null;
        IAsyncResult g_invoked04 = null;

        private const int WM_APP = 0x8000;
        private const int WMU_MotInterrupt = WM_APP + 1;

        int g_nAxis = 0;
        int g_nLatchSource = 0;

        public PositionLatchQueue()
        {
            InitializeComponent();
        }

        // common function
        void InitMotionDevices()
        {
            ////////////////////////////////////////////////////////////////////////////////////////
            // [CAUTION]: 아래와 다른 CME파일을 사용하는 경우에는 아래 코드를 수정하여 올바른 CME 파일의
            // 경로를 지정하십시오.
            const string CME_FILE_NAME = "Default.cme2";

            string szCmeFilePath, szSystemDir;

            szSystemDir = Environment.SystemDirectory; // 윈도우즈 시스템 디렉토리명 얻어오기 (일반적으로는 C:\Windows\System32)
            szCmeFilePath = szSystemDir + "\\" + CME_FILE_NAME;

            if (CMDLL.cmmGnInitFromFile(szCmeFilePath) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(IntPtr.Zero); // 에러 발생하였으면 에러 디스플레이.
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////////////
                // CME 파일에서 정의한 축수와 실제 장착되어 있는 모션 축수를 비교확인 하고자할 때는 
                // 아래와 같이 하면 됩니다.

                int nNumAxes_installed = 0;
                int nNumAxes_defined = 0;

                CMDLL.cmmAdvGetNumAvailAxes(ref nNumAxes_installed); // 현재 실제로 PC에 장착되어 있는 모션축 수 (cmmGnDeviceLoad() 함수에서 반환하는 축수와 동일)
                CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes_defined); // CME 파일에서 정의한 축 수(CME 파일이 지정되지 않는 경우에는 현재 장착된 축 수와 일치합니다.)

                if (nNumAxes_installed != nNumAxes_defined)
                {
                    MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }

            // SERVO-ON //
            int nNumAxes = 0;
            CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes);
            for (int i = 0; i < nNumAxes; i++)
            {
                CMDLL.cmmGnSetServoOn(i, (int)Defines._TCmBool.cmTRUE);
            }
        }

        private void PositionLatchQueue_Load(object sender, EventArgs e)
        {
            // Load Motion & DIO devices //
            int nNumAxes = 0;
            if (CMDLL.cmmGnDeviceLoad((int)Defines._TCmBool.cmTRUE, ref nNumAxes) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle); // 에러 발생하였으면 에러 디스플레이합니다.
                return;
            }
            else
            {
                // Motion 환경 초기화 //
                InitMotionDevices();

                // 사용 가능한 축의 갯수 얻어오기
                int anNumAxesToDisp = 0;
                CMDLL.cmmAdvGetNumAvailAxes(ref anNumAxesToDisp);
                nNumAxes = anNumAxesToDisp;

                for (int i = 0; i < nNumAxes; i++)
                {
                    this.cmbAxis.Items.Insert(i, "Axis" + i.ToString());
                    CMDLL.cmmLtcQue_ResetItemCount(i);
                }
                this.cmbAxis.SelectedIndex = 0;

                cmbAxis_SelectedIndexChanged(null, null);

                // Set timer to read and display current command position //
                this.timer1.Start();
            }
        }

        private void cmbAxis_SelectedIndexChanged(object sender, EventArgs e)
        {
            g_nAxis = cmbAxis.SelectedIndex;

            this.cmbLtcSource.SelectedIndex = 0;
            this.cmbLtcLogic.SelectedIndex = 0;

            this.chkLtcQEnable.Checked = false;
            this.chkLtcMask.Checked = false;

            this.chkIsIntLatched.Checked = false;
            this.chkIsReadLatched.Checked = false;

            this.txtItemCount.Text = "";
            this.txtPeekAtValue.Text = "";
            this.txtDequeValue.Text = "";

            SetLatchQueueList();
        }

        // 래치큐 리스트 정보를 Refresh 한다
        void SetLatchQueueList()
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nLtcItemCount = 0;
            double fLatchData = 0;

            CMDLL.cmmLtcQue_GetItemCount(nAxis, ref nLtcItemCount);
            if (nLtcItemCount == 1)
            {
                this.chkIsReadLatched.Checked = true;
            }
            else
            {
                this.chkIsReadLatched.Checked = false;
            }

            this.lstLatchQueue.Items.Clear();
            this.cmbQIndex.Items.Clear();
            for (int i = 0; i < nLtcItemCount; i++)
            {
                CMDLL.cmmLtcQue_PeekAt(nAxis, i, ref fLatchData);
                this.lstLatchQueue.Items.Add(fLatchData.ToString());
                this.cmbQIndex.Items.Add(i.ToString());
            }

            if (nLtcItemCount > 0)
            {
                this.lstLatchQueue.SelectedIndex = this.lstLatchQueue.Items.Count - 1;
                this.cmbQIndex.SelectedIndex = 0;
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            this.lstLatchQueue.SelectedIndex = this.lstLatchQueue.Items.Count - 1;

            int nLtcItemCount=0;
            double fLatchData=0;

            int nAxis = cmbAxis.SelectedIndex;

            lstLatchQueue.Items.Clear();
            CMDLL.cmmLtcQue_GetItemCount(nAxis, ref nLtcItemCount);
            chkIsReadLatched.Checked = (nLtcItemCount > 0) ? true : false;

            if (nLtcItemCount < 1)
            {
                chkIsIntLatched.Checked = false;
                chkIsReadLatched.Checked = false;
                cmbQIndex.Items.Clear();

                this.txtItemCount.Text = "";
                this.txtPeekAtValue.Text = "";
                this.txtDequeValue.Text = "";
            }


            for (int i = 0; i < nLtcItemCount; i++)
            {
                CMDLL.cmmLtcQue_PeekAt(nAxis, i, ref fLatchData);
                lstLatchQueue.Items.Add(fLatchData.ToString());
            }
            lstLatchQueue.SelectedIndex = lstLatchQueue.Items.Count - 1;
            
            if (this.lstLatchQueue.Items.Count > 0)
            {
                this.btnGetItemCount.Enabled = true;
                this.btnPeekAt.Enabled = true;
                this.cmbQIndex.Enabled = true;
                this.btnDeque.Enabled = true;
            }
            else
            {
                this.btnGetItemCount.Enabled = false;
                this.btnPeekAt.Enabled = false;
                this.cmbQIndex.Enabled = false;
                this.btnDeque.Enabled = false;
            }
        }

        private void cmbLtcLogic_SelectedIndexChanged(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nLatchLogic = this.cmbLtcLogic.SelectedIndex;

            CMDLL.cmmCfgSetMioProperty(nAxis, (int)Defines._TCmMioPropId.cmLTC_LOGIC, nLatchLogic);
        }

        private void chkLtcQEnable_CheckedChanged(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nLatchSource = this.cmbLtcSource.SelectedIndex;

            int nEnableLtcQ = 0;
            if (this.chkLtcQEnable.Checked)
            {
                nEnableLtcQ = 1;
            }
            CMDLL.cmmLtcQue_SetEnable(nAxis, nEnableLtcQ);

            if (nEnableLtcQ > 0)
            {
                CMDLL.cmmMstAll_SetCfg(nAxis, QUE_SIZE, nLatchSource);
            }
        }

        private void btnGetItemCount_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nQueItemCount = 0;

            CMDLL.cmmLtcQue_GetItemCount(nAxis, ref nQueItemCount);

            this.txtItemCount.Text = nQueItemCount.ToString();
        }

        private void btnLatchQueueReset_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            CMDLL.cmmLtcQue_ResetItemCount(nAxis);

            this.txtItemCount.Text = "";
            this.txtPeekAtValue.Text = "";
            this.txtDequeValue.Text = "";

            chkIsIntLatched.Checked = false;
            chkIsReadLatched.Checked = false;
            cmbQIndex.Items.Clear();
        }

        private void btnDeque_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nLatchQueueCount = 0;
            double fLtcData = 0;

            CMDLL.cmmLtcQue_GetItemCount(nAxis, ref nLatchQueueCount);

            if (nLatchQueueCount > 0)
            {
                CMDLL.cmmLtcQue_Deque(nAxis, ref fLtcData);
                this.txtDequeValue.Text = fLtcData.ToString();
            }

            SetLatchQueueList();
        }

        private void btnPeekAt_Click(object sender, EventArgs e)
        {
            double fLtcData=0.0;
            int nAxis = this.cmbAxis.SelectedIndex;
            int nCurQidx = this.cmbQIndex.SelectedIndex;
            int nItemCount=0;
            CMDLL.cmmLtcQue_GetItemCount(nAxis, ref nItemCount);
            if (nCurQidx >= 0 && nCurQidx < nItemCount)
            {
                if (CMDLL.cmmLtcQue_PeekAt(nAxis, nCurQidx, ref fLtcData) == Defines.cmERR_NONE)
                {
                    this.txtPeekAtValue.Text = fLtcData.ToString();
                }
            }
        }

        private void chkLtcMask_CheckedChanged(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nIntMask = 0x0;
            if (this.chkLtcMask.Checked)
            {
                nIntMask |= (1 << 14); // BIT14 (zero-based) 값이 Latch 에 대한 Mask 값이다.
            }
            else
            {
                nIntMask &= ~(1 << 14); // BIT14 (zero-based) 값이 Latch 에 대한 Mask 값이다.
            }

            CMDLL.cmmIntSetMask(nAxis, nIntMask);
            CMDLL.cmmIntHandlerSetup_MSG((int)Defines._TCmIntHandlerType.cmIHT_MESSAGE, this.Handle, WMU_MotInterrupt, this.Handle);
            CMDLL.cmmIntHandlerEnable((int)Defines._TCmBool.cmTRUE);
        }

        /***********************************************************************************************
        * WINDOW MESSAGE 방식 처리를 위한 Proc
        ************************************************************************************************/
        protected override void WndProc(ref Message m)
        {
            switch (m.Msg)
            {
                case WMU_MotInterrupt:
                    OnCmmInterruptMessage(this.Handle);
                    break;
                default:
                    base.WndProc(ref m);
                    break;
            }
        }

        /***********************************************************************************************
        * WINDOWS MESSAGE 방식 처리 
        ************************************************************************************************/
        void OnCmmInterruptMessage(IntPtr lParam)
        {
            //////////////////////////////////////////////////////////////////////////////
            // Windows Message 방식의 이벤트핸들러는 GUI 관련 작업을 직접 수행할 수 있다.
            // 단, 본 예제의 구성을 간단하게 하기 위해서 InterruptHandlerCommon() 함수를 
            // 호출하였다. 
            InterruptHandlerCommon(lParam);
        }

        /***********************************************************************************************
        * InterrHandlerCommon(): 각 Event Handler에서 공통으로 호출하는 함수. 
        ************************************************************************************************/
        void InterruptHandlerCommon(IntPtr lParam)
        {
            int nEvtState = 0;
            double fLatchValue = 0;
            int nLatchQueueCount = 0;
            int nIsIntLatched = 0; // 인터럽트 이벤트로 래치 발생 여부

            CMDLL.cmmIntReadEventStatus(g_nAxis, ref nEvtState);
            nIsIntLatched = ((nEvtState >> 14) & 0x1); // BIT14 (zero-based) 값이 Latch 에 대한 Mask 값이다.
            if (nIsIntLatched == 1)
            {
                SetCheckIsIntLatched(true);

                CMDLL.cmmLtcReadLatch(g_nAxis, g_nLatchSource, ref fLatchValue);
                SetLtcQueueValue(fLatchValue.ToString());

                CMDLL.cmmLtcQue_GetItemCount(g_nAxis, ref nLatchQueueCount);
                nLatchQueueCount--;
                SetLtcPeekAtValue(nLatchQueueCount.ToString());
                SetLtcPeekIndex(0);
            }
            else
            {
                SetCheckIsIntLatched(false);
            }
        }


        // 인터럽트 이벤트로 확인된 래치 발생 여부를 체크한다
        void SetCheckIsIntLatched(bool bVal)
        {
            if (this.chkIsIntLatched.InvokeRequired)
            {
                if (g_invoked01 == null)
                {
                    g_invoked01 = BeginInvoke(new delChkIsIntLatched(SetCheckIsIntLatched), new object[] { bVal });
                }
                EndInvoke(g_invoked01);
            }
            else
            {
                this.chkIsIntLatched.Checked = bVal;
            }
        }

        // 래치큐 리스트에 값을 추가한다
        void SetLtcQueueValue(string sVal)
        {
            if (this.lstLatchQueue.InvokeRequired)
            {
                if (g_invoked02 == null)
                {
                    g_invoked02 = BeginInvoke(new delSetLtcQueueValue(SetLtcQueueValue), new object[] { sVal });
                }
                object obj = EndInvoke(g_invoked02);
            }
            else
            {
                this.lstLatchQueue.Items.Add(sVal);
            }
        }

        // PEEK AT LIST 에 값을 추가한다
        void SetLtcPeekAtValue(string sVal)
        {
            if (this.cmbQIndex.InvokeRequired)
            {
                if (g_invoked03 == null)
                {
                    g_invoked03 = BeginInvoke(new delSetLtcPeekAtValue(SetLtcPeekAtValue), new object[] { sVal });
                }
                EndInvoke(g_invoked03);
            }
            else
            {
                this.cmbQIndex.Items.Add(sVal);
            }
        }

        // PEEK AT LIST 의 선택 값을 0 번째 INDEX 로 초기화 한다
        void SetLtcPeekIndex(int nVal)
        {
            if (this.cmbQIndex.InvokeRequired)
            {
                if (g_invoked04 == null)
                {
                    g_invoked04 = BeginInvoke(new delSetLtcPeekIndex(SetLtcPeekIndex), new object[] { nVal });
                }
                EndInvoke(g_invoked04);
            }
            else
            {
                this.cmbQIndex.SelectedIndex = nVal;
            }
        }

        private void cmbLtcSource_SelectedIndexChanged(object sender, EventArgs e)
        {
            g_nLatchSource = cmbLtcSource.SelectedIndex;
        }
    }
}
