﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Shared;
using System.Threading;

using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace InterruptEvent
{
    public partial class InterruptEvent : Form
    {
        private static int NUM_CHECKBOX = 15;
        
        // Array List
        CheckBox[] chkIntpEventMask = new CheckBox[NUM_CHECKBOX]; // Interrupt Event Mask CHECKBOX
        
        // delegate
        delegate int delGetAxis(); // 선택 축 정보
        delegate void delSetInterruptHandlerMessage(string sVal); // Interrupt Event Message

        private const int WM_APP = 0x8000;
        private const int WMU_MotInterrupt = WM_APP + 1;

        public InterruptEvent()
        {
            InitializeComponent();
        }

        // common function
        void InitMotionDevices()
        {
            ////////////////////////////////////////////////////////////////////////////////////////
            // [CAUTION]: 아래와 다른 CME파일을 사용하는 경우에는 아래 코드를 수정하여 올바른 CME 파일의
            // 경로를 지정하십시오.
            const string CME_FILE_NAME = "Default.cme2";

            string szCmeFilePath, szSystemDir;

            szSystemDir = Environment.SystemDirectory; // 윈도우즈 시스템 디렉토리명 얻어오기 (일반적으로는 C:\Windows\System32)
            szCmeFilePath = szSystemDir + "\\" + CME_FILE_NAME;

            if (CMDLL.cmmGnInitFromFile(szCmeFilePath) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(IntPtr.Zero); // 에러 발생하였으면 에러 디스플레이.
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////////////
                // CME 파일에서 정의한 축수와 실제 장착되어 있는 모션 축수를 비교확인 하고자할 때는 
                // 아래와 같이 하면 됩니다.

                int nNumAxes_installed = 0;
                int nNumAxes_defined = 0;

                CMDLL.cmmAdvGetNumAvailAxes(ref nNumAxes_installed); // 현재 실제로 PC에 장착되어 있는 모션축 수 (cmmGnDeviceLoad() 함수에서 반환하는 축수와 동일)
                CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes_defined); // CME 파일에서 정의한 축 수(CME 파일이 지정되지 않는 경우에는 현재 장착된 축 수와 일치합니다.)

                if (nNumAxes_installed != nNumAxes_defined)
                {
                    MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }

            // SERVO-ON //
            int nNumAxes = 0;
            CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes);
            for (int i = 0; i < nNumAxes; i++)
            {
                CMDLL.cmmGnSetServoOn(i, (int)Defines._TCmBool.cmTRUE);
            }
        }
        
        private void InterruptEvent_Load(object sender, EventArgs e)
        {
            // Load Motion & DIO devices //
            int anNumAxes = 0;
            if (CMDLL.cmmGnDeviceLoad((int)Defines._TCmBool.cmTRUE, ref anNumAxes) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle); // 에러 발생하였으면 에러 디스플레이합니다.
                return;
            }
            else
            {
                // Motion 환경 초기화 //
                InitMotionDevices();

                ////////////////////////////////////////////////////////////////////////////////////
                int i;

                // 사용 가능한 축의 갯수 얻어오기
                int anNumAxesToDisp = 0;
                CMDLL.cmmAdvGetNumAvailAxes(ref anNumAxesToDisp);
                int nNumAxes = anNumAxesToDisp;

                for (i = 0; i < nNumAxes; i++)
                {
                    this.cmbAxis.Items.Insert(i, "Axis" + i.ToString());
                }

                this.cmbAxis.SelectedIndex = 0;
                this.cmbIntHandlerType.SelectedIndex = 0;

                chkIntpEventMask[0] = chkMask0;
                chkIntpEventMask[1] = chkMask1;
                chkIntpEventMask[2] = chkMask2;
                chkIntpEventMask[3] = chkMask3;
                chkIntpEventMask[4] = chkMask4;
                chkIntpEventMask[5] = chkMask5;
                chkIntpEventMask[6] = chkMask6;
                chkIntpEventMask[7] = chkMask7;
                chkIntpEventMask[8] = chkMask8;
                chkIntpEventMask[9] = chkMask9;
                chkIntpEventMask[10] = chkMask10;
                chkIntpEventMask[11] = chkMask11;
                chkIntpEventMask[12] = chkMask12;
                chkIntpEventMask[13] = chkMask13;
                chkIntpEventMask[14] = chkMask14;
            }
        }

        //*****************************************************************************
        // OnTimer(): Timer 이벤트 핸들러. 리스트박스에 버퍼링된 인터럽트 메세지들을 
        // 디스플레이한다.
        //*****************************************************************************
        private void timer1_Tick(object sender, EventArgs e)
        {
            
        }

        private void btnSelectAll_Click(object sender, EventArgs e)
        {
            bool bIsChecked = true;
            OnCheckInterruptMask(bIsChecked);
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            bool bIsChecked = false;
            OnCheckInterruptMask(bIsChecked);
        }

        void OnCheckInterruptMask(bool bIsChecked)
        {
            if (bIsChecked)
            {
                for (int i = 0; i < chkIntpEventMask.Length; i++ )
                {
                    chkIntpEventMask[i].Checked = true;
                }
            }
            else
            {
                for (int i = 0; i < chkIntpEventMask.Length; i++)
                {
                    chkIntpEventMask[i].Checked = false;
                }
            }
        }

        //*****************************************************************************
        // OnchkEnInterrupt(): "Enable Interrupt Monitor" CheckBox의 이벤트핸들러. 
        // Interrupt Handler를 등록하고 Interrup Handling 기능을 활성화한다.
        // Interrupt Handler는 1)Window Message방식, 2)Event Object 방식, 
        // 3)CALLBACK 방식의 3가지 방식으로 등록할 수 있다.
        //*****************************************************************************

        private void chkEnInterrupt_CheckedChanged(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nIntMask = 0x0;
            int nHandlerType = this.cmbIntHandlerType.SelectedIndex;
            bool bIsChecked = this.chkEnInterrupt.Checked;

            // 선택 된 Interrupt Event Mask 체크
            CMDLL.cmmIntGetMask(nAxis, ref nIntMask);

            if (bIsChecked)
            {
                this.cmbAxis.Enabled = false;
                this.grbIntrEventMask.Enabled = false;
                this.cmbIntHandlerType.Enabled = false;

                switch (nHandlerType)
                {
                    case (int)Defines._TCmIntHandlerType.cmIHT_MESSAGE :
                        CMDLL.cmmIntHandlerSetup_MSG((int)Defines._TCmIntHandlerType.cmIHT_MESSAGE, this.Handle, WMU_MotInterrupt, this.Handle);
                        break;

                    case (int)Defines._TCmIntHandlerType.cmIHT_EVENT:
                        MessageBox.Show("지원되지 않는 기능입니다.", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        this.chkEnInterrupt.Checked = false;
                        //CMDLL.cmmIntHandlerSetup_EVT((int)Defines._TCmIntHandlerType.cmIHT_EVENT, goEvent, 0, this.Handle);
                        break;

                    case (int)Defines._TCmIntHandlerType.cmIHT_CALLBACK:
                        CMDLL.cmmIntHandlerSetup_CLB((int)Defines._TCmIntHandlerType.cmIHT_CALLBACK, InterruptCallback, 0, this.Handle);
                        break;
                }

                CMDLL.cmmIntHandlerEnable((int)Defines._TCmBool.cmTRUE);
                this.lblStatus.Text = "인터럽트 모니터 기능이 활성화 되었습니다. 인터럽트 발생 감시를 시작합니다.";
            }
            else
            {
                this.cmbAxis.Enabled = true;
                this.grbIntrEventMask.Enabled = true;
                this.cmbIntHandlerType.Enabled = true;

                CMDLL.cmmIntHandlerEnable((int)Defines._TCmBool.cmFALSE);
                this.lblStatus.Text = "인터럽트 모니터 기능이 비활성화 되었습니다.";
            }
        }

        //*****************************************************************************
        // OnbtnApplyMask(): "Apply Mask Change" 이벤트 핸들러. EventInterrupt의 마스크
        // 를 설정한다. 마스크는 인터럽트를 받고자하는 종류를 선택하는 것이다. 
        // 참고: ErrorInterrupt는 마스크에 관계없이 항상 인터럽트를 받을 수 있도록 되어 
        // 있다.
        //*****************************************************************************
        private void btnApplyMask_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;

            // IntMask에서 몇 개의 비트가 Reserved로 되어 있어서 화면상의 CheckBox 순서가
            // 비트 순서와 일치하지 않아서 아래와 같이 각 CheckBox별로 비트 순서를 알려주는 
            // 배열을 작성하였다.
            int[] nBitIdx = new int[] { 0, 1, 4, 5, 6, 7, 10, 11, 12, 13, 14, 15, 16, 17, 18 };
            int nIntMask = 0x0;
            for(int i=0; i<nBitIdx.Length; i++)
            {
                if (chkIntpEventMask[i].Checked)
                {
                    (nIntMask) |= (1 << (nBitIdx[i]));
                }
                else
                {
                    (nIntMask) &= ~(1 << (nBitIdx[i]));
                }
            }
            
            CMDLL.cmmIntSetMask(nAxis, nIntMask);
        }

        //*****************************************************************************
        // btnListClear_Click(): "Clear" 버튼 이벤트 핸들러. 리스트박스의 내용을 클리어한다.
        //*****************************************************************************
        private void btnListClear_Click(object sender, EventArgs e)
        {
            this.lstEvent.Items.Clear();
            this.lblStatus.Text = "이벤트 인터럽트 발생 이력이 모두 삭제 되었습니다.";
        }

        //*****************************************************************************
        // GetEventIntString(): 이벤트 인터럽트 상태코드의 각 비트별로 의미하는 인터럽트
        // 이름을 스트링으로 반환해주는 함수.
        //*****************************************************************************
        string GetEventIntString(int nBitIdx)
        {
            string[] szEventState = {
                "Normal Stop", // 0
                "Succesive start", // 1
                "", // 2
                "", // 3
                "Start of acceleration", //4
                "End of acceleration", // 5
                "Start of deceleration", // 6
                "End of deceleration", // 7
                "", // 8
                "", // 9
                "Position error tolerance exceed", // 10
                "General Comparator", // 11
                "Compared trigger", // 12
                "CLR signal resets the counter value", // 13
                "LTC input making counter value latched", // 14
                "ORG input signal ON", // 15
                "SD input signal ON", // 16
                "+DR input changes", // 17
                "-DR input changes" // 18
            };

            return szEventState[nBitIdx];
        }

        /***********************************************************************************************
        * CALLBACK 방식 처리 
        ************************************************************************************************/
        void InterruptCallback(IntPtr lParam)
        {
            //////////////////////////////////////////////////////////////////////////////////////////
            // 주의: 쓰레드내에서 GUI 관련작업을 수행하면 에러를 발생시키는 것과 마찬가지로 CALLBACK
            // 함수내에서 GUI 관련작업을 수행하면 예기치않은 에러를 발생시킬 수 있으므로, 가급적 
            // GUI 관련작업은 피하는 것이 좋다. 단, GUI의 속성을 참조하는 것은 상관없다.
            InterruptHandlerCommon(lParam);
        }

        /***********************************************************************************************
        * InterrHandlerCommon(): 각 Event Handler에서 공통으로 호출하는 함수. 
        ************************************************************************************************/
        void InterruptHandlerCommon(IntPtr lParam)
        {
            int nFlag1 = 0, nFlag2 = 0;
            int nIsTheAxis = 0;
            int nErrState = 0, nEvtState = 0;
            string strDateTime = "";
            string strMsg = "";

            int nAxis = GetAxis();

            CMDLL.cmmIntReadFlag(ref nFlag1, ref nFlag2);
            ///////////////////////////////////////////////////////////////////////////////////////////////////
            // 본 예제에서는 지정한 축의 인터럽트만 처리하는 것이므로 지정한 축의 인터럽트가 발생했는지 체크한다. 
            // cmmIntReadFlag() 함수 대신에 cmmIntReadErrorStatus()와 cmmIntReadEventStatus()를 사용해서도
            // 인터럽트가 발생되었는지를 체크할 수 있다. 다만 여러축의 인터럽트 상태를 체크할 때에는 
            // cmmIntReadFlag() 함수를 사용하는 것이 처리속도 측면에서 상당히 유리하다.
            if(nAxis < 32)
            {
                nIsTheAxis = (nFlag1 >> nAxis) & 1;
            }
            else
            {
                nIsTheAxis = (nFlag1 >> (nAxis - 32)) & 1;
            }

            ////////////////////////////////////////////////////////////////////////////////////////////////////
            // 참고: GUI 관련 작업을 이 함수내에서 수행하는 것을 피하기 위하여 List에 보여줄 아이템들을 CArray
            // 변수인 CInterruptEventDlg 클래스의 m_IntItemArray 멤버변수에 저장하고, CInterruptEventDlg 타이머
            // 이벤트에서 m_IntItemArray 멤버에 저장된 아이템들을 디스플레이하는 방식으로 구성되었다.

            // 해당축에 인터럽트가 발생되었으면... //
            if (nIsTheAxis == 1)
            {
                DateTime currTime = DateTime.Now;

                // Write time stamp string //
                strDateTime += "[";
                strDateTime += currTime.ToString("HH:mm:ss") + ":";
                strDateTime += GetZeroNumber(currTime.Millisecond.ToString());
                //strDateTime += currTime.ToString("ms");
                strDateTime += "]";

                // Error Interrupt가 발생하였는지 체크 //
                if (CMDLL.cmmIntReadErrorStatus(nAxis, ref nErrState) == Defines.cmERR_NONE && nErrState < 0)
                {
                    strMsg = "[Error Interrupt] " + strDateTime + " " + GetEventIntString(nErrState);
                    SetInterruptHandlerMessage(strMsg);
                }

                // Event Interrupt가 발생하였는지 체크 //
                if (CMDLL.cmmIntReadEventStatus(nAxis, ref nEvtState) == Defines.cmERR_NONE && nEvtState != 0)
                {
                    for (int i = 0; i < 19; i++)
                    {
                        if (((nEvtState >> i) & 1) == (int)Defines._TCmBool.cmTRUE)
                        {
                            strMsg = "[Event Interrupt] " + strDateTime + " " + GetEventIntString(i);
                            SetInterruptHandlerMessage(strMsg);
                        }
                    }
                }
            }
        }

        // 0으로 채우기
        string GetZeroNumber(string sVal)
        {
            string sZero = "";
            
            for (int i = 0; i < (3-sVal.Length); i++ )
            {
                sZero += "0";
            }

            return sZero + sVal;
        }

        // 현재 선택 된 축을 가져온다
        int GetAxis()
        {
            if (this.cmbAxis.InvokeRequired)
            {
                delGetAxis d = new delGetAxis(GetAxis);
                return (int)this.Invoke(d, new object[] { });
            }
            else
            {
                return this.cmbAxis.SelectedIndex;
            }
        }

        // 래치큐 리스트에 값을 추가한다
        void SetInterruptHandlerMessage(string sVal)
        {
            if (this.lstEvent.InvokeRequired)
            {
                delSetInterruptHandlerMessage d = new delSetInterruptHandlerMessage(SetInterruptHandlerMessage);
                this.Invoke(d, new object[] { sVal });
            }
            else
            {
                this.lstEvent.Items.Add(sVal);
            }
        }

        /***********************************************************************************************
        * WINDOWS MESSAGE 방식 처리 
        ************************************************************************************************/
        void OnCmmInterruptMessage(IntPtr lParam)
        {
            //////////////////////////////////////////////////////////////////////////////
            // Windows Message 방식의 이벤트핸들러는 GUI 관련 작업을 직접 수행할 수 있다.
            // 단, 본 예제의 구성을 간단하게 하기 위해서 InterruptHandlerCommon() 함수를 
            // 호출하였다. 
            InterruptHandlerCommon(lParam);
        }

        /***********************************************************************************************
        * WINDOW MESSAGE 방식 처리를 위한 Proc
        ************************************************************************************************/
        protected override void WndProc(ref Message m)
        {
            switch(m.Msg)
            {
                case WMU_MotInterrupt:
                    OnCmmInterruptMessage(this.Handle);
                    break;
                default:
                    base.WndProc(ref m);
                    break;
            }
        }

        private void InterruptEvent_FormClosing(object sender, FormClosingEventArgs e)
        {
            CMDLL.cmmGnDeviceUnload(); // Unload Device
        }
    }
}
