﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Shared;

using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace ManualPulsar
{
    public partial class frmManualPulsar : Form
    {
        public frmManualPulsar()
        {
            InitializeComponent();
        }

        // common function
        void InitMotionDevices()
        {
            ////////////////////////////////////////////////////////////////////////////////////////
            // [CAUTION]: 아래와 다른 CME파일을 사용하는 경우에는 아래 코드를 수정하여 올바른 CME 파일의
            // 경로를 지정하십시오.
            const string CME_FILE_NAME = "Default.cme2";

            string szCmeFilePath, szSystemDir;

            szSystemDir = Environment.SystemDirectory; // 윈도우즈 시스템 디렉토리명 얻어오기 (일반적으로는 C:\Windows\System32)
            szCmeFilePath = szSystemDir + "\\" + CME_FILE_NAME;

            if (CMDLL.cmmGnInitFromFile(szCmeFilePath) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(IntPtr.Zero); // 에러 발생하였으면 에러 디스플레이.
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////////////
                // CME 파일에서 정의한 축수와 실제 장착되어 있는 모션 축수를 비교확인 하고자할 때는 
                // 아래와 같이 하면 됩니다.

                int nNumAxes_installed = 0;
                int nNumAxes_defined = 0;

                CMDLL.cmmAdvGetNumAvailAxes(ref nNumAxes_installed); // 현재 실제로 PC에 장착되어 있는 모션축 수 (cmmGnDeviceLoad() 함수에서 반환하는 축수와 동일)
                CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes_defined); // CME 파일에서 정의한 축 수(CME 파일이 지정되지 않는 경우에는 현재 장착된 축 수와 일치합니다.)

                if (nNumAxes_installed != nNumAxes_defined)
                {
                    MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }

            // SERVO-ON //
            int nNumAxes = 0;
            CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes);
            for (int i = 0; i < nNumAxes; i++)
            {
                CMDLL.cmmGnSetServoOn(i, (int)Defines._TCmBool.cmTRUE);
            }
        }

        private void frmManualPulsar_Load(object sender, EventArgs e)
        {
            // Load Motion & DIO devices //
            int nNumAxes = 0;
            if (CMDLL.cmmGnDeviceLoad((int)Defines._TCmBool.cmTRUE, ref nNumAxes) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle); // 에러 발생하였으면 에러 디스플레이합니다.
                return;
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////
                // 로드된 축수가 올바른지 체크: 본 예제에서는 원하는 축 수를 모르기 때문에 
                // 주석처리 해놓았습니다.

                //				const int NUM_AXES_TO_LOAD = 4; // 이 값을 시스템에서 필요한 축 수에 맞게 조정 
                //				if(anNumAxes[0] < NUM_AXES_TO_LOAD){
                //					MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                //				}

                // Motion 환경 초기화 //
                InitMotionDevices();

                ////////////////////////////////////////////////////////////////////////////////////
                // 축설정 콤보박스에 축리스트를 채워넣습니다. 
                byte[] aszAxisTitle = new byte[Defines.CMM_MAX_STR_LEN_AXIS_TITLE];

                this.cmbAxis.Items.Clear(); // Clear combobox
                string szTitle;

                for (int i = 0; i < nNumAxes; i++)
                {
                    CMDLL.cmmAdvGetString(i, (int)Defines._TCmStringID.cmSTR_AXIS_NAME, aszAxisTitle); // CME 빌더에서 정의한 각 축의 타이틀을 가져오는 함수.

                    szTitle = "";
                    for (int j = 0; j < aszAxisTitle.Length; j++)
                    {
                        szTitle += Convert.ToChar(aszAxisTitle[j]);
                    }

                    this.cmbAxis.Items.Add(szTitle);
                }

                if (this.cmbAxis.Items.Count > 0)
                    this.cmbAxis.SelectedIndex = 0;

                GetStatement();

                // Set timer to read and display current command position //
                this.timer1.Start();
            }
        }

        //환경을 불러 온다
        public void GetStatement()
        {
            int nSpeedMode = 0, nInputMode = 0, nInverse = 0, nEnable = 0, nGainFactor = 0, nDivFactor = 0;
            double fWork = 0, fAcc = 0, fDec = 0, fMinSpeed = 0, fMaxSpeed = 0;
            int nAxis = this.cmbAxis.SelectedIndex;

            //Move Mode
            CMDLL.cmmCfgGetSpeedPattern(nAxis, ref nSpeedMode, ref fWork, ref fAcc, ref fDec);
            this.cmbMoveMode.SelectedIndex = nSpeedMode;

            //Max Speed
            CMDLL.cmmCfgGetSpeedRange(nAxis, ref fMinSpeed, ref fMaxSpeed);
            this.txtMaxSpeed.Text = fMaxSpeed.ToString();

            //Distance
            CMDLL.cmmPlsrGetInMode(nAxis, ref nInputMode, ref nInverse);
            this.cmbInputMode.SelectedIndex = nInputMode;
            if (nInverse > 0)
            {
                this.chkReverse.Checked = true;
            }

            //Noise Filter
            CMDLL.cmmCfgGetFilterAB(nAxis, (int)Defines._TCmABFilter.cmAB_PULSAR, ref nEnable);
            if (nEnable > 0)
            {
                this.chkNoiseFilter.Checked = true;
            }

            //PA/PB Gain/Division
            CMDLL.cmmPlsrGetGain(nAxis, ref nGainFactor, ref nDivFactor);
            this.txtGainFactor.Text = nGainFactor.ToString();
            this.txtDivFactor.Text = nDivFactor.ToString();
        }

        private void btnManPulsarRun_Click(object sender, EventArgs e)
        {
            this.btnManPulsarRun.Enabled = false;

            int nAxis = this.cmbAxis.SelectedIndex;
            
            // Manual Pulsar Move
            double fDistance = Convert.ToDouble(this.txtDistance.Text);
            CMDLL.cmmPlsrMove(nAxis, fDistance, (int)Defines._TCmBool.cmFALSE);
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            this.btnManPulsarRun.Enabled = true;

            int nAxis = this.cmbAxis.SelectedIndex;

            CMDLL.cmmSxStop(nAxis, (int)Defines._TCmBool.cmTRUE, (int)Defines._TCmBool.cmFALSE);
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            string sPos;
            double afCurPos = 0;
            int nIsActive = 0;

            CMDLL.cmmStGetPosition(nAxis, (int)Defines._TCmCntr.cmCNT_COMM, ref afCurPos);
            sPos = afCurPos.ToString();
            this.txtCurPos.Text = sPos;

            CMDLL.cmmPlsrIsActive(nAxis, ref nIsActive);
            if(nIsActive == 1)
            {
                this.txtPulsarState.Text = "Active(Wait PLSR)";
            }
            else
            {
                this.txtPulsarState.Text = "Inactive";
            }
        }

        private void btnApplyPulsar_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            
            //Distance
            int nInputMode = this.cmbInputMode.SelectedIndex;
            int nInverse = 0;
            
            if(this.chkReverse.Checked)
            {
                nInverse = 1;
            }

            CMDLL.cmmPlsrSetInMode(nAxis, nInputMode, nInverse);
        }

        private void btnApplyGain_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            
            //PA/PB Gain/Division
            int nGainFactor = int.Parse(this.txtGainFactor.Text);
            int nDivFactor = int.Parse(this.txtDivFactor.Text);

            CMDLL.cmmPlsrSetGain(nAxis, nGainFactor, nDivFactor);
        }

        private void btnApplyBasic_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;

            //Move Mode
            double fMaxSpeed = Convert.ToDouble(this.txtMaxSpeed.Text);
            int nSpeedMode = this.cmbMoveMode.SelectedIndex;
            CMDLL.cmmCfgSetSpeedPattern(nAxis, nSpeedMode, fMaxSpeed / 10, fMaxSpeed, fMaxSpeed);

            //Max Speed
            CMDLL.cmmCfgSetSpeedRange(nAxis, fMaxSpeed); //최저 속도는 10으로 고정

            //Noise Filter
            int nEnable = 0;
            if (this.chkNoiseFilter.Checked)
            {
                nEnable = 1;
            }
            CMDLL.cmmCfgGetFilterAB(nAxis, (int)Defines._TCmABFilter.cmAB_PULSAR, ref nEnable);
        }
    }
}
