﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Shared;
using System.Threading;

using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace CalcTaktTime
{
    public partial class CalcTaktTime : Form
    {
        private int LM_MAP_IDX = 0;
        private int STATUS_BUF = 20;

        bool g_bListMotionThreadStop = true;
        bool g_bCannotRunState = false;

        int g_dwTaktTime = 0;
        int g_dwThreadInterval = 15;
        int g_nVelSpeedMultiply = 1;
        int g_nAccSpeedMultiply = 1;

        double g_fDist = 1000;
        
        public CalcTaktTime()
        {
            InitializeComponent();
        }

        // common function
        void InitMotionDevices()
        {
            CMDLL.cmmCfgSetSpeedRange((int)Defines._TCmAxis.cmX1, 6553500); // 6.5MPPS => Max PPS (Pulse/Sec) Setting for Axis X
            
            this.txtThreadIntv.Text = "15";
        }

        private void CalcTaktTime_Load(object sender, EventArgs e)
        {
            // Load Motion & DIO devices //
            int nNumAxes = 0;
            if (CMDLL.cmmGnDeviceLoad((int)Defines._TCmBool.cmTRUE, ref nNumAxes) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle); // 에러 발생하였으면 에러 디스플레이합니다.
                return;
            }
            else
            {
                // Motion 환경 초기화 //
                InitMotionDevices();

                // ALM Logic X축
                this.cmbAlmLogicX.SelectedIndex = 0;
                cmbAlmLogicX_SelectedIndexChanged(null, null);

                // Limit Logic X축
                this.cmbLimitLogicX.SelectedIndex = 0;
                cmbLimitLogicX_SelectedIndexChanged(null, null);

                // Command Output Mode X축
                this.cmbCmdOutModeX.SelectedIndex = 4;
                cmbCmdOutModeX_SelectedIndexChanged(null, null);

                // Feedback Input Mode X축
                this.cmbFeedInModeX.SelectedIndex = 0;
                cmbFeedInModeX_SelectedIndexChanged(null, null);

                this.trbVelRatio.SetRange(1, 10);
                this.trbAccRatio.SetRange(1, 10);

                this.cmbSpeedMode.SelectedIndex = (int)Defines._TCmSpeedMode.cmSMODE_S;
                this.txtWorkSpeed.Text = "100000";
                this.txtAccSpeed.Text = "1000000";

                this.chkInpEnable.Checked = true;
                chkInpEnable_CheckedChanged(null, null);

                this.txtDistance.Text = "1000";

                // Set timer to read and display current command position //
                this.timer1.Start();
            }
        }

        private void cmbAlmLogicX_SelectedIndexChanged(object sender, EventArgs e)
        {
            int nLogic = this.cmbAlmLogicX.SelectedIndex;

            CMDLL.cmmCfgSetMioProperty((int)Defines._TCmAxis.cmX1, (int)Defines._TCmMioPropIdEx.cmMPID_ALM_LOGIC, nLogic);
        }

        private void cmbLimitLogicX_SelectedIndexChanged(object sender, EventArgs e)
        {
            int nLogic = this.cmbLimitLogicX.SelectedIndex;

            CMDLL.cmmCfgSetMioProperty((int)Defines._TCmAxis.cmX1, (int)Defines._TCmMioPropIdEx.cmMPID_EL_LOGIC, nLogic);
        }

        private void cmbCmdOutModeX_SelectedIndexChanged(object sender, EventArgs e)
        {
            int nOutputMode = this.cmbCmdOutModeX.SelectedIndex;

            CMDLL.cmmCfgSetOutMode((int)Defines._TCmAxis.cmX1, nOutputMode);
        }

        private void cmbFeedInModeX_SelectedIndexChanged(object sender, EventArgs e)
        {
            int nInputMode = this.cmbFeedInModeX.SelectedIndex;

            CMDLL.cmmCfgSetInMode((int)Defines._TCmAxis.cmX1, nInputMode, (int)Defines._TCmBool.cmFALSE);
        }

        private void chkInpEnable_CheckedChanged(object sender, EventArgs e)
        {
            int nChkBox = 0;

            if (this.chkInpEnable.Checked)
            {
                nChkBox = 1;
            }

            CMDLL.cmmCfgSetMioProperty((int)Defines._TCmAxis.cmX1, (int)Defines._TCmMioPropIdEx.cmMPID_INP_EN, nChkBox);
        }

        private void btnResetPos_Click(object sender, EventArgs e)
        {
            CMDLL.cmmStSetCount((int)Defines._TCmAxis.cmX1, (int)Defines._TCmCntr.cmCNT_COMM, 0);
            CMDLL.cmmStSetCount((int)Defines._TCmAxis.cmX1, (int)Defines._TCmCntr.cmCNT_FEED, 0);
        }

        private void chkServoOnX_CheckStateChanged(object sender, EventArgs e)
        {
            int nChkBox = 0;

            if (this.chkServoOnX.Checked)
            {
                nChkBox = 1;
            }

            CMDLL.cmmGnSetServoOn((int)Defines._TCmAxis.cmX1, nChkBox);
        }

        private void btnIntvApply_Click(object sender, EventArgs e)
        {
            string sTemp = this.txtThreadIntv.Text;
            int nIntv = 0;
            if (sTemp.CompareTo("") == 0)
            {
                nIntv = 100;
            }
            else
            {
                nIntv = int.Parse(sTemp);
            }

            g_dwThreadInterval = nIntv;
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            // Get current command position of specified axis //
            byte[] sStatus = new byte[STATUS_BUF];
            string sStatusOut;
            int nCheckMio = 0;
            int nCurXCntCmd = 0, nCurXCntFeed = 0;
            int nMioStatesX = 0, nMotStatesX = 0;
            double fCurXSpdCmd = 0, fCurXSpdFeed = 0;

            // speed x multiply value
            g_nVelSpeedMultiply = this.trbVelRatio.Value;
            g_nAccSpeedMultiply = this.trbAccRatio.Value;

            // Machine I/O Status
            CMDLL.cmmStReadMioStatuses((int)Defines._TCmAxis.cmX1, ref nMioStatesX);
            
            g_bCannotRunState = false; // initialize

            nCheckMio = (nMioStatesX >> (int)Defines._TCmMioState.cmIOST_ALM) & 1;
            if (nCheckMio == 1)
            {
                this.chkALM_X.Checked = true;
                g_bCannotRunState = true;
            }
            else
            {
                this.chkALM_X.Checked = false;
                g_bCannotRunState = false;
            }

            nCheckMio = (nMioStatesX >> (int)Defines._TCmMioState.cmIOST_ELN) & 1;
            if (nCheckMio == 1)
            {
                this.chkELN_X.Checked = true;
                g_bCannotRunState = true;
            }
            else
            {
                this.chkELN_X.Checked = false;
                g_bCannotRunState = false;
            }

            nCheckMio = (nMioStatesX >> (int)Defines._TCmMioState.cmIOST_ELP) & 1;
            if (nCheckMio == 1)
            {
                this.chkELP_X.Checked = true;
                g_bCannotRunState = true;
            }
            else
            {
                this.chkELP_X.Checked = false;
                g_bCannotRunState = false;
            }

            nCheckMio = (nMioStatesX >> (int)Defines._TCmMioState.cmIOST_DIR) & 1;
            if (nCheckMio == 1)
            {
                this.chkDIR_X.Checked = true;
            }
            else
            {
                this.chkDIR_X.Checked = false;
            }

            nCheckMio = (nMioStatesX >> (int)Defines._TCmMioState.cmIOST_RDY) & 1;
            if (nCheckMio == 1)
            {
                this.chkRDY_X.Checked = true;
            }
            else
            {
                this.chkRDY_X.Checked = false;
            }

            nCheckMio = (nMioStatesX >> (int)Defines._TCmMioState.cmIOST_ORG) & 1;
            if (nCheckMio == 1)
            {
                this.chkORG_X.Checked = true;
            }
            else
            {
                this.chkORG_X.Checked = false;
            }

            if (g_bCannotRunState)
            {
                this.txtStatus.Text = "Cannot Run List Motion. Check Servo Alarm or Limit Signal";
                g_bListMotionThreadStop = true;
            }
            else
            {
                this.txtStatus.Text = "Ready";
            }

            // Motion Status
            CMDLL.cmmStReadMotionState((int)Defines._TCmAxis.cmX1, ref nMotStatesX);

            CMDLL.cmmStGetMstString(nMotStatesX, sStatus, STATUS_BUF);
            sStatusOut = "";
            for (int j = 0; j < sStatus.Length; j++)
            {
                sStatusOut += Convert.ToChar(sStatus[j]);
            }
            this.lblMotStatesX.Text = sStatusOut;
            
            CMDLL.cmmStGetCount((int)Defines._TCmAxis.cmX1, (int)Defines._TCmCntr.cmCNT_COMM, ref nCurXCntCmd);
            CMDLL.cmmStGetCount((int)Defines._TCmAxis.cmX1, (int)Defines._TCmCntr.cmCNT_FEED, ref nCurXCntFeed);

            CMDLL.cmmStGetSpeed((int)Defines._TCmAxis.cmX1, (int)Defines._TCmCntr.cmCNT_COMM, ref fCurXSpdCmd); // mm/sec
            CMDLL.cmmStGetSpeed((int)Defines._TCmAxis.cmX1, (int)Defines._TCmCntr.cmCNT_FEED, ref fCurXSpdFeed); // mm/sec

            this.lblXPosCmd.Text = nCurXCntCmd.ToString();
            this.lblXPosFeed.Text = nCurXCntFeed.ToString();
            
            this.lblXSpeedCmd.Text = fCurXSpdCmd.ToString();
            this.lblXSpeedFeed.Text = fCurXSpdFeed.ToString();

            // last takt time
            this.txtTaktTime.Text = g_dwTaktTime.ToString();
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            this.cmbAlmLogicX.Enabled = false;
            this.cmbLimitLogicX.Enabled = false;
            this.cmbCmdOutModeX.Enabled = false;
            this.cmbFeedInModeX.Enabled = false;
            
            this.btnStart.Enabled = false;
            this.btnHome.Enabled = false;

            g_bListMotionThreadStop = false;

            // Thread Move Start
            new Thread(new ThreadStart(ThfuncListMotion)).Start();
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            g_bListMotionThreadStop = true;
            CMDLL.cmmLmxEnd(LM_MAP_IDX);
            CMDLL.cmmSxStop((int)Defines._TCmAxis.cmX1, (int)Defines._TCmBool.cmFALSE, (int)Defines._TCmBool.cmFALSE);
            
            this.cmbAlmLogicX.Enabled = true;
            this.cmbLimitLogicX.Enabled = true;
            this.cmbCmdOutModeX.Enabled = true;
            this.cmbFeedInModeX.Enabled = true;
            
            this.btnStart.Enabled = true;
            this.btnHome.Enabled = true;
        }

        private void btnHome_Click(object sender, EventArgs e)
        {
            // Thread Homming Start
            new Thread(new ThreadStart(ThFuncHomming)).Start();
        }

        private void btnEmgStop_Click(object sender, EventArgs e)
        {
            g_bListMotionThreadStop = true;
            CMDLL.cmmLmxEnd(LM_MAP_IDX);
            CMDLL.cmmSxStopEmg((int)Defines._TCmAxis.cmX1);

            this.cmbAlmLogicX.Enabled = true;
            this.cmbLimitLogicX.Enabled = true;
            this.cmbCmdOutModeX.Enabled = true;
            this.cmbFeedInModeX.Enabled = true;

            this.btnStart.Enabled = true;
            this.btnHome.Enabled = true;
        }

        void ThFuncHomming()
        {
            CMDLL.cmmHomeSetConfig((int)Defines._TCmAxis.cmX1, 1, 0, 1000, 0);
            CMDLL.cmmHomeSetSpeedPattern((int)Defines._TCmAxis.cmX1, (int)Defines._TCmSpeedMode.cmSMODE_T, 5000, 50000, 50000, 250);
            
            CMDLL.cmmHomeMoveStart((int)Defines._TCmAxis.cmX1, (int)Defines._TCmDir.cmDIR_N);
            CMDLL.cmmHomeWaitDone((int)Defines._TCmAxis.cmZ1, (int)Defines._TCmBool.cmFALSE);
        }

        void ThfuncListMotion()
        {
            CalcTaktTime pForm = new CalcTaktTime();
            
            int nVal = 0;
            int nSpeedMode = pForm.cmbSpeedMode.SelectedIndex;
            double fWork = Convert.ToDouble(pForm.txtWorkSpeed.Text);
            double fAcc = Convert.ToDouble(pForm.txtAccSpeed.Text);

            // Extend List Motion Starts
            CMDLL.cmmCfgGetMioProperty((int)Defines._TCmAxis.cmX1, (int)Defines._TCmMioPropIdEx.cmMPID_INP_EN, ref nVal);
            if (nVal == 1) // (int)Defines._TCmMioPropIdEx.cmMPID_INP_EN
            {
                CMDLL.cmmCfgSetMioProperty((int)Defines._TCmAxis.cmX1, (int)Defines._TCmMioPropIdEx.cmMPID_INP_EN, 0);
            }

            CMDLL.cmmCfgSetSpeedPattern((int)Defines._TCmAxis.cmX1, nSpeedMode, fWork * g_nVelSpeedMultiply, fAcc * g_nAccSpeedMultiply, fAcc * g_nAccSpeedMultiply); // 본 예제에서 가속도 감속도는 동일하게 설정.
            while (!g_bListMotionThreadStop)
            {
                if (!g_bCannotRunState)
                {
                    g_fDist = Convert.ToDouble(pForm.txtDistance.Text);
                    CASSDK.casTmrQPC_Start();
                    TaktMain();
                    CASSDK.casTmrQPC_Stop();
                    g_dwTaktTime = CASSDK.casTmrQPC_Result(0); // Measurement Mode (0 == uSec, 1 == millisec, 2 == sec)
                }
                Thread.Sleep(g_dwThreadInterval);
            }
        }

        void TaktMain()
        {
            CMDLL.cmmSxMoveStart((int)Defines._TCmAxis.cmX1, g_fDist);
            CMDLL.cmmSxWaitDone((int)Defines._TCmAxis.cmX1, (int)Defines._TCmBool.cmTRUE);
            CMDLL.cmmSxMoveStart((int)Defines._TCmAxis.cmX1, g_fDist*(-1));
            CMDLL.cmmSxWaitDone((int)Defines._TCmAxis.cmX1, (int)Defines._TCmBool.cmTRUE);
        }

        private void CalcTaktTime_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!g_bListMotionThreadStop)
            {
                btnStop_Click(null, null);
                Thread.Sleep(500); // wait some time until all thread terminated.
            }

            // 사용중인 타이머 종료.
            timer1.Stop();

            CMDLL.cmmGnDeviceUnload(); // Unload Device
        }
    }
}
