﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Shared;
using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace SxOptSetSyncMode
{
    public partial class SxOptSetSyncMode : Form
    {
        public SxOptSetSyncMode()
        {
            InitializeComponent();
        }

        // common function
        void InitMotionDevices()
        {
            ////////////////////////////////////////////////////////////////////////////////////////
            // [CAUTION]: 아래와 다른 CME파일을 사용하는 경우에는 아래 코드를 수정하여 올바른 CME 파일의
            // 경로를 지정하십시오.
            const string CME_FILE_NAME = "Default.cme2";

            string szCmeFilePath, szSystemDir;

            szSystemDir = Environment.SystemDirectory; // 윈도우즈 시스템 디렉토리명 얻어오기 (일반적으로는 C:\Windows\System32)
            szCmeFilePath = szSystemDir + "\\" + CME_FILE_NAME;

            if (CMDLL.cmmGnInitFromFile(szCmeFilePath) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(IntPtr.Zero); // 에러 발생하였으면 에러 디스플레이.
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////////////
                // CME 파일에서 정의한 축수와 실제 장착되어 있는 모션 축수를 비교확인 하고자할 때는 
                // 아래와 같이 하면 됩니다.

                int nNumAxes_installed = 0;
                int nNumAxes_defined = 0;

                CMDLL.cmmAdvGetNumAvailAxes(ref nNumAxes_installed); // 현재 실제로 PC에 장착되어 있는 모션축 수 (cmmGnDeviceLoad() 함수에서 반환하는 축수와 동일)
                CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes_defined); // CME 파일에서 정의한 축 수(CME 파일이 지정되지 않는 경우에는 현재 장착된 축 수와 일치합니다.)

                if (nNumAxes_installed != nNumAxes_defined)
                {
                    MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }

            // SERVO-ON //
            int nNumAxes = 0;
            CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes);
            for (int i = 0; i < nNumAxes; i++)
            {
                CMDLL.cmmGnSetServoOn(i, (int)Defines._TCmBool.cmTRUE);
            }
        }

        private void SxOptSetSyncMode_Load(object sender, EventArgs e)
        {
            // Load Motion & DIO devices //
            int nNumAxes = 0;
            if (CMDLL.cmmGnDeviceLoad((int)Defines._TCmBool.cmTRUE, ref nNumAxes) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle); // 에러 발생하였으면 에러 디스플레이합니다.
                return;
            }
            else
            {
                const int NUM_AXES_TO_LOAD = 4; // 이 값을 시스템에서 필요한 축 수에 맞게 조정
                if (nNumAxes < NUM_AXES_TO_LOAD)
                {
                    MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }

                // Motion 환경 초기화 //
                InitMotionDevices();
            }

            //////////////////////////////////////////////////////////////////////////
            // 사용자 인터페이스를 초기화 합니다.
            //////////////////////////////////////////////////////////////////////////
            InitializeUI();
            //////////////////////////////////////////////////////////////////////////
        }

        void InitializeUI()
        {
            for(int i=0; i<=8; i++)
            {
                this.cmbCondition.Items.Insert(i, "MODE 0" + i.ToString()); // MODE 0 ~ MODE 8
            }
            this.cmbCondition.SelectedIndex = 2; // 참조축이 가속을 끝내고 정속모드로 들어서는 순간에 이송을 시작합니다.
            cmbCondition_SelectedIndexChanged(null, null);

            this.cmbSetSyncMode.SelectedIndex = 1; // cmSYNC_DISABLE

            this.cmbCmpSrc.SelectedIndex = 0;

            this.txtCmpPos.Text = "20000.0";
        }

        // Combo 박스의 선택 상태가 변경될 때마다 호출됩니다. (CMBCondition) 
        private void cmbCondition_SelectedIndexChanged(object sender, EventArgs e)
        {
            int nCondition = this.cmbCondition.SelectedIndex;
            switch (nCondition)
            {
                case 7:
                    // MODE 7 번인 경우만 컨트롤을 활성화합니다.
                    this.cmbCmpSrc.Enabled = true;
                    //this.txtCmpPos.Enabled = true;
                    break;

                default:
                    this.cmbCmpSrc.Enabled = false;
                    //this.txtCmpPos.Enabled = false;
                    break;
            }
        }

        private void btnViewCondition_Click(object sender, EventArgs e)
        {
            string sMsg = "";
            sMsg += "0: 범용비교기의 조건이 충족되었을 때 이송을 시작합니다. 단, 이 방법을 사용하려면\n";
            sMsg += "CmpSenSetConfig() 함수의 CmpAction 매개 변수(媒介變數)를 0 으로 하여야 합니다.\n";
            sMsg += "1: 참조축이 가속을 시작할 때 이송을 시작합니다.\n";
            sMsg += "2: 참조축이 가속을 끝내고 정속모드로 들어서는 순간에 이송을 시작합니다.\n";
            sMsg += "3: 참조축이 감속을 시작할 때 이송을 시작합니다.\n";
            sMsg += "4: 참조축이 감속을 끝낼 때 이송을 시작합니다.\n";
            sMsg += "5: - SL 신호가 검출되었을 때 이송을 시작합니다.\n";
            sMsg += "6: +SL 신호가 검출되었을 때 이송을 시작합니다.\n";
            sMsg += "7: 범용 비교기에 설정된 조건이 만족되었을 때 이송을 시작합니다. *\n";
            sMsg += "8: TRG-CMP 조건이 만족되었을 때 이송을 시작합니다.\n";

            MessageBox.Show(sMsg);
        }

        // SxOptSetSyncMode 함수를 통해서 LX502 를 기준으로
        // 환경 설정을 진행합니다. 이 환경설정에는 모드 7 번에서의
        // 범용 비교기에 포지션 데이터를 설정하는 것을 포함합니다.
        private void btnApply_Click(object sender, EventArgs e)
        {
            int nSyncMode = this.cmbSetSyncMode.SelectedIndex;
            int nCmpSrc = this.cmbCmpSrc.SelectedIndex;
            int nCondition = this.cmbCondition.SelectedIndex;
            string sCmpPos = this.txtCmpPos.Text;
            if(nCondition == 7)
            {
                // X1 축의 범용 비교기에 비교 데이터를 설정합니다.
                CMDLL.cmmCmpTrgSetConfig((int)Defines._TCmAxis.cmX1, nCmpSrc, 1);

                if(sCmpPos.Equals(""))
                {
                    MessageBox.Show("Compare point don't available. it works has been canceled", "Alert", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }

                CMDLL.cmmCmpTrgSetOneData((int)Defines._TCmAxis.cmX1, Convert.ToDouble(sCmpPos));
            }

            //////////////////////////////////////////////////////////////////////////
            // 여기부터 실제 동기 구동 설정을 시작합니다.

            /*
            지정한 축에 대하여 이송명령이 전달되었을 때 이송동작의 시작이 다른축(Other Axis, 참조축)의 동작 상황에 동기되어
            시작되도록 할 때 사용하는 함수입니다. 예를 들어 Y1 축의 이송시작이 X1 축의 가속이 완료되는 시점 또는 감속이
            시작되는 시점에 수행되도록 하고자할 때 이 함수를 사용하면 해당 동작을 구현할 수 있습니다.
            */

            CMDLL.cmmSxOptSetSyncMode(
                (int)Defines._TCmAxis.cmY1, /* Channel */
                nSyncMode,                  /* Mode */
                (int)Defines._TCmAxis.cmX1, /* RefChannel */
                nCondition                  /* Condition */
                );

            /* Mode 에 대한 설명
            ----------------------
            0 번 :
            동기모드를 사용하지 않습니다. 따라서 이송명령이 하달되면 바로 이송을 시작합니다.
            ※ 이 모드에서는 RefAxis 와 Condition 매개 변수(媒介變數)가 무시됩니다.
        		  
            1 번 :
           “Start by Internal Sync. Signal” 모드: 내부동기 신호에 의하여 이송이 시작됩니다. 따라서
            이송명령이 하달되면 바로 이송을 시작하지 않고 내부동기 신호가 발생할 때 실제 이송이
            시작됩니다.
            내부동기 신호는 Condition 매개 변수(媒介變數)에 의해서 지정된 조건이 만족되면 자동으로
            발생됩니다.
            ※ 이 모드에서는 RefAxis 와 Condition 매개 변수(媒介變數)가 모두 참조됩니다..
        			
             2 번 :
           “Start by Other Axis/Axes Stop”모드: RefAxis 매개 변수(媒介變數)에 의하여 지정된 축의
            이송이 완료될 때 이송을 시작합니다. 따라서 RefAxis 에 의하여 지정된 축이 이송이
            완료되지 않은상태에서 이송명령이 하달되면 RefAxis 축이 정지(停止)하는 순간에 실제
            이송이 시작됩니다.
            ※ 이 모드에서는 RefAxis 만 참조 되며, Condition 매개 변수(媒介變數)는 무시됩니다.

	        모드 2 번일 경우에는 RefChannel 은 복수의 축을 의미하므로, 비트마스크(Bit Mask) 를 통한
	        인자 전달이 이루어 져야합니다. 자세한 내용은 매뉴얼을 참조하십시요.
	        */

            /* Condition 에 대한 설명
               ----------------------
            0 범용비교기의 조건이 충족되었을 때 이송을 시작합니다. 단, 이 방법을 사용하려면
            CmpSenSetConfig() 함수의 CmpAction 매개 변수(媒介變數)를 0 으로 하여야 합니다.
            1 참조축이 가속을 시작할 때 이송을 시작합니다.
            2 참조축이 가속을 끝내고 정속모드로 들어서는 순간에 이송을 시작합니다.
            3 참조축이 감속을 시작할 때 이송을 시작합니다.
            4 참조축이 감속을 끝낼 때 이송을 시작합니다.
            5 - SL 신호가 검출되었을 때 이송을 시작합니다.
            6 +SL 신호가 검출되었을 때 이송을 시작합니다.
            7 범용 비교기에 설정된 조건이 만족되었을 때 이송을 시작합니다. * 
            8 TRG-CMP 조건이 만족되었을 때 이송을 시작합니다.
            */

            //////////////////////////////////////////////////////////////////////////
            // Y1 축의 이송을 시작합니다.
            // 이때 Y1 축은 바로 이송을 하지 않고, 지정된 조건이 충족되었을 때,
            // 비로소 이송을 시작합니다.
            CMDLL.cmmSxMoveStart((int)Defines._TCmAxis.cmY1, Convert.ToDouble(sCmpPos));
        }

        // Negative(-) 방향으로 속도 이송
        private void btnMoveN_Click(object sender, EventArgs e)
        {
            double fCmpPos = Convert.ToDouble(this.txtCmpPos.Text);

            CMDLL.cmmSxMoveStart((int)Defines._TCmAxis.cmX1, -fCmpPos);
        }

        // Positive(+) 방향으로 속도 이송
        private void btnMoveP_Click(object sender, EventArgs e)
        {
            double fCmpPos = Convert.ToDouble(this.txtCmpPos.Text);

            CMDLL.cmmSxMoveStart((int)Defines._TCmAxis.cmX1, fCmpPos);
        }

        // 감속 후 정지
        private void btnStop_Click(object sender, EventArgs e)
        {
            CMDLL.cmmSxStop((int)Defines._TCmAxis.cmX1, (int)Defines._TCmBool.cmTRUE, (int)Defines._TCmBool.cmFALSE);
            CMDLL.cmmSxStop((int)Defines._TCmAxis.cmY1, (int)Defines._TCmBool.cmTRUE, (int)Defines._TCmBool.cmFALSE);
        }

        // 즉시 정지
        private void btnStopEmg_Click(object sender, EventArgs e)
        {
            CMDLL.cmmSxStopEmg((int)Defines._TCmAxis.cmX1);
            CMDLL.cmmSxStopEmg((int)Defines._TCmAxis.cmY1);
        }

        private void SxOptSetSyncMode_FormClosing(object sender, FormClosingEventArgs e)
        {
            CMDLL.cmmGnDeviceUnload(); // Unload Device
        }
    }
}
