﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using Shared;
using CMDLL = ComiLib.CMM.SafeNativeMethods;

namespace UsingSeqMode
{
    public partial class UsingSeqMode : Form
    {
        int g_uRunCount = 0; // Seq Mode Count
        int m_nMoveDir = (int)Defines._TCmDir.cmDIR_P; // Move 방향

        int g_nMoveMode=0;
        int g_nAxis=0;
        string g_sMsg = "";
        double g_fDistance;
        
        public UsingSeqMode()
        {
            InitializeComponent();
        }

        // common function
        void InitMotionDevices()
        {
            ////////////////////////////////////////////////////////////////////////////////////////
            // [CAUTION]: 아래와 다른 CME파일을 사용하는 경우에는 아래 코드를 수정하여 올바른 CME 파일의
            // 경로를 지정하십시오.
            const string CME_FILE_NAME = "Default.cme2";

            string szCmeFilePath, szSystemDir;

            szSystemDir = Environment.SystemDirectory; // 윈도우즈 시스템 디렉토리명 얻어오기 (일반적으로는 C:\Windows\System32)
            szCmeFilePath = szSystemDir + "\\" + CME_FILE_NAME;

            if (CMDLL.cmmGnInitFromFile(szCmeFilePath) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(IntPtr.Zero); // 에러 발생하였으면 에러 디스플레이.
            }
            else
            {
                ////////////////////////////////////////////////////////////////////////////////////////
                // CME 파일에서 정의한 축수와 실제 장착되어 있는 모션 축수를 비교확인 하고자할 때는 
                // 아래와 같이 하면 됩니다.

                int nNumAxes_installed = 0;
                int nNumAxes_defined = 0;

                CMDLL.cmmAdvGetNumAvailAxes(ref nNumAxes_installed); // 현재 실제로 PC에 장착되어 있는 모션축 수 (cmmGnDeviceLoad() 함수에서 반환하는 축수와 동일)
                CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes_defined); // CME 파일에서 정의한 축 수(CME 파일이 지정되지 않는 경우에는 현재 장착된 축 수와 일치합니다.)

                if (nNumAxes_installed != nNumAxes_defined)
                {
                    MessageBox.Show("현재 장착되어 있는 모션 축 수가 정의된 축 수와 일치하지 않습니다", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }

            // SERVO-ON //
            int nNumAxes = 0;
            CMDLL.cmmAdvGetNumDefinedAxes(ref nNumAxes);
            for (int i = 0; i < nNumAxes; i++)
            {
                CMDLL.cmmGnSetServoOn(i, (int)Defines._TCmBool.cmTRUE);
            }
        }

        private void UsingSeqMode_Load(object sender, EventArgs e)
        {
            // Load Motion & DIO devices //
            int anNumAxes = 0;
            if (CMDLL.cmmGnDeviceLoad((int)Defines._TCmBool.cmTRUE, ref anNumAxes) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle); // 에러 발생하였으면 에러 디스플레이합니다.
                return;
            }
            else
            {
                // Motion 환경 초기화 //
                InitMotionDevices();

                ////////////////////////////////////////////////////////////////////////////////////
                int i;

                // 사용 가능한 축의 갯수 얻어오기
                int anNumAxesToDisp = 0;
                CMDLL.cmmAdvGetNumAvailAxes(ref anNumAxesToDisp);
                int nNumAxes = anNumAxesToDisp;

                for (i = 0; i < nNumAxes; i++)
                {
                    this.cmbAxis.Items.Insert(i, "Axis" + i.ToString());
                }

                this.cmbAxis.SelectedIndex = 0;
                this.cmbMoveMode.SelectedIndex = 0;
                this.cmbSeqMode.SelectedIndex = 0;

                cmbMoveMode_SelectedIndexChanged(null, null); // Move 모드에 따른 각 에디트박스 Dimmed 처리를 위해서...
                
                // Set timer to read and display current command position //
                this.timer1.Start();
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            //------- 모션제어기 위치 모니터링 -------------------------------//
            int nAxis = this.cmbAxis.SelectedIndex;
            string sPos;
            double afCurPos = 0;

            // Command position display //
            CMDLL.cmmStGetPosition(nAxis, (int)Defines._TCmCntr.cmCNT_COMM, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.lblPosC.Text = sPos;

            // Feedback position display //
            CMDLL.cmmStGetPosition(nAxis, (int)Defines._TCmCntr.cmCNT_FEED, ref afCurPos);
            sPos = Convert.ToString(afCurPos);
            this.lblPosF.Text = sPos;
            this.lblStatus.Text = g_sMsg;
        }

        private void cmbMoveMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            g_nMoveMode = this.cmbMoveMode.SelectedIndex;

            if (g_nMoveMode == 0) // "Relative Position Move" Mode
            {
                this.lblDist.Enabled = true;
                this.txtDist.Enabled = true;
                this.lblPos1.Enabled = false;
                this.txtPos1.Enabled = false;
                this.lblPos2.Enabled = false;
                this.txtPos2.Enabled = false;
                this.btnStop.Enabled = true;
                this.btnMoveN.Text = "Move (-)";
                this.btnMoveP.Text = "Move (+)";
            }
            else if (g_nMoveMode == 1) // "Absolute Position Move" Mode
            {
                this.lblDist.Enabled = false;
                this.txtDist.Enabled = false;
                this.lblPos1.Enabled = true;
                this.txtPos1.Enabled = true;
                this.lblPos2.Enabled = true;
                this.txtPos2.Enabled = true;
                this.btnStop.Enabled = false;
                this.btnMoveN.Text = "Position1";
                this.btnMoveP.Text = "Position2";
            }
        }

        /***********************************************************************
        * OnResetPos() : Position "Reset" 버튼의 콜백함수. Command & Feedback 
        * position을 0으로 리셋한다.
        ***********************************************************************/
        private void btnResetPos_Click(object sender, EventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;

            CMDLL.cmmStSetPosition(nAxis, (int)Defines._TCmCntr.cmCNT_COMM, 0.0); // Reset COMMAND Position
            CMDLL.cmmStSetPosition(nAxis, (int)Defines._TCmCntr.cmCNT_FEED, 0.0); // Reset FEEDBACK Position
        }

        private void cmbSeqMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            int nSeqMode = this.cmbSeqMode.SelectedIndex;

            CMDLL.cmmCfgSetSeqMode(nSeqMode); // Sequence Mode 는 전 축에 적용 됩니다. 개별 축에 대해서 별도로 설정할 수 없습니다. 0: cmSEQM_SKIP_RUN, 1: cmSEQM_WAIT_RUN
        }

        private void btnMoveP_Click(object sender, EventArgs e)
        {
            g_fDistance = Convert.ToDouble(this.txtDist.Text);
            m_nMoveDir = (int)Defines._TCmDir.cmDIR_P;
            if (g_uRunCount == 0)
            {
                // 쓰레드 동작을 시작한다.
                new Thread(new ThreadStart(ThFirstRun)).Start();
            }
            else
            {
                // 쓰레드 동작을 시작한다.
                new Thread(new ThreadStart(ThSecondRun)).Start();
            }
            g_uRunCount++;
        }

        private void btnMoveN_Click(object sender, EventArgs e)
        {
            g_fDistance = Convert.ToDouble(this.txtDist.Text);
            m_nMoveDir = (int)Defines._TCmDir.cmDIR_N;
            if (g_uRunCount == 0)
            {
                // 쓰레드 동작을 시작한다.
                new Thread(new ThreadStart(ThFirstRun)).Start();
            }
            else
            {
                // 쓰레드 동작을 시작한다.
                new Thread(new ThreadStart(ThSecondRun)).Start();
            }
            g_uRunCount++;
        }

        /***********************************************************************
        * OnStop() : "Stop" Button's event handler
        * : This button stops motion of specifed axis
        ***********************************************************************/
        private void btnStop_Click(object sender, EventArgs e)
        {
            g_uRunCount = 0; // 다음 시퀀스 모드 테스트를 위해 이송 수행 카운트를 리셋 시킵니다.

            int nAxis = this.cmbAxis.SelectedIndex;
            CMDLL.cmmSxStop(nAxis, (int)Defines._TCmBool.cmTRUE, (int)Defines._TCmBool.cmFALSE);

            this.lblStatus.Text = "정지됨";
        }

        void ThFirstRun()
        {
            int nRunResult = Defines.cmERR_NONE;

            // "Move (-)"버튼이면 Distance에 음수를 취하고, "Move (+)" 버튼이면 양수를 취한다 //
            g_fDistance = (m_nMoveDir == (int)Defines._TCmDir.cmDIR_N) ? -g_fDistance : g_fDistance;

            if (g_nMoveMode == 0) // 상대좌표 이송 모드일때
            {
                nRunResult = CMDLL.cmmSxMoveStart(g_nAxis, g_fDistance);
            }
            else // 절대좌표 이송 모드일때
            {
                nRunResult = CMDLL.cmmSxMoveToStart(g_nAxis, g_fDistance);
            }

            if(nRunResult != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle);
            }
            else
            {
                g_sMsg = "새로운 이송 명령을 내려 보세요.";
                CMDLL.cmmSxWaitDone(g_nAxis, (int)Defines._TCmBool.cmTRUE);
            }
        }

        void ThSecondRun()
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            int nMoveMode = this.cmbMoveMode.SelectedIndex;
            int nSeqMode = this.cmbSeqMode.SelectedIndex;
            double fDistance = Convert.ToDouble(this.txtDist.Text);
            int nRunResult = Defines.cmERR_NONE;

            // "Move (-)"버튼이면 Distance에 음수를 취하고, "Move (+)" 버튼이면 양수를 취한다 //
            fDistance = (m_nMoveDir == (int)Defines._TCmDir.cmDIR_N) ? -fDistance : fDistance;

            if (nMoveMode == 0) // 상대좌표 이송 모드일때
            {
                nRunResult = CMDLL.cmmSxMoveStart(nAxis, fDistance);
            }
            else // 절대좌표 이송 모드일때
            {
                nRunResult = CMDLL.cmmSxMoveToStart(nAxis, fDistance);
            }

            if(nRunResult != Defines.cmERR_NONE)
            {
                if (nRunResult == Defines.cmERR_MOT_SEQ_SKIPPED)
                {
                    if (nSeqMode == (int)Defines._TCmLmxSeqMode.cmLMX_SEQM_SKIP_RUN)
                    {
                        this.lblStatus.Text = "시퀀스 모드가 cmSEQM_SKIP_RUN 으로 설정되어 새로운 이송 명령이 SKIP 됨";
                    }
                }
                else
                {
                    CMDLL.cmmErrShowLast(this.Handle);
                }
            }
            else
            {
                if (nSeqMode == (int)Defines._TCmLmxSeqMode.cmLMX_SEQM_WAIT_RUN)
                {
                    this.lblStatus.Text = "시퀀스 모드가 cmSEQM_WAIT_RUN 으로 설정되어 새로운 이송 명령 대기중임";
                }
                CMDLL.cmmSxWaitDone(nAxis, (int)Defines._TCmBool.cmTRUE);
            }
        }

        private void UsingSeqMode_FormClosing(object sender, FormClosingEventArgs e)
        {
            int nAxis = this.cmbAxis.SelectedIndex;
            CMDLL.cmmSxStop(nAxis, (int)Defines._TCmBool.cmTRUE, (int)Defines._TCmBool.cmFALSE);

            // 사용중인 타이머 종료.
            timer1.Stop();

            CMDLL.cmmGnDeviceUnload(); // Unload Device
        }

        private void cmbAxis_SelectedIndexChanged(object sender, EventArgs e)
        {
            g_nAxis = this.cmbAxis.SelectedIndex;
        }
    }
}
